@@ -5,14 +5,14 @@
 extern "C" {
 #endif
 
-#define LIBAVFORMAT_BUILD       4610
+#define LIBAVFORMAT_BUILD       4611
 
 #define LIBAVFORMAT_VERSION_INT FFMPEG_VERSION_INT
 #define LIBAVFORMAT_VERSION     FFMPEG_VERSION
 #define LIBAVFORMAT_IDENT	"FFmpeg" FFMPEG_VERSION "b" AV_STRINGIFY(LIBAVFORMAT_BUILD)
 
 #include <time.h>
-
+#include <stdio.h>  /* FILE */
 #include "avcodec.h"
 
 #include "avio.h"
@@ -197,7 +197,10 @@ typedef struct AVIndexEntry {
     int64_t pos;
     int64_t timestamp;
 #define AVINDEX_KEYFRAME 0x0001
+/* the following 2 flags indicate that the next/prev keyframe is known, and scaning for it isnt needed */
     int flags;
+    int min_distance;         /* min distance between this and the previous key
+frame, used to avoid unneeded searching */
 } AVIndexEntry;
 
 typedef struct AVStream {
@@ -227,9 +230,7 @@ typedef struct AVStream {
     /* av_read_frame() support */
     int need_parsing;
     struct AVCodecParserContext *parser;
-    int got_frame;
-    int64_t cur_frame_pts;
-    int64_t cur_frame_dts;
+
     int64_t cur_dts;
     int last_IP_duration;
     /* av_seek_frame() support */
@@ -238,6 +239,7 @@ typedef struct AVStream {
     int nb_index_entries;
     int index_entries_allocated_size;
 
+    int got_frame;
     int64_t cur_frame_startpos;
 } AVStream;
 
@@ -483,6 +485,12 @@ int film_init(void);
 /* idcin.c */
 int idcin_init(void);
 
+/* flic.c */
+int flic_init(void);
+
+/* sierravmd.c */
+int vmd_init(void);
+
 //#include "rtp.h"
 
 //#include "rtsp.h"
@@ -534,6 +542,7 @@ int av_open_input_file(AVFormatContext **ic_ptr, const char *filename,
 #define AVERROR_NOTSUPP     (-7)  /* operation not supported */
  
 int av_find_stream_info(AVFormatContext *ic);
+void av_estimate_timings(AVFormatContext *ic);
 int av_read_packet(AVFormatContext *s, AVPacket *pkt);
 int av_read_frame(AVFormatContext *s, AVPacket *pkt);
 void av_read_frame_flush(AVFormatContext *s);
@@ -545,6 +554,12 @@ AVStream *av_new_stream(AVFormatContext *s, int id);
 void av_set_pts_info(AVFormatContext *s, int pts_wrap_bits,
                      int pts_num, int pts_den);
 
+int av_find_default_stream_index(AVFormatContext *s);
+int av_index_search_timestamp(AVStream *st, int timestamp);
+int av_add_index_entry(AVStream *st,
+                       int64_t pos, int64_t timestamp, int distance, int flags);
+
+
 /* media file output */
 int av_set_parameters(AVFormatContext *s, AVFormatParameters *ap);
 int av_write_header(AVFormatContext *s);
