@@ -18,6 +18,9 @@
  */
 #include "avformat.h"
 
+#undef NDEBUG
+#include <assert.h>
+
 AVInputFormat *first_iformat;
 AVOutputFormat *first_oformat;
 AVImageFormat *first_image_format;
@@ -167,12 +170,14 @@ int av_dup_packet(AVPacket *pkt)
 {
     if (pkt->destruct != av_destruct_packet) {
         uint8_t *data;
-        /* we duplicate the packet */
-        data = av_malloc(pkt->size);
+        /* we duplicate the packet and don't forget to put the padding
+           again */
+        data = av_malloc(pkt->size + FF_INPUT_BUFFER_PADDING_SIZE);
         if (!data) {
             return AVERROR_NOMEM;
         }
         memcpy(data, pkt->data, pkt->size);
+        memset(data + pkt->size, 0, FF_INPUT_BUFFER_PADDING_SIZE);
         pkt->data = data;
         pkt->destruct = av_destruct_packet;
     }
@@ -663,7 +668,7 @@ static void av_destruct_packet_nofree(AVPacket *pkt)
 static int av_read_frame_internal(AVFormatContext *s, AVPacket *pkt)
 {
     AVStream *st;
-    int len, ret;
+    int len, ret, i;
     int64_t startpos;
 
     for(;;) {
@@ -678,29 +683,26 @@ static int av_read_frame_internal(AVFormatContext *s, AVPacket *pkt)
                 s->cur_st = NULL;
                 return 0;
             } else if (s->cur_len > 0) {
-                /* we use the MPEG semantics: the pts and dts in a
-                   packet are given from the first frame beginning in
-                   it */
                 if (!st->got_frame) {
-                    st->cur_frame_pts = s->cur_pkt.pts;
-                    st->cur_frame_dts = s->cur_pkt.dts;
                     st->cur_frame_startpos = s->cur_pkt.startpos;
-                    s->cur_pkt.pts = AV_NOPTS_VALUE;
-                    s->cur_pkt.dts = AV_NOPTS_VALUE;
                     st->got_frame = 1;
                 }
                 len = av_parser_parse(st->parser, &st->codec, &pkt->data, &pkt->size, 
-                                      s->cur_ptr, s->cur_len);
+                                      s->cur_ptr, s->cur_len,
+                                      s->cur_pkt.pts, s->cur_pkt.dts);
+                s->cur_pkt.pts = AV_NOPTS_VALUE;
+                s->cur_pkt.dts = AV_NOPTS_VALUE;
                 /* increment read pointer */
                 s->cur_ptr += len;
                 s->cur_len -= len;
                 
                 /* return packet if any */
                 if (pkt->size) {
+                got_packet:
                     pkt->duration = 0;
                     pkt->stream_index = st->index;
-                    pkt->pts = st->cur_frame_pts;
-                    pkt->dts = st->cur_frame_dts;
+                    pkt->pts = st->parser->pts;
+                    pkt->dts = st->parser->dts;
                     pkt->destruct = av_destruct_packet_nofree;
                     pkt->startpos = st->cur_frame_startpos;
                     compute_pkt_fields(s, st, st->parser, pkt);
@@ -708,22 +710,34 @@ static int av_read_frame_internal(AVFormatContext *s, AVPacket *pkt)
                     return 0;
                 }
             } else {
-                if (s->cur_st && s->cur_st->parser)
-                    av_free_packet(&s->cur_pkt);
+                /* free packet */
+                av_free_packet(&s->cur_pkt); 
                 s->cur_st = NULL;
             }
         } else {
-            /* free previous packet */
-            if (s->cur_st && s->cur_st->parser)
-                av_free_packet(&s->cur_pkt); 
-
             startpos = url_ftell(&s->pb);
             s->cur_pkt.startpos = 0;
 
             /* read next packet */
             ret = av_read_packet(s, &s->cur_pkt);
-            if (ret < 0)
+            if (ret < 0) {
+                if (ret == -EAGAIN)
+                    return ret;
+                /* return the last frames, if any */
+                for(i = 0; i < s->nb_streams; i++) {
+                    st = s->streams[i];
+                    if (st->parser) {
+                        av_parser_parse(st->parser, &st->codec, 
+                                        &pkt->data, &pkt->size, 
+                                        NULL, 0, 
+                                        AV_NOPTS_VALUE, AV_NOPTS_VALUE);
+                        if (pkt->size)
+                            goto got_packet;
+                    }
+                }
+                /* no more packets: really terminates parsing */
                 return ret;
+            }
 
             if (s->cur_pkt.startpos == 0)
                 s->cur_pkt.startpos = startpos;
@@ -738,7 +752,7 @@ static int av_read_frame_internal(AVFormatContext *s, AVPacket *pkt)
                                                &s->last_pkt_stream_dts,
                                                s->cur_pkt.dts);
 #if 0
-            if (s->cur_pkt.stream_index == 1) {
+            if (s->cur_pkt.stream_index == 0) {
                 if (s->cur_pkt.pts != AV_NOPTS_VALUE) 
                     printf("PACKET pts=%0.3f\n", 
                            (double)s->cur_pkt.pts / AV_TIME_BASE);
@@ -820,6 +834,22 @@ static void flush_packet_queue(AVFormatContext *s)
 /*******************************************************/
 /* seek support */
 
+int av_find_default_stream_index(AVFormatContext *s)
+{
+    int i;
+    AVStream *st;
+
+    if (s->nb_streams <= 0)
+        return -1;
+    for(i = 0; i < s->nb_streams; i++) {
+        st = s->streams[i];
+        if (st->codec.codec_type == CODEC_TYPE_VIDEO) {
+            return i;
+        }
+    }
+    return 0;
+}
+
 /* flush the frame reader */
 void av_read_frame_flush(AVFormatContext *s)
 {
@@ -851,22 +881,47 @@ void av_read_frame_flush(AVFormatContext *s)
     }
 }
 
-static void add_index_entry(AVStream *st,
-                            int64_t pos, int64_t timestamp, int flags)
+/* add a index entry into a sorted list updateing if it is already there */
+int av_add_index_entry(AVStream *st,
+                            int64_t pos, int64_t timestamp, int distance, int flags)
 {
     AVIndexEntry *entries, *ie;
+    int index;
     
     entries = av_fast_realloc(st->index_entries,
                               &st->index_entries_allocated_size,
                               (st->nb_index_entries + 1) * 
                               sizeof(AVIndexEntry));
-    if (entries) {
-        st->index_entries = entries;
-        ie = &entries[st->nb_index_entries++];
-        ie->pos = pos;
-        ie->timestamp = timestamp;
-        ie->flags = flags;
+    st->index_entries= entries;
+
+    if(st->nb_index_entries){
+        index= av_index_search_timestamp(st, timestamp);
+        ie= &entries[index];
+
+        if(ie->timestamp != timestamp){
+            if(ie->timestamp < timestamp){
+                index++; //index points to next instead of previous entry, maybe nonexistant
+                ie= &st->index_entries[index];
+            }else
+                assert(index==0);
+                
+            if(index != st->nb_index_entries){
+                assert(index < st->nb_index_entries);
+                memmove(entries + index + 1, entries + index, sizeof(AVIndexEntry)*(st->nb_index_entries - index));
+            }
+            st->nb_index_entries++;
+        }
+    }else{
+        index= st->nb_index_entries++;
+        ie= &entries[index];
     }
+    
+    ie->pos = pos;
+    ie->timestamp = timestamp;
+    ie->min_distance= distance;
+    ie->flags = flags;
+
+    return index;
 }
 
 /* build an index for raw streams using a parser */
@@ -886,8 +941,8 @@ static void av_build_index_raw(AVFormatContext *s)
             break;
         if (pkt->stream_index == 0 && st->parser &&
             (pkt->flags & PKT_FLAG_KEY)) {
-            add_index_entry(st, st->parser->frame_offset, pkt->dts, 
-                            AVINDEX_KEYFRAME);
+            av_add_index_entry(st, st->parser->frame_offset, pkt->dts, 
+                            0, AVINDEX_KEYFRAME);
         }
         av_free_packet(pkt);
     }
@@ -909,9 +964,10 @@ static int is_raw_stream(AVFormatContext *s)
 
 /* return the largest index entry whose timestamp is <=
    wanted_timestamp */
-static int index_search_timestamp(AVIndexEntry *entries, 
-                                  int nb_entries, int wanted_timestamp)
+int av_index_search_timestamp(AVStream *st, int wanted_timestamp)
 {
+    AVIndexEntry *entries= st->index_entries;
+    int nb_entries= st->nb_index_entries;
     int a, b, m;
     int64_t timestamp;
 
@@ -920,22 +976,17 @@ static int index_search_timestamp(AVIndexEntry *entries,
     
     a = 0;
     b = nb_entries - 1;
-    while (a <= b) {
-        m = (a + b) >> 1;
+
+    while (a < b) {
+        m = (a + b + 1) >> 1;
         timestamp = entries[m].timestamp;
-        if (timestamp == wanted_timestamp)
-            goto found;
-        else if (timestamp > wanted_timestamp) {
+        if (timestamp > wanted_timestamp) {
             b = m - 1;
         } else {
-            a = m + 1;
+            a = m;
         }
     }
-    m = a;
-    if (m > 0)
-        m--;
- found:
-    return m;
+    return a;
 }
 
 static int av_seek_frame_generic(AVFormatContext *s, 
@@ -957,8 +1008,7 @@ static int av_seek_frame_generic(AVFormatContext *s,
     if (stream_index < 0)
         stream_index = 0;
     st = s->streams[stream_index];
-    index = index_search_timestamp(st->index_entries, st->nb_index_entries,
-                                   timestamp);
+    index = av_index_search_timestamp(st, timestamp);
     if (index < 0)
         return -1;
 
@@ -1228,7 +1278,7 @@ static void av_estimate_timings_from_pts(AVFormatContext *ic)
     url_fseek(&ic->pb, 0, SEEK_SET);
 }
 
-static void av_estimate_timings(AVFormatContext *ic)
+void av_estimate_timings(AVFormatContext *ic)
 {
     URLContext *h;
     int64_t file_size;
@@ -2070,8 +2120,12 @@ void av_pkt_dump(FILE *f, AVPacket *pkt, int dump_payload)
     fprintf(f, "stream #%d:\n", pkt->stream_index);
     fprintf(f, "  keyframe=%d\n", ((pkt->flags & PKT_FLAG_KEY) != 0));
     fprintf(f, "  duration=%0.3f\n", (double)pkt->duration / AV_TIME_BASE);
-    /* DTS is _always_ valid */
-    fprintf(f, "  dts=%0.3f\n", (double)pkt->dts / AV_TIME_BASE);
+    /* DTS is _always_ valid after av_read_frame() */
+    fprintf(f, "  dts=");
+    if (pkt->dts == AV_NOPTS_VALUE)
+        fprintf(f, "N/A");
+    else
+        fprintf(f, "%0.3f", (double)pkt->dts / AV_TIME_BASE);
     /* PTS may be not known if B frames are present */
     fprintf(f, "  pts=");
     if (pkt->pts == AV_NOPTS_VALUE)
