@@ -1,13 +1,13 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
 #include <fcntl.h>
 #include <sys/time.h>
 #include <unistd.h>
 #include <time.h>
 #include <sys/soundcard.h>
 #include <sys/ioctl.h>
-#include <errno.h>
+#include <cerrno>
+#include <cstring>
 
 #include <iostream>
 #include <qdatetime.h>
@@ -75,13 +75,13 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
     numbadioctls = 0;
     numlowbuffer = 0;
 
-    QDateTime curtime = QDateTime::currentDateTime();
-    curtime = curtime.addSecs(2);
-    
+    QTime timer;
+    timer.start();
+
     VERBOSE(VB_GENERAL, QString("Opening OSS audio device '%1'.")
             .arg(audiodevice));
     
-    while (QDateTime::currentDateTime() < curtime && audiofd == -1)
+    while (timer.elapsed() < 2000 && audiofd == -1)
     {
         audiofd = open(audiodevice.ascii(), O_WRONLY | O_NONBLOCK);
         if (audiofd < 0 && errno != EAGAIN && errno != EINTR)
@@ -90,7 +90,7 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
             {
                 cerr << "ERROR: something is currently using: " << audiodevice
                      << "\nFix this, then run mythfrontend again\n";
-                exit(0);
+                exit(1);
             }
             cerr << "Error opening audio device (" << audiodevice << "):\n";
             perror(audiodevice.ascii());
@@ -152,11 +152,15 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
 
     int caps;
     
-    if (ioctl(audiofd, SNDCTL_DSP_GETCAPS, &caps) >= 0 && 
-        !(caps & DSP_CAP_REALTIME))
+    if (ioctl(audiofd, SNDCTL_DSP_GETCAPS, &caps) == 0)
     {
-        cerr << "audio device cannot report buffer state accurately,\n"
-             << "audio/video sync will be bad, continuing anyway\n";
+        if (!(caps & DSP_CAP_REALTIME))
+        {
+            cerr << "audio device cannot report buffer state accurately!\n"
+                 << "audio/video sync will be bad, continuing anyway...\n";
+        }
+    } else {
+        perror("ioctl(SNDCTL_DSP_GETCAPS)");
     }
 
     audbuf_timecode = 0;
