@@ -26,6 +26,7 @@ class QSqlError;
 class QSocket;
 class LCD;
 class MythMainWindow;
+class MythPluginManager;
 
 enum VerboseMask {
     VB_IMPORTANT = 0x0001,
@@ -42,9 +43,11 @@ enum VerboseMask {
 };
 
 #define VERBOSE(mask,args...) \
+do { \
 if ((print_verbose_messages & mask) != 0) \
     cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss") \
-         << " " << args << endl;
+         << " " << args << endl; \
+} while (0)
 
 class MythEvent : public QCustomEvent
 {
@@ -73,10 +76,13 @@ class MythEvent : public QCustomEvent
     QString extradata;
 };
 
-#define MYTH_BINARY_VERSION "0.13.12102003-1"
+#define MYTH_BINARY_VERSION "0.14.20040123-1"
+#define MYTH_PROTO_VERSION "1"
 
 extern int print_verbose_messages;
 
+class MythContextPrivate;
+
 class MythContext : public QObject
 {
     Q_OBJECT
@@ -86,16 +92,16 @@ class MythContext : public QObject
 
     QString GetMasterHostPrefix(void);
 
-    QString GetHostName(void) { return m_localhostname; }
+    QString GetHostName(void);
 
     bool ConnectToMasterServer(void);
     bool ConnectServer(const QString &hostname, int port);
-    bool IsConnectedToMaster(void) { return serverSock; }
+    bool IsConnectedToMaster(void);
 
-    QString GetInstallPrefix() { return m_installprefix; }
-    QString GetShareDir() { return m_installprefix + "/share/mythtv/"; }
+    QString GetInstallPrefix(void);
+    QString GetShareDir(void);
 
-    QString GetFilePrefix();
+    QString GetFilePrefix(void);
 
     bool LoadSettingsFiles(const QString &filename);
     void LoadQtConfig(void);
@@ -107,19 +113,19 @@ class MythContext : public QObject
     void GetScreenSettings(int &xbase, int &width, float &wmult,
                            int &ybase, int &height, float &hmult);
    
-    QString FindThemeDir(QString themename);
-    QString GetThemeDir(void) { return m_themepathname; }
+    QString FindThemeDir(const QString &themename);
+    QString GetThemeDir(void);
 
     int OpenDatabase(QSqlDatabase *db);
     static void KickDatabase(QSqlDatabase *db);
-    static void DBError(QString where, const QSqlQuery& query);
+    static void DBError(const QString &where, const QSqlQuery& query);
     static QString DBErrorMessage(const QSqlError& err);
 
-    Settings *settings() { return m_settings; }
-    Settings *qtconfig() { return m_qtThemeSettings; }
+    Settings *settings(void);
+    Settings *qtconfig(void);
 
-    void SaveSetting(QString key, int newValue);
-    void SaveSetting(QString key, QString newValue);
+    void SaveSetting(const QString &key, int newValue);
+    void SaveSetting(const QString &key, const QString &newValue);
     QString GetSetting(const QString &key, const QString &defaultval = "");
     int GetNumSetting(const QString &key, int defaultval = 0);
 
@@ -130,10 +136,6 @@ class MythContext : public QObject
 
     void SetSetting(const QString &key, const QString &newValue);
 
-    int GetBigFontSize() { return bigfontsize; }
-    int GetMediumFontSize() { return mediumfontsize; }
-    int GetSmallFontSize() { return smallfontsize; }
-
     QFont GetBigFont();
     QFont GetMediumFont();
     QFont GetSmallFont();
@@ -157,12 +159,18 @@ class MythContext : public QObject
     void SetMainWindow(MythMainWindow *mainwin);
     MythMainWindow *GetMainWindow(void);
 
-    LCD *GetLCDDevice(void) { return lcd_device; }
+    LCD *GetLCDDevice(void);
 
     bool TestPopupVersion(const QString &name, const QString &libversion,
                           const QString &pluginversion);
 
-    void SetDisableLibraryPopup(bool check) { disablelibrarypopup = check; }
+    void SetDisableLibraryPopup(bool check);
+
+    void SetPluginManager(MythPluginManager *pmanager);
+    MythPluginManager *getPluginManager(void);
+
+    bool CheckProtoVersion(QSocketDevice* socket);
+    bool CheckProtoVersion(QSocket* socket);
     
   private slots:
     void EventSocketRead();
@@ -175,50 +183,11 @@ class MythContext : public QObject
 
     void ClearOldImageCache(void);
     void CacheThemeImages(void);
-    void CacheThemeImagesDirectory(const QString &dir);
-    void RemoveCacheDir(const QString &dir);
+    void CacheThemeImagesDirectory(const QString &dirname, 
+                                   const QString &subdirname = "");
+    void RemoveCacheDir(const QString &dirname);
 
-    Settings *m_settings;
-    Settings *m_qtThemeSettings;
-
-    QString m_installprefix;
-
-    bool m_themeloaded;
-    QString m_themepathname;
-    QPixmap *m_backgroundimage;
-    QPalette m_palette;
-
-    int m_xbase, m_ybase;
-    int m_height, m_width;
-
-    QString m_localhostname;
-
-    QMutex serverSockLock;
-
-    QPtrList<QObject> listeners;
-
-    QSqlDatabase* m_db;
-    QMutex dbLock;
-
-    QMap<QString, QImage> imageCache;
-
-    LCD *lcd_device;
-
-    QString language;
-
-    MythMainWindow *mainWindow;
-
-    float m_wmult, m_hmult;
-    int m_screenwidth, m_screenheight;
-
-    QString themecachedir;
-
-    int bigfontsize, mediumfontsize, smallfontsize; 
-
-    QSocketDevice *serverSock;
-    QSocket *eventSock;
-
-    bool disablelibrarypopup;
+    MythContextPrivate *d;
 };
 
 extern MythContext *gContext;
