@@ -25,6 +25,7 @@ using namespace std;
 #include "xmlparse.h"
 #include "mythdialogs.h"
 #include "lcddevice.h"
+#include "mythmediamonitor.h"
 
 #ifdef USE_LIRC
 static void *SpawnLirc(void *param)
@@ -68,9 +69,24 @@ struct JumpData
     QString description;
 };
 
+struct MHData
+{
+    void (*callback)(void);
+    int MediaType;
+    QString destination;
+    QString description;
+};
+
+struct MPData {
+    QString description;
+    MediaPlayCallback playFn;
+};
+
 class MythMainWindowPrivate
 {
   public:
+    int TranslateKeyNum(QKeyEvent *e);
+
     float wmult, hmult;
     int screenwidth, screenheight;
     int xbase, ybase;
@@ -83,12 +99,38 @@ class MythMainWindowPrivate
 
     QDict<KeyContext> keyContexts;
     QMap<int, JumpData> jumpMap;
+    QMap<QString, MHData> mediaHandlerMap;
+    QMap<QString, MPData> mediaPluginMap;
 
     void (*exitmenucallback)(void);
 
     int escapekey;
 };
 
+// Make keynum in QKeyEvent be equivalent to what's in QKeySequence
+int MythMainWindowPrivate::TranslateKeyNum(QKeyEvent* e)
+{
+    int keynum = e->key();
+
+    if (keynum != Qt::Key_Escape &&
+        (keynum <  Qt::Key_Shift || keynum > Qt::Key_ScrollLock))
+    {
+        Qt::ButtonState modifiers;
+        // if modifiers have been pressed, rebuild keynum
+        if ((modifiers = e->state()) != 0)
+        {
+            int modnum = ((modifiers & Qt::ShiftButton) ? Qt::SHIFT : 0) |
+                         ((modifiers & Qt::ControlButton) ? Qt::CTRL : 0) |
+                         ((modifiers & Qt::MetaButton) ? Qt::META : 0) |
+                         ((modifiers & Qt::AltButton) ? Qt::ALT : 0);
+            modnum &= ~Qt::UNICODE_ACCEL;
+            return (keynum |= modnum);
+        }
+    }
+
+    return keynum;
+}
+
 MythMainWindow::MythMainWindow(QWidget *parent, const char *name, bool modal)
               : QDialog(parent, name, modal)
 {
@@ -123,6 +165,9 @@ MythMainWindow::MythMainWindow(QWidget *parent, const char *name, bool modal)
     RegisterKey("Global", "PAGEUP", "Page Up", "PgUp");
     RegisterKey("Global", "PAGEDOWN", "Page Down", "PgDown");
 
+    RegisterKey("Global", "PREVVIEW", "Previous View", "Home");
+    RegisterKey("Global", "NEXTVIEW", "Next View", "End");
+
     RegisterKey("Global", "0", "0", "0");
     RegisterKey("Global", "1", "1", "1");
     RegisterKey("Global", "2", "2", "2");
@@ -149,6 +194,7 @@ void MythMainWindow::Init(void)
 
     setFont(gContext->GetMediumFont());
     setCursor(QCursor(Qt::BlankCursor));
+    qApp->setOverrideCursor(QCursor(Qt::BlankCursor));
 
     gContext->ThemeWidget(this);
 
@@ -166,6 +212,9 @@ void MythMainWindow::Show(void)
 
 void MythMainWindow::attach(QWidget *child)
 {
+    if (currentWidget())
+        currentWidget()->setEnabled(false);
+
     d->widgetList.push_back(child);
     child->raise();
     child->setFocus();
@@ -183,7 +232,10 @@ void MythMainWindow::detach(QWidget *child)
     QWidget *current = currentWidget();
 
     if (current)
+    {
+        current->setEnabled(true);
         current->setFocus();
+    }
 
     if (d->exitingtomain)
         QApplication::postEvent(this, new ExitToMainMenuEvent());
@@ -238,7 +290,7 @@ bool MythMainWindow::TranslateKeyPress(const QString &context,
                                        QKeyEvent *e, QStringList &actions)
 {
     actions = QStringList();
-    int keynum = e->key();
+    int keynum = d->TranslateKeyNum(e);
 
     if (d->jumpMap.count(keynum) > 0 && d->exitmenucallback == NULL)
     {
@@ -403,10 +455,67 @@ void MythMainWindow::RegisterJump(const QString &destination,
     //                               .arg(destination));
 }
 
+void MythMainWindow::RegisterMediaHandler(const QString &destination,
+                                          const QString &description,
+                                          const QString &key, 
+                                          void (*callback)(void),
+                                          int mediaType)
+{
+    (void)key;
+
+    if (d->mediaHandlerMap.count(destination) == 0) 
+    {
+        MHData mhd = { callback, mediaType, destination, description };
+
+        VERBOSE(VB_GENERAL, QString("Registering %1 as a media handler")
+                                   .arg(destination));
+
+        d->mediaHandlerMap[destination] = mhd;
+    }
+    else 
+    {
+       VERBOSE(VB_GENERAL, QString("%1 is already registered as a media "
+                                   "handler.").arg(destination));
+    }
+}
+
+void MythMainWindow::RegisterMediaPlugin(const QString &name, 
+                                         const QString &desc, 
+                                         MediaPlayCallback fn)
+{
+    if (d->mediaPluginMap.count(name) == 0) 
+    {
+        VERBOSE(VB_GENERAL, QString("Registering %1 as a media playback "
+                                    "plugin.").arg(name));
+        MPData mpd = {desc, fn};
+        d->mediaPluginMap[name] = mpd;
+    } 
+    else
+    {
+        VERBOSE(VB_GENERAL, QString("%1 is already registered as a media "
+                                    "playback plugin.").arg(name));
+    }
+}
+
+bool MythMainWindow::HandleMedia(QString &handler, const QString &mrl)
+{
+    if (handler.length() < 1)
+        handler = "Internal";
+
+    // Let's see if we have a plugin that matches the handler name...
+    if (d->mediaPluginMap.count(handler)) 
+    {
+        d->mediaPluginMap[handler].playFn(mrl);
+        return true;
+    }
+
+    return false;
+}
+
 void MythMainWindow::keyPressEvent(QKeyEvent *e)
 {
     QWidget *current = currentWidget();
-    if (current)
+    if (current && current->isEnabled())
         qApp->notify(current, e);
     else
         QDialog::keyPressEvent(e);
@@ -429,6 +538,32 @@ void MythMainWindow::customEvent(QCustomEvent *ce)
 
         QApplication::sendEvent(key_target, &key);
     }
+    else if (ce->type() == kMediaEventType) 
+    {
+        MediaEvent *media_event = (MediaEvent*)ce;
+        // Let's see which of our jump points are configured to handle this 
+        // type of media...  If there's more than one we'll want to show some 
+        // UI to allow the user to select which jump point to use. But for 
+        // now we're going to just use the first one.
+        QMap<QString, MHData>::Iterator itr = d->mediaHandlerMap.begin();
+        MythMediaDevice *pDev = media_event->getDevice();
+
+        if (pDev) 
+        {
+            while (itr != d->mediaHandlerMap.end())
+            {
+                if ((itr.data().MediaType & (int)pDev->getMediaType()))
+                {
+                    cout << "Found a handler" << endl;
+                    d->exitingtomain = true;
+                    d->exitmenucallback = itr.data().callback;
+                    QApplication::postEvent(this, new ExitToMainMenuEvent());
+                    break;
+                }
+                itr++;
+            }
+        }
+    }
 #ifdef USE_LIRC
     else if (ce->type() == kLircKeycodeEventType && !d->ignore_lirc_keys) 
     {
@@ -537,11 +672,12 @@ MythDialog::~MythDialog()
 
 void MythDialog::setNoErase(void)
 {
-#ifndef QWS
     WFlags flags = getWFlags();
     flags |= WRepaintNoErase;
-    setWFlags(flags);
+#ifdef QWS
+    flags |= WResizeNoErase;
 #endif
+    setWFlags(flags);
 }
 
 void MythDialog::Show(void)
@@ -780,6 +916,11 @@ QButton *MythPopupBox::addButton(QString caption, QObject *target,
     return button;
 }
 
+void MythPopupBox::addLayout(QLayout *layout, int stretch)
+{
+    vbox->addLayout(layout, stretch);
+}
+
 void MythPopupBox::ShowPopup(QObject *target, const char *slot)
 {
     ShowPopupAtXY(-1, -1, target, slot);
@@ -812,10 +953,18 @@ void MythPopupBox::ShowPopupAtXY(int destx, int desty,
         ++it;
         if (objs->isWidgetType())
         {
-            QWidget *widget = (QWidget *)objs;
-            poph += widget->height();
-            if (widget->width() > maxw)
-                maxw = widget->width();
+            QString objname = objs->name();
+            if (objname != "nopopsize")
+            {
+                // little extra padding for these guys
+                if (objs->isA("MythListBox"))
+                    poph += (int)(25 * hmult);
+
+                QWidget *widget = (QWidget *)objs;
+                poph += widget->height();
+                if (widget->width() > maxw)
+                    maxw = widget->width();
+            }
         }
     }
 
@@ -897,9 +1046,12 @@ void MythPopupBox::defaultButtonPressedHandler(void)
         if (objs->isWidgetType())
         {
             QWidget *widget = (QWidget *)objs;
-            if (widget->hasFocus())
-                break;
-            i++;
+            if (widget->isA("MythPushButton"))
+            {
+                if (widget->hasFocus())
+                    break;
+                i++;
+            }
         }
     }
     done(i);
@@ -932,7 +1084,27 @@ bool MythPopupBox::showOkCancelPopup(MythMainWindow *parent, QString title,
     else
         cancelButton->setFocus();
 
-    return (popup.ExecPopup() == 1);
+    return (popup.ExecPopup() == 0);
+}
+
+int MythPopupBox::show2ButtonPopup(MythMainWindow *parent, QString title,
+                                   QString message, QString button1msg,
+                                   QString button2msg, int defvalue)
+{
+    MythPopupBox popup(parent, title);
+
+    popup.addLabel(message, Medium, true);
+    popup.addLabel("");
+
+    QButton *but1 = popup.addButton(button1msg);
+    QButton *but2 = popup.addButton(button2msg);
+
+    if (defvalue == 1)
+        but1->setFocus();
+    else
+        but2->setFocus();
+
+    return popup.ExecPopup();
 }
 
 MythProgressDialog::MythProgressDialog(const QString &message, int totalSteps)
@@ -1728,24 +1900,27 @@ MythPasswordDialog::MythPasswordDialog(QString message,
                                        bool)
                    :MythDialog(parent, name, false)
 {
+    int textWidth =  fontMetrics().width(message);
+    int totalWidth = textWidth + 175;
+
     success_flag = success;
     target_text = target;
 
     gContext->GetScreenSettings(screenwidth, wmult, screenheight, hmult);
     this->setGeometry((screenwidth - 250 ) / 2,
                       (screenheight - 50 ) / 2,
-                      300,50);
+                      totalWidth,50);
     QFrame *outside_border = new QFrame(this);
-    outside_border->setGeometry(0,0,300,50);
+    outside_border->setGeometry(0,0,totalWidth,50);
     outside_border->setFrameStyle(QFrame::Panel | QFrame::Raised );
     outside_border->setLineWidth(4);
     QLabel *message_label = new QLabel(message, this);
     message_label->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
-    message_label->setGeometry(15,10,130,30);
+    message_label->setGeometry(15,10,textWidth,30);
     message_label->setBackgroundOrigin(ParentOrigin);
     password_editor = new MythLineEdit(this);
     password_editor->setEchoMode(QLineEdit::Password);
-    password_editor->setGeometry(150,10,135,30);
+    password_editor->setGeometry(textWidth + 20,10,135,30);
     password_editor->setBackgroundOrigin(ParentOrigin);
     connect(password_editor, SIGNAL(textChanged(const QString &)),
             this, SLOT(checkPassword(const QString &)));
@@ -1794,118 +1969,6 @@ MythPasswordDialog::~MythPasswordDialog()
 ---------------------------------------------------------------------
  */
 
-Myth2ButtonDialog::Myth2ButtonDialog(MythMainWindow *parent, QString title1,
-                                     QString message, QString button1msg,
-                                     QString button2msg, int defvalue)
-                 : MythDialog(parent, title1, false)
-{
-    gContext->GetScreenSettings(screenwidth, wmult, screenheight, hmult);
-    QFont bigFont = gContext->GetBigFont();
-    QFont medFont = gContext->GetMediumFont();
-
-    popup = new MythPopupBox(gContext->GetMainWindow(), "2 button dialog");
-    popup->setFrameStyle( QFrame::Box | QFrame::Plain );
-    gContext->ThemeWidget(popup);
-    QLabel *msg = new QLabel(tr(message), popup);
-    msg->setBackgroundOrigin(ParentOrigin);
-    QLabel *filler1 = new QLabel("", popup);
-    filler1->setBackgroundOrigin(ParentOrigin);
-
-    QLabel *title = new QLabel(title1, popup);
-    title->setBackgroundOrigin(ParentOrigin);
-    title->setFont(bigFont);
-    title->setMaximumWidth((int)(width() / 2));
-
-    QLabel *filler2 = new QLabel("", popup);
-    filler2->setBackgroundOrigin(ParentOrigin);
-
-    but1 = new MythPushButton(tr(button1msg), popup);
-    but2 = new MythPushButton(tr(button2msg), popup);
-
-    popup->addWidget(msg, false);
-    popup->addWidget(filler1, false);
-    popup->addWidget(title, false);
-    popup->addWidget(filler2, false);
-    popup->addWidget(but1, false);
-    popup->addWidget(but2, false);
-
-    if (defvalue == 1)
-        but1->setFocus();
-    else
-        but2->setFocus();
-
-    msg->adjustSize();
-    filler1->adjustSize();
-    filler2->adjustSize();
-    title->adjustSize();
-    but1->adjustSize();
-    but2->adjustSize();
-
-    popup->polish();
-
-    int x, y, maxw, poph;
-    poph = msg->height() + filler1->height() +
-           title->height() + but1->height() + but2->height() +
-           (int)(110 * hmult);
-    popup->setMinimumHeight(poph);
-    maxw = 0;
-
-    if (title->width() > maxw)
-        maxw = title->width();
-    if (but1->width() > maxw)
-        maxw = but1->width();
-    if (but2->width() > maxw)
-        maxw = but2->width();
-
-    maxw += (int)(80 * wmult);
-
-    x = (int)(screenwidth / 2) - (int)(maxw / 2);
-    y = (int)(screenheight / 2) - (int)(poph / 2);
-
-    popup->setFixedSize(maxw, poph);
-    popup->setGeometry(x, y, maxw, poph);
-//    popup->setFocusPolicy(StrongFocus);
-    popup->Show();
-    grabKeyboard();
-}
-
-void Myth2ButtonDialog::keyPressEvent(QKeyEvent *e)
-{
-    bool handled = false;
-    QStringList actions;
-    if (gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions))
-    {
-        for (unsigned int i = 0; i < actions.size() && !handled; i++)
-        {
-            QString action = actions[i];
-            handled = true;
-
-            if (action == "ESCAPE")
-                done(0);
-            else if (action == "SELECT")
-            {
-                if (but1->hasFocus())
-                    done(1);
-                else
-                    done(2);
-            }
-            else if (action == "UP" || action == "DOWN")
-            {
-                if (but1->hasFocus())
-                    but2->setFocus();
-                else
-                    but1->setFocus();
-            }
-            else
-                handled = false;
-        }
-    }
-}
-
-/*
----------------------------------------------------------------------
-*/
-
 MythImageFileDialog::MythImageFileDialog(QString *result,
                                          QString top_directory,
                                          MythMainWindow *parent, 
