@@ -1,4 +1,3 @@
-#include <qapplication.h>
 #include <qsocket.h>
 #include <qurl.h>
 
@@ -11,7 +10,7 @@ using namespace std;
 #include "util.h"
 #include "mythcontext.h"
 
-RemoteFile::RemoteFile(const QString &url, bool needevents, int lrecordernum)
+RemoteFile::RemoteFile(const QString &url, int lrecordernum)
 {
     type = 0;
     path = url;
@@ -33,16 +32,8 @@ RemoteFile::RemoteFile(const QString &url, bool needevents, int lrecordernum)
         recordernum = -1;
     }
 
-    if (needevents)
-    {
-        controlSock = NULL;
-        sock = NULL;
-    }
-    else
-    {
-        controlSock = openSocket(true);
-        sock = openSocket(false);
-    }
+    controlSock = openSocket(true);
+    sock = openSocket(false);
 }
 
 RemoteFile::~RemoteFile()
@@ -54,15 +45,6 @@ RemoteFile::~RemoteFile()
         delete sock;
 }
 
-void RemoteFile::Start(void)
-{
-    if (!controlSock)
-    {
-        controlSock = openSocket(true);
-        sock = openSocket(false);
-    }
-}
-
 QSocketDevice *RemoteFile::openSocket(bool control)
 {
     QUrl qurl(path);
@@ -147,7 +129,6 @@ void RemoteFile::Close(void)
     
     lock.unlock();
 
-    qApp->lock();
     if (sock)
     {
         delete sock;
@@ -158,25 +139,20 @@ void RemoteFile::Close(void)
         delete controlSock;
         controlSock = NULL;
     }    
-    qApp->unlock();
 }
 
 void RemoteFile::Reset(void)
 {
-    int avail;
-    char *trash;
-
-    usleep(10000);
-
     while (sock->bytesAvailable() > 0)
     {
+        int avail;
+        char *trash;
+
         lock.lock();
-        qApp->lock();
         avail = sock->bytesAvailable();
         trash = new char[avail + 1];
         sock->readBlock(trash, avail);
         delete [] trash;
-        qApp->unlock();
         lock.unlock();
 
         VERBOSE(VB_NETWORK, QString ("%1 bytes available during reset.")
@@ -184,20 +160,6 @@ void RemoteFile::Reset(void)
         usleep(30000);
     }
 }
-    
-bool RemoteFile::RequestBlock(int size)
-{
-    QStringList strlist = QString(query).arg(recordernum);
-    strlist << "REQUEST_BLOCK" + append;
-    strlist << QString::number(size);
-
-    lock.lock();
-    WriteStringList(controlSock, strlist);
-    ReadStringList(controlSock, strlist, true);
-    lock.unlock();
-    
-    return strlist[0].toInt();
-}
 
 long long RemoteFile::Seek(long long pos, int whence, long long curpos)
 {
@@ -223,48 +185,102 @@ long long RemoteFile::Seek(long long pos, int whence, long long curpos)
     return retval;
 }
 
-int RemoteFile::Read(void *data, int size, bool singlefile)
+int RemoteFile::Read(void *data, int size)
 {
-    int ret;
-    unsigned tot = 0;
+    int recv = 0;
+    int sent = 0;
     unsigned zerocnt = 0;
+    bool error = false;
+    bool response = false;
     
-    qApp->lock();
-    while (sock->bytesAvailable() < size)
+    if (!sock->isOpen() || sock->error())
+        return -1;
+   
+    if (!controlSock->isOpen() || controlSock->error())
+        return -1;
+    
+    lock.lock();
+    
+    if (sock->bytesAvailable() > 0)
     {
-        int reqsize = 64000;
-        if (singlefile && size - sock->bytesAvailable() < 64000)
-            reqsize = size - sock->bytesAvailable();
-        qApp->unlock();
-
-        RequestBlock(reqsize);
-
-        zerocnt++;
-        if (zerocnt == 100)
+        VERBOSE(VB_NETWORK, 
+                "RemoteFile::Read(): Read socket not empty to start!");
+        while (sock->waitForMore(5) > 0)
         {
-            printf("EOF %u\n", size);
-            break;
+            int avail = sock->bytesAvailable();
+            char *trash = new char[avail + 1];
+            sock->readBlock(trash, avail);
+            delete [] trash;
         }
-        usleep(50);
-        qApp->lock();
     }
-
-    if (sock->bytesAvailable() >= size)
+    
+    if (controlSock->bytesAvailable() > 0)
     {
-        ret = sock->readBlock(((char *)data) + tot, size - tot);
-        tot += ret;
+        VERBOSE(VB_NETWORK, 
+                "RemoteFile::Read(): Control socket not empty to start!");
+        QStringList tempstrlist;
+        ReadStringList(controlSock, tempstrlist);
+    }
+    
+    QStringList strlist = QString(query).arg(recordernum);
+    strlist << "REQUEST_BLOCK" + append;
+    strlist << QString::number(size);
+    WriteStringList(controlSock, strlist);
+
+    sent = size;
+    
+    while (recv < sent && !error && zerocnt++ < 50)
+    {
+        while (recv < sent && sock->waitForMore(200) > 0)
+        {
+            int ret = sock->readBlock(((char *)data) + recv, sent - recv);
+            if (ret > 0)
+            {
+                recv += ret;
+            }
+            else if (sock->error() != QSocketDevice::NoError)
+            {
+                VERBOSE(VB_IMPORTANT, "RemoteFile::Read(): socket error");
+                error = true;
+                break;
+            }
+        }
+
+        if (controlSock->bytesAvailable() > 0)
+        {
+            ReadStringList(controlSock, strlist, true);
+            sent = strlist[0].toInt(); // -1 on backend error
+            response = true;
+        }
+    } 
+    
+    if (!error && !response)
+    {
+        if (ReadStringList(controlSock, strlist, true))
+        {
+            sent = strlist[0].toInt(); // -1 on backend error
+        }
+        else
+        {
+            VERBOSE(VB_IMPORTANT, 
+                   "RemoteFile::Read(): No response from control socket.");
+            sent = -1;
+        }
     }
 
-    qApp->unlock();
+    lock.unlock();
 
-    readposition += tot;
-    return tot;
+    VERBOSE(VB_NETWORK, QString("Read(): reqd=%1, rcvd=%2, rept=%3, error=%4")
+                                .arg(size).arg(recv).arg(sent).arg(error));
+
+    if (error || sent != recv)
+        recv = -1;
+
+    return recv;
 }
 
 bool RemoteFile::SaveAs(QByteArray &data)
 {
-    Start();
-
     if (filesize < 0)
         return false;
 
