@@ -1,6 +1,8 @@
 #include <iostream>
 using namespace std;
 
+#include <cmath>
+
 #include "xmlparse.h"
 
 
@@ -73,7 +75,7 @@ bool XMLParse::LoadTheme(QDomElement &ele, QString winName, QString specialfile)
                 if (name.isNull() || name.isEmpty())
                 {
                     cerr << "Window needs a name\n";
-                    exit(0);
+                    return false;
                 }
 
                 if (name == winName)
@@ -85,7 +87,7 @@ bool XMLParse::LoadTheme(QDomElement &ele, QString winName, QString specialfile)
             else
             {
                 cerr << "Unknown element: " << e.tagName() << endl;
-                exit(0);
+                return false;
             }
         }
         n = n.nextSibling();
@@ -122,14 +124,14 @@ void XMLParse::parseFont(QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "Font needs a name\n";
-        exit(0);
+        return;
     }
 
     face = element.attribute("face", "");
     if (face.isNull() || face.isEmpty())
     {
         cerr << "Font needs a face\n";
-        exit(0);
+        return;
     }
 
     for (QDomNode child = element.firstChild(); !child.isNull();
@@ -171,7 +173,7 @@ void XMLParse::parseFont(QDomElement &element)
             else
             {
                 cerr << "Unknown tag " << info.tagName() << " in font\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -180,7 +182,7 @@ void XMLParse::parseFont(QDomElement &element)
     if (testFont)
     {
         cerr << "Error: already have a font called: " << name << endl;
-        exit(0);
+        return;
     }
 
     if (fontSizeType == "small")
@@ -197,7 +199,7 @@ void XMLParse::parseFont(QDomElement &element)
     if (size < 0)
     {
         cerr << "Error: font size must be > 0\n";
-        exit(0);
+        return;
     }
 
     size = (int)ceil(size * hmult);
@@ -224,14 +226,14 @@ void XMLParse::parseImage(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "Image needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "Image needs an order\n";
-        exit(0);
+        return;
     }
 
     QString filename = "";
@@ -273,7 +275,7 @@ void XMLParse::parseImage(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in image\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -310,14 +312,14 @@ void XMLParse::parseRepeatedImage(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "Repeated Image needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "Repeated Image needs an order\n";
-        exit(0);
+        return;
     }
 
     QString filename = "";
@@ -379,7 +381,7 @@ void XMLParse::parseRepeatedImage(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in repeated image\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -431,14 +433,14 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "Guide needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "Guide needs an order\n";
-        exit(0);
+        return;
     }
 
     for (QDomNode child = element.firstChild(); !child.isNull();
@@ -548,7 +550,7 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in bar\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -556,7 +558,7 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
     if (!testfont)
     {
         cerr << "Unknown font: " << font << " in guidegrid: " << name << endl;
-        exit(0);
+        return;
     }
 
     UIGuideType *guide = new UIGuideType(name, order.toInt());
@@ -624,14 +626,14 @@ void XMLParse::parseBar(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "Bar needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "Bar needs an order\n";
-        exit(0);
+        return;
     }
 
     for (QDomNode child = element.firstChild(); !child.isNull();
@@ -686,7 +688,7 @@ void XMLParse::parseBar(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in bar\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -694,7 +696,7 @@ void XMLParse::parseBar(LayerSet *container, QDomElement &element)
     if (!testfont)
     {
         cerr << "Unknown font: " << font << " in bar: " << name << endl;
-        exit(0);
+        return;
     }
 
     UIBarType *bar = new UIBarType(name, filename, order.toInt(), area);
@@ -777,14 +779,14 @@ void XMLParse::parseContainer(QDomElement &element, QString &newname, int &conte
     if (name.isNull() || name.isEmpty())
     {
         cerr << "Container needs a name\n";
-        exit(0);
+        return;
     }
 
     LayerSet *container = GetSet(name);
     if (container)
     {
         cerr << "Container: " << name << " already exists\n";
-        exit(0);
+        return;
     }
     newname = name;
 
@@ -820,6 +822,10 @@ void XMLParse::parseContainer(QDomElement &element, QString &newname, int &conte
             {
                 parseListArea(container, info);
             }
+            else if (info.tagName() == "listbtnarea")
+            {
+                parseListBtnArea(container, info);
+            }
             else if (info.tagName() == "textarea")
             {
                 parseTextArea(container, info);
@@ -869,7 +875,7 @@ void XMLParse::parseContainer(QDomElement &element, QString &newname, int &conte
             else
             {
                 cerr << "Unknown container child: " << info.tagName() << endl;
-                exit(0);
+                return;
             }
         }
     }
@@ -897,14 +903,14 @@ void XMLParse::parseTextArea(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "Text area needs a name\n";
-        exit(0);
+        return;
     }
 
     QString layerNum = element.attribute("draworder", "");
     if (layerNum.isNull() && layerNum.isEmpty())
     {
         cerr << "Text area needs a draworder\n";
-        exit(0);
+        return;
     }
     draworder = layerNum.toInt();
 
@@ -937,7 +943,7 @@ void XMLParse::parseTextArea(LayerSet *container, QDomElement &element)
                 if ((value.isNull() || value.isEmpty()) && 
                     info.attribute("lang","") == "")
                 {
-                    value = getFirstText(info);
+                    value = qApp->translate("ThemeUI", getFirstText(info));
                 }
                 else if (info.attribute("lang","").lower() == 
                          gContext->GetLanguage())
@@ -962,7 +968,7 @@ void XMLParse::parseTextArea(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown tag in listarea: " << info.tagName() << endl;
-                exit(0);
+                return;
             }
         }
     }
@@ -971,7 +977,7 @@ void XMLParse::parseTextArea(LayerSet *container, QDomElement &element)
     if (!testfont)
     {
         cerr << "Unknown font: " << font << " in textarea: " << name << endl;
-        exit(0);
+        return;
     }
 
     UITextType *text = new UITextType(name, testfont, value, draworder, area,
@@ -1042,14 +1048,14 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "List area needs a name\n";
-        exit(0);
+        return;
     }
 
     QString layerNum = element.attribute("draworder", "");
     if (layerNum.isNull() && layerNum.isEmpty())
     {
         cerr << "List area needs a draworder\n";
-        exit(0);
+        return;
     }
     draworder = layerNum.toInt();
 
@@ -1095,13 +1101,13 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                 if (fontname.isNull() || fontname.isEmpty())
                 {   
                     cerr << "FcnFont needs a name\n";
-                    exit(0);
+                    return;
                 }
 
                 if (fontfcn.isNull() || fontfcn.isEmpty())
                 {   
                     cerr << "FcnFont needs a function\n";
-                    exit(0);
+                    return;
                 }
                 fontFunctions[fontfcn] = fontname;
             }
@@ -1120,12 +1126,12 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                 if (fillfcn.isNull() || fillfcn.isEmpty())
                 {
                     cerr << "Fill needs a function\n";
-                    exit(0);
+                    return;
                 }
                 if (fillcolor.isNull() || fillcolor.isEmpty())
                 {
                     cerr << "Fill needs a color\n";
-                    exit(0);
+                    return;
                 }
                 if (filltype == "5050")
                     fill_type = 1;
@@ -1154,21 +1160,21 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                 if (imgname.isNull() || imgname.isEmpty())
                 {
                     cerr << "Image needs a function\n";
-                    exit(0);
+                    return;
                 }
 
                 imgfile = info.attribute("filename", "");
                 if (imgfile.isNull() || imgfile.isEmpty())
                 {
                     cerr << "Image needs a filename\n";
-                    exit(0);
+                    return;
                 }
 
                 imgpoint = info.attribute("location", "");
                 if (imgpoint.isNull() && imgpoint.isEmpty())
                 {
                     cerr << "Image needs a location\n";
-                    exit(0);
+                    return;
                 }
 
                 if (imgname.lower() == "selectionbar")
@@ -1202,14 +1208,14 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                 if (colnum.isNull() || colnum.isEmpty())
                 {
                     cerr << "Column needs a number\n";
-                    exit(0);
+                    return;
                 }
 
                 colwidth = info.attribute("width", "");
                 if (colwidth.isNull() && colwidth.isEmpty())
                 {
                     cerr << "Column needs a width\n";
-                    exit(0);
+                    return;
                 }
 
                 colcontext = info.attribute("context", "");
@@ -1227,7 +1233,7 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown tag in listarea: " << info.tagName() << endl;
-                exit(0);
+                return;
             }
         }
     }
@@ -1307,14 +1313,14 @@ void XMLParse::parseStatusBar(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "StatusBar needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "StatusBar needs an order\n";
-        exit(0);
+        return;
     }
 
     QPoint pos = QPoint(0, 0);
@@ -1405,12 +1411,13 @@ void XMLParse::parseStatusBar(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in statusbar\n";
-                exit(0);
+                return;
             }
         }
     }
 
     UIStatusBarType *sb = new UIStatusBarType(name, pos, order.toInt());
+    sb->SetScreen(wmult, hmult);
     sb->SetFiller(imgFillSpace);
     if (imgContainer)
     {
@@ -1466,14 +1473,14 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "ManagedTreeList needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "ManagedTreeList needs an order\n";
-        exit(0);
+        return;
     }
 
     QString bins_string = element.attribute("bins", "");
@@ -1502,14 +1509,14 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                 if (imgname.isNull() || imgname.isEmpty())
                 {
                     cerr << "Image needs a function\n";
-                    exit(0);
+                    return;
                 }
 
                 file = info.attribute("filename", "");
                 if (file.isNull() || file.isEmpty())
                 {
                     cerr << "Image needs a filename\n";
-                    exit(0);
+                    return;
                 }
     
                 if(info.tagName() == "context")
@@ -1569,12 +1576,12 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                 if(whichbin < 1)
                 {
                     cerr << "xmlparse.o: Bad setting for bin number in bin tag" << endl;
-                    exit(0);
+                    return;
                 }
                 if(whichbin > bins + 1)
                 {
                     cerr << "xmlparse.o: Attempt to set bin with a reference larger than number of bins" << endl;
-                    exit(0);
+                    return;
                 }
                 for (QDomNode child = info.firstChild(); !child.isNull();
                 child = child.nextSibling())
@@ -1599,13 +1606,13 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                             if (fontname.isNull() || fontname.isEmpty())
                             {   
                                 cerr << "FcnFont needs a name\n";
-                                exit(0);
+                                return;
                             }
 
                             if (fontfcn.isNull() || fontfcn.isEmpty())
                             {   
                                 cerr << "FcnFont needs a function\n";
-                                exit(0);
+                                return;
                             }
                             QString a_string = QString("bin%1-%2").arg(whichbin).arg(fontfcn);
                             fontFunctions[a_string] = fontname;
@@ -1613,7 +1620,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                         else
                         {
                             cerr << "Unknown tag in bin: " << info.tagName() << endl;
-                            exit(0);
+                            return;
                         }
                     }
                 }
@@ -1621,13 +1628,14 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in ManagedTreeList\n";
-                exit(0);
+                return;
             }
         }
     }
 
 
     UIManagedTreeListType *mtl = new UIManagedTreeListType(name);
+    mtl->SetScreen(wmult, hmult);
     mtl->setArea(area);
     mtl->setBins(bins);
     mtl->setBinAreas(bin_corners);
@@ -1691,14 +1699,14 @@ void XMLParse::parsePushButton(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "PushButton needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "PushButton needs an order\n";
-        exit(0);
+        return;
     }
 
     for (QDomNode child = element.firstChild(); !child.isNull();
@@ -1726,14 +1734,14 @@ void XMLParse::parsePushButton(LayerSet *container, QDomElement &element)
                 if (imgname.isNull() || imgname.isEmpty())
                 {
                     cerr << "Image in a push button needs a function\n";
-                    exit(0);
+                    return;
                 }
 
                 file = info.attribute("filename", "");
                 if (file.isNull() || file.isEmpty())
                 {
                     cerr << "Image in a push button needs a filename\n";
-                    exit(0);
+                    return;
                 }
 
                 if (imgname.lower() == "on")
@@ -1764,7 +1772,7 @@ void XMLParse::parsePushButton(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in PushButton\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -1783,6 +1791,7 @@ void XMLParse::parsePushButton(LayerSet *container, QDomElement &element)
     delete image_off;
     delete image_pushed;
 
+    pbt->SetScreen(wmult, hmult);
     pbt->setPosition(pos);
     pbt->SetOrder(order.toInt());
     if (context != -1)
@@ -1807,14 +1816,14 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "TextButton needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "TextButton needs an order\n";
-        exit(0);
+        return;
     }
 
     for (QDomNode child = element.firstChild(); !child.isNull();
@@ -1846,14 +1855,14 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
                 if (imgname.isNull() || imgname.isEmpty())
                 {
                     cerr << "Image in a text button needs a function\n";
-                    exit(0);
+                    return;
                 }
 
                 file = info.attribute("filename", "");
                 if (file.isNull() || file.isEmpty())
                 {
                     cerr << "Image in a text button needs a filename\n";
-                    exit(0);
+                    return;
                 }
 
                 if (imgname.lower() == "on")
@@ -1885,7 +1894,7 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in TextButton\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -1894,7 +1903,7 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
     if (!testfont)
     {
         cerr << "Unknown font: " << font << " in textbutton: " << name << endl;
-        exit(0);
+        return;
     }
 
     if (!image_on)
@@ -1911,6 +1920,7 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
     delete image_off;
     delete image_pushed;
 
+    tbt->SetScreen(wmult, hmult);
     tbt->setPosition(pos);
     tbt->setFont(testfont);
     tbt->SetOrder(order.toInt());
@@ -1936,14 +1946,14 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "CheckBox needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "CheckBox needs an order\n";
-        exit(0);
+        return;
     }
 
     for (QDomNode child = element.firstChild(); !child.isNull();
@@ -1971,14 +1981,14 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
                 if (imgname.isNull() || imgname.isEmpty())
                 {
                     cerr << "Image in a CheckBox needs a function\n";
-                    exit(0);
+                    return;
                 }
 
                 file = info.attribute("filename", "");
                 if (file.isNull() || file.isEmpty())
                 {
                     cerr << "Image in a CheckBox needs a filename\n";
-                    exit(0);
+                    return;
                 }
 
                 if (imgname.lower() == "checked")
@@ -2017,7 +2027,7 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in CheckBox\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -2040,6 +2050,7 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
     delete image_checked_high;
     delete image_unchecked_high;
 
+    cbt->SetScreen(wmult, hmult);
     cbt->setPosition(pos);
     cbt->SetOrder(order.toInt());
     if (context != -1)
@@ -2064,14 +2075,14 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "Selector needs a name\n";
-        exit(0);
+        return;
     }
 
     QString order = element.attribute("draworder", "");
     if (order.isNull() || order.isEmpty())
     {
         cerr << "Selector needs an order\n";
-        exit(0);
+        return;
     }
 
     for (QDomNode child = element.firstChild(); !child.isNull();
@@ -2102,14 +2113,14 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
                 if (imgname.isNull() || imgname.isEmpty())
                 {
                     cerr << "Image in a selector needs a function\n";
-                    exit(0);
+                    return;
                 }
 
                 file = info.attribute("filename", "");
                 if (file.isNull() || file.isEmpty())
                 {
                     cerr << "Image in a selector needs a filename\n";
-                    exit(0);
+                    return;
                 }
 
                 if (imgname.lower() == "on")
@@ -2141,7 +2152,7 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in Selector\n";
-                exit(0);
+                return;
             }
         }
     }
@@ -2150,7 +2161,7 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
     if (!testfont)
     {
         cerr << "Unknown font: " << font << " in Selector: " << name << endl;
-        exit(0);
+        return;
     }
 
     if (!image_on)
@@ -2167,6 +2178,7 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
     delete image_off;
     delete image_pushed;
 
+    st->SetScreen(wmult, hmult);
     st->setPosition(area.topLeft());
     st->setFont(testfont);
     st->SetOrder(order.toInt());
@@ -2188,7 +2200,7 @@ void XMLParse::parseBlackHole(LayerSet *container, QDomElement &element)
     if (name.isNull() || name.isEmpty())
     {
         cerr << "BlackHole needs a name\n";
-        exit(0);
+        return;
     }
 
     for (QDomNode child = element.firstChild(); !child.isNull();
@@ -2205,16 +2217,161 @@ void XMLParse::parseBlackHole(LayerSet *container, QDomElement &element)
             else
             {
                 cerr << "Unknown: " << info.tagName() << " in Black Hole\n";
-                exit(0);
+                return;
             }
         }
     }
 
 
     UIBlackHoleType *bh = new UIBlackHoleType(name);
+    bh->SetScreen(wmult, hmult);
     bh->setArea(area);
     bh->SetParent(container);
     bh->calculateScreenArea();
     container->AddType(bh);
 }
 
+void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
+{
+    QRect   area = QRect(0,0,0,0);
+    QString fontActive;
+    QString fontInactive;
+    bool    showArrow = true;
+    bool    showScrollArrows = false;
+    bool    showChecked = false;
+    int     draworder = 0;
+    QColor  grUnselectedBeg(Qt::black);
+    QColor  grUnselectedEnd(80,80,80);
+    uint    grUnselectedAlpha(100);
+    QColor  grSelectedBeg(82,202,56);
+    QColor  grSelectedEnd(52,152,56);
+    uint    grSelectedAlpha(255);
+    int     spacing = 2;
+    int     margin = 3;
+
+    QString name = element.attribute("name", "");
+    if (name.isEmpty()) {
+        std::cerr << "ListBtn area needs a name" << std::endl;
+        return;
+    }
+
+    QString layerNum = element.attribute("draworder", "");
+    if (layerNum.isNull() || layerNum.isEmpty())
+    {
+        cerr << "ListBtn area needs a draworder\n";
+        return;
+    }
+    
+    draworder = layerNum.toInt();
+    for (QDomNode child = element.firstChild(); !child.isNull();
+         child = child.nextSibling()) {
+        QDomElement info = child.toElement();
+        if (!info.isNull())
+        {
+            if (info.tagName() == "area")
+            {
+                area = parseRect(getFirstText(info));
+                normalizeRect(area);
+            }
+            else if (info.tagName() == "fcnfont")
+            {
+                QString fontName = info.attribute("name", "");
+                QString fontFcn  = info.attribute("function", "");
+
+                if (fontFcn.lower() == "active")
+                    fontActive = fontName;
+                else if (fontFcn.lower() == "inactive")
+                    fontInactive = fontFcn;
+                else {
+                    std::cerr << "Unknown font function for listbtn area: "
+                              << fontFcn 
+                              << std::endl;
+                    return;
+                }
+            }
+            else if (info.tagName() == "showarrow") {
+                if (getFirstText(info).lower() == "no")
+                    showArrow = false;
+            }
+            else if (info.tagName() == "showchecks") {
+                if (getFirstText(info).lower() == "yes")
+                    showChecked = true;
+            }
+            else if (info.tagName() == "showscrollarrows") {
+                if (getFirstText(info).lower() == "yes")
+                    showScrollArrows = true;
+            }
+            else if (info.tagName() == "gradient") {
+
+                if (info.attribute("type","").lower() == "selected") {
+                    grSelectedBeg = QColor(info.attribute("start"));
+                    grSelectedEnd = QColor(info.attribute("end"));
+                    grSelectedAlpha = info.attribute("alpha","255").toUInt();
+                }
+                else if (info.attribute("type","").lower() == "unselected") {
+                    grUnselectedBeg = QColor(info.attribute("start"));
+                    grUnselectedEnd = QColor(info.attribute("end"));
+                    grUnselectedAlpha = info.attribute("alpha","100").toUInt();
+                }
+                else {
+                    std::cerr << "Unknown type for gradient in listbtn area"
+                              << std::endl;
+                    return;
+                }
+
+                if (!grSelectedBeg.isValid() || !grSelectedEnd.isValid() ||
+                    !grUnselectedBeg.isValid() || !grUnselectedEnd.isValid()) {
+                    std::cerr << "Unknown color for gradient in listbtn area"
+                              << std::endl;
+                    return;
+                }
+
+                if (grSelectedAlpha > 255 || grUnselectedAlpha > 255) {
+                    std::cerr << "Incorrect alpha for gradient in listbtn area"
+                              << std::endl;
+                    return;
+                }
+            }
+            else if (info.tagName() == "spacing") {
+                spacing = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "margin") {
+                margin = getFirstText(info).toInt();
+            }
+            else
+            {
+                std::cerr << "Unknown tag in listbtn area: " << info.tagName() << endl;
+                return;
+            }
+            
+        }
+    }
+
+    fontProp *fpActive = GetFont(fontActive);
+    if (!fpActive)
+    {
+        cerr << "Unknown font: " << fontActive << " in listbtn area: " << name << endl;
+        return;
+    }
+    
+    fontProp *fpInactive = GetFont(fontInactive);
+    if (!fpInactive)
+    {
+        cerr << "Unknown font: " << fontInactive << " in listbtn area: " << name << endl;
+        return;
+    }
+    
+
+    UIListBtnType *l = new UIListBtnType(name, area, draworder, showArrow,
+                                         showScrollArrows, showChecked);
+    l->SetScreen(wmult, hmult);
+    l->SetFontActive(fpActive);
+    l->SetFontInactive(fpInactive);
+    l->SetItemRegColor(grUnselectedBeg, grUnselectedEnd, grUnselectedAlpha);
+    l->SetItemSelColor(grSelectedBeg, grSelectedEnd, grSelectedAlpha);
+    l->SetSpacing((int)(spacing*hmult));
+    l->SetMargin((int)(margin*wmult));
+    
+    container->AddType(l);
+    
+}
