@@ -5,6 +5,7 @@
 #include <qmutex.h>
 #include <qwaitcondition.h>
 #include <qptrqueue.h>
+#include <sys/time.h>
 
 #include "RingBuffer.h"
 #include "osd.h"
@@ -20,10 +21,6 @@ using namespace std;
 
 #define MAXTBUFFER 21
 
-#define REENCODE_OK              1
-#define REENCODE_CUTLIST_CHANGE -1
-#define REENCODE_ERROR           0
-
 class VideoOutput;
 class OSDSet;
 class RemoteEncoder;
@@ -52,14 +49,17 @@ class NuppelVideoPlayer
 
     void SetAsPIP(void) { disableaudio = disablevideo = true; }
     void SetNoAudio(void) { disableaudio = true; }
+    void SetNoVideo(void) { disablevideo = true; }
 
     void SetAudioDevice(QString device) { audiodevice = device; }
     void SetFileName(QString lfilename) { filename = lfilename; }
 
     void SetExactSeeks(bool exact) { exactseeks = exact; }
     void SetAutoCommercialSkip(int autoskip);
+    void SetTryUnflaggedSkip(bool tryskip) { tryunflaggedskip = tryskip; };
     void SetCommercialSkipMethod(int method) { commercialskipmethod = method; }
 
+    int OpenFile(bool skipDsp = false);
     void StartPlaying(void);
     void StopPlaying(void) { killplayer = true; decoder_thread_alive = false; }
     
@@ -70,16 +70,11 @@ class NuppelVideoPlayer
 
     void SetAudioSampleRate(int rate) { audio_samplerate = rate; }
 
-    bool TogglePause(void) { if (paused) Unpause(); else Pause(); 
-                             return paused; 
-                           }
     void Pause(bool waitvideo = true);
-    void Unpause(bool unpauseaudio = true); 
+    bool Play(float speed = 1.0, bool normal = true, 
+              bool unpauseaudio = true); 
     bool GetPause(void);
 
-    float GetPlaySpeed(void) { return play_speed; };
-    void SetPlaySpeed(float speed, bool normal);
-
     bool FastForward(float seconds);
     bool Rewind(float seconds);
 
@@ -87,8 +82,11 @@ class NuppelVideoPlayer
 
     void ResetPlaying(void);
 
+    int GetVideoWidth(void) { return video_width; }
+    int GetVideoHeight(void) { return video_height; }
     float GetFrameRate(void) { return video_frame_rate; } 
     long long GetFramesPlayed(void) { return framesPlayed; }
+    int GetSecondsBehind(void);
 
     void SetRecorder(RemoteEncoder *recorder);
 
@@ -105,14 +103,20 @@ class NuppelVideoPlayer
     void SetLength(int len) { totalLength = len; }
     int GetLength(void) { return totalLength; }
 
-    int ReencodeFile(char *inputname, char *outputname,
-                     QString profileName, bool honorCutList = false,
-                     bool framecontrol = false, bool chkTranscodeDB = false,
-                     QString fifodir = NULL);
+    QString GetEncodingType(void);
+    void SetAudioOutput (AudioOutput *ao) { audioOutput = ao; }
+    void FlushTxtBuffers(void) { rtxt = wtxt; }
+    bool WriteStoredData(RingBuffer *outRingBuffer, bool writevideo);
+    long UpdateStoredFrameNum(long curFrameNum);
+    void InitForTranscode(bool copyaudio, bool copyvideo);
+    bool TranscodeGetNextFrame(QMap<long long, int>::Iterator &dm_iter,
+                               int *did_ff, bool *is_key, bool honorCutList);
+    void TranscodeWriteText(void (*func)(void *, unsigned char *, int, int, int), void *ptr);
 
     int FlagCommercials(bool showPercentage = false, bool fullSpeed = false);
 
     VideoFrame *GetCurrentFrame(int &w, int &h);
+    void ReleaseCurrentFrame(VideoFrame *frame);
 
     void SetPipPlayer(NuppelVideoPlayer *pip) { setpipplayer = pip; 
                                                 needsetpipplayer = true; }
@@ -132,7 +136,7 @@ class NuppelVideoPlayer
     bool DoKeypress(QKeyEvent *e);
     bool GetEditMode(void) { return editmode; }
 
-    void EmbedInWidget(unsigned long wid, int x, int y, int w, int h);
+    void EmbedInWidget(WId wid, int x, int y, int w, int h);
     void StopEmbedding(void);
 
     // decoder stuff..
@@ -150,6 +154,7 @@ class NuppelVideoPlayer
 
     void DrawSlice(VideoFrame *frame, int x, int y, int w, int h);
 
+    bool GetRawAudioState(void);
     void AddAudioData(char *buffer, int len, long long timecode);
     void AddAudioData(short int *lbuffer, short int *rbuffer, int samples,
                       long long timecode);
@@ -162,7 +167,7 @@ class NuppelVideoPlayer
 
     VideoOutput *getVideoOutput(void) { return videoOutput; }
 
-    int calcSliderPos(float offset, QString &desc);
+    int calcSliderPos(QString &desc);
 
     bool GetLimitKeyRepeat(void) { return limitKeyRepeat; }
 
@@ -193,8 +198,6 @@ class NuppelVideoPlayer
 
     void setPrebuffering(bool prebuffer);
  
-    int OpenFile(bool skipDsp = false);
-
     bool DecodeFrame(struct rtframeheader *frameheader,
                      unsigned char *strm, unsigned char *outbuf);
     void GetFrame(int onlyvideo, bool unsafe = false);
@@ -247,8 +250,7 @@ class NuppelVideoPlayer
     void SetBlankIter(void);
     void SetCommBreakIter(void);
 
-    void ReencoderAddKFA(QPtrList<struct kfatable_entry> *kfa_table,
-                         long curframe, long lastkey, long num_keyframes);
+    float WarpFactor(void);
 
     QString filename;
     
@@ -271,6 +273,8 @@ class NuppelVideoPlayer
 
     /* Video circular buffer */
     bool prebuffering;	/* don't play until done prebuffering */ 
+    QMutex prebuffering_lock;
+    QWaitCondition prebuffering_wait;
 
     /* Text circular buffer */
     int wtxt;          /* next slot to write */
@@ -289,7 +293,7 @@ class NuppelVideoPlayer
 
     AudioOutput *audioOutput;
 
-    bool paused, pausevideo;
+    bool paused, previously_paused, pausevideo;
     bool actuallypaused, video_actually_paused;
     QWaitCondition decoderThreadPaused, videoThreadPaused;
  
@@ -359,7 +363,7 @@ class NuppelVideoPlayer
     bool hascommbreaktable;
     bool hasFullPositionMap;
 
-    unsigned int embedid;
+    WId embedid;
     int embx, emby, embw, embh;
 
     QSqlDatabase *m_db;
@@ -372,6 +376,7 @@ class NuppelVideoPlayer
     int skipcommercials;
     int autocommercialskip;
     int commercialskipmethod;
+    bool tryunflaggedskip;
 
     QString cclines[4];
     int ccindent[4];
@@ -384,10 +389,19 @@ class NuppelVideoPlayer
     int delay;
     int avsync_delay;
     int avsync_avg;
+    int avsync_oldavg;
     int refreshrate;
     int frame_interval; // always adjusted for play_speed
     float play_speed;  
     bool normal_speed;
+
+    float warpfactor;
+    float warpfactor_avg;
+    int rtcfd;
+    int vsynctol;
+    short int *warplbuff;
+    short int *warprbuff;
+    int warpbuffsize;
  
     bool delay_clipping;
     struct timeval nexttrigger, now;
@@ -398,6 +412,10 @@ class NuppelVideoPlayer
 
     Jitterometer *output_jmeter;
 
+    void InitVTAVSync(void);
+    void VTAVSync(void);
+    void ShutdownVTAVSync(void);
+
     void InitExAVSync(void);
     void OldAVSync(void);
     void ExAVSync(void);
@@ -405,10 +423,13 @@ class NuppelVideoPlayer
 
     bool reducejitter;
     bool experimentalsync;
+    bool usevideotimebase;
 
     bool limitKeyRepeat;
 
     QWidget *parentWidget;
+
+    QMutex vidExitLock;
 };
 
 #endif
