@@ -1,5 +1,5 @@
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -7,8 +7,8 @@
 #include <sys/soundcard.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
-#include <errno.h>
-#include <math.h>
+#include <cerrno>
+#include <cmath>
 
 #include <qstringlist.h>
 
@@ -18,12 +18,13 @@ using namespace std;
 #include "mythcontext.h"
 #include "NuppelVideoRecorder.h"
 #include "commercial_skip.h"
-#include "../libvbitext/cc.h"
+#include "vbitext/cc.h"
 #include "channelbase.h"
 #include "filtermanager.h"
+#include "recordingprofile.h"
 
 extern "C" {
-#include "../libvbitext/vbi.h"
+#include "vbitext/vbi.h"
 }
 
 #include "videodev_myth.h"
@@ -39,7 +40,7 @@ extern "C" {
 
 #include "programinfo.h"
 
-pthread_mutex_t avcodeclock = PTHREAD_MUTEX_INITIALIZER;
+extern pthread_mutex_t avcodeclock;
 
 NuppelVideoRecorder::NuppelVideoRecorder(ChannelBase *channel)
                    : RecorderBase()
@@ -88,6 +89,7 @@ NuppelVideoRecorder::NuppelVideoRecorder(ChannelBase *channel)
     text_buffer_size = 0;
 
     childrenLive = false;
+    errored = false;
 
     recording = false;
 
@@ -268,6 +270,93 @@ void NuppelVideoRecorder::SetOption(const QString &opt, int value)
         RecorderBase::SetOption(opt, value);
 }
 
+void NuppelVideoRecorder::SetOptionsFromProfile(RecordingProfile *profile,
+                                                const QString &videodev, 
+                                                const QString &audiodev,
+                                                const QString &vbidev, 
+                                                int ispip)
+{
+    SetOption("videodevice", videodev);
+    SetOption("vbidevice", vbidev);
+    SetOption("tvformat", gContext->GetSetting("TVFormat"));
+    SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
+    SetOption("audiodevice", audiodev);
+
+    QString setting = profile->byName("videocodec")->getValue();
+    if (setting == "MPEG-4")
+    {
+        SetOption("codec", "mpeg4");
+
+        SetIntOption(profile, "mpeg4bitrate");
+        SetIntOption(profile, "mpeg4scalebitrate");
+        SetIntOption(profile, "mpeg4maxquality");
+        SetIntOption(profile, "mpeg4minquality");
+        SetIntOption(profile, "mpeg4qualdiff");
+        SetIntOption(profile, "mpeg4optionvhq");
+        SetIntOption(profile, "mpeg4option4mv");
+    }
+    else if (setting == "RTjpeg")
+    {
+        SetOption("codec", "rtjpeg");
+
+        SetIntOption(profile, "rtjpegquality");
+        SetIntOption(profile, "rtjpegchromafilter");
+        SetIntOption(profile, "rtjpeglumafilter");
+    }
+    else if (setting == "Hardware MJPEG")
+    {
+        SetOption("codec", "hardware-mjpeg");
+
+        SetIntOption(profile, "hardwaremjpegquality");
+        SetIntOption(profile, "hardwaremjpeghdecimation");
+        SetIntOption(profile, "hardwaremjpegvdecimation");
+    }
+    else
+    {
+        cerr << "Unknown video codec\n";
+        cerr << "Please go into the TV Settings, Recording Profiles and\n";
+        cerr << "setup the four 'Software Encoders' profiles.\n";
+        cerr << "Assuming RTjpeg for now.\n";
+
+        SetOption("codec", "rtjpeg");
+
+        SetIntOption(profile, "rtjpegquality");
+        SetIntOption(profile, "rtjpegchromafilter");
+        SetIntOption(profile, "rtjpeglumafilter");
+    }
+
+    setting = profile->byName("audiocodec")->getValue();
+    if (setting == "MP3")
+    {
+        SetOption("audiocompression", 1);
+        SetIntOption(profile, "mp3quality");
+        SetIntOption(profile, "samplerate");
+    }
+    else if (setting == "Uncompressed")
+        SetOption("audiocompression", 0);
+    else
+    {
+        cerr << "Unknown audio codec\n";
+        SetOption("audiocompression", 0);
+    }
+
+    SetIntOption(profile, "width");
+    SetIntOption(profile, "height");
+
+    if (ispip)
+    {
+        SetOption("codec", "rtjpeg");
+
+        SetOption("width", 160);
+        SetOption("height", 128);
+        SetOption("rtjpegchromafilter", 0);
+        SetOption("rtjpeglumafilter", 0);
+        SetOption("rtjpegquality", 255);
+        SetOption("audiocompression", 0);
+        SetOption("pip_mode", 1);
+    }
+}
+
 void NuppelVideoRecorder::Pause(bool clear)
 {
     cleartimeonpause = clear;
@@ -304,6 +393,11 @@ bool NuppelVideoRecorder::IsRecording(void)
     return recording;
 }
 
+bool NuppelVideoRecorder::IsErrored(void)
+{
+    return errored;
+}
+
 long long NuppelVideoRecorder::GetFramesWritten(void)
 {
     return framesWritten;
@@ -461,7 +555,7 @@ void NuppelVideoRecorder::Initialize(void)
     if (!ringBuffer)
     {
         cerr << "Warning: Old ringbuf creation\n";
-        ringBuffer = new RingBuffer(NULL, "output.nuv", true);
+        ringBuffer = new RingBuffer("output.nuv", true);
         weMadeBuffer = true;
         livetv = false;
     }
@@ -703,6 +797,7 @@ void NuppelVideoRecorder::StartRecording(void)
     if (lzo_init() != LZO_E_OK)
     {
         cerr << "lzo_init() failed, exiting\n";
+        errored = true;
         return;
     }
 
@@ -727,18 +822,21 @@ void NuppelVideoRecorder::StartRecording(void)
     {
         cerr << "Cannot open '" << ringBuffer->GetFilename() << "' for "
              << "writing, exiting\n";
+        errored = true;
         return;
     }
 
     if (childrenLive)
     {
         cerr << "Error: children are already alive\n";
+        errored = true;
         return;
     }
 
     if (SpawnChildren() < 0)
     {
         cerr << "Couldn't spawn children\n";
+        errored = true;
         return;
     }
 
@@ -760,13 +858,20 @@ void NuppelVideoRecorder::StartRecording(void)
     if (getuid() == 0)
         nice(-10);
 
+    int retries = 0;
     fd = open(videodevice.ascii(), O_RDWR);
-    if (fd <= 0)
+    while (fd <= 0)
     {
-        cerr << "Can't open video device: " << videodevice << endl;
-        perror("open video:");
-        KillChildren();
-        return;
+        usleep(30000);
+        fd = open(videodevice.ascii(), O_RDWR);
+        if (retries++ > 5)
+        { 
+            cerr << "Can't open video device: " << videodevice << endl;
+            perror("open video:");
+            KillChildren();
+            errored = true;
+            return;
+        }
     }
 
     usingv4l2 = true;
@@ -1001,19 +1106,42 @@ void NuppelVideoRecorder::DoV4L2(void)
 
     vfmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
+    vfmt.fmt.pix.width = w;
+    vfmt.fmt.pix.height = h;
+    vfmt.fmt.pix.field = V4L2_FIELD_INTERLACED;
+
+    // this is our preferred format, try this first
     if (inpixfmt == FMT_YUV422P)
         vfmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUV422P;
     else
         vfmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUV420;
 
-    vfmt.fmt.pix.width = w;
-    vfmt.fmt.pix.height = h;
-    vfmt.fmt.pix.field = V4L2_FIELD_INTERLACED;
-
     if (ioctl(fd, VIDIOC_S_FMT, &vfmt) < 0)
     {
-        printf("Unable to set desired format\n");
-        exit(0);
+        // this is supported by the cx88 and various ati cards.
+        vfmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUYV;
+
+        if (ioctl(fd, VIDIOC_S_FMT, &vfmt) < 0)
+        {
+            VERBOSE(VB_ALL, "Unable to set desired format");
+            exit(0);
+        }
+        else
+        {
+            // we need to convert the buffer - we can't deal with yuyv directly.
+            if (inpixfmt == FMT_YUV422P)
+            {
+                cerr << "v4l2: yuyv format supported, but yuv422 requested." << endl;
+                cerr << "v4l2: unfortunately, this converter hasn't been written yet." << endl;
+                exit(0);
+            }
+            cerr << "v4l2: format set, getting yuyv from v4l, converting" << endl;
+        }
+    }
+    else
+    {
+        // cool, we can do our preferred format, most likely running on bttv.
+        cerr << "v4l2: format set, getting yuv420 from v4l";
     }
 
     int numbuffers = 5;
@@ -1129,7 +1257,58 @@ again:
         frame = vbuf.index;
 
         if (!paused)
-            BufferIt(buffers[frame], video_buffer_size);
+        {
+            if (vfmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YUYV)
+            {
+                // Convert YUYV to YUV420P
+                unsigned conversion_buffer_size = h * w * 3 / 2;
+                uint8_t conversion_buffer[conversion_buffer_size];
+
+                uint8_t *y_plane = conversion_buffer;
+                uint8_t *cr_plane = y_plane + w * h;
+                uint8_t *cb_plane = cr_plane + w * h / 4;
+
+                uint8_t *src = buffers[frame];
+
+                // Round height to multiple of two.
+                unsigned height = (h / 2) * 2;
+
+                // Treat lines in batches of two, first use color information, then don't
+                unsigned line_size = w * 2;
+
+                for (unsigned line = 0; line < height; line += 2)
+                {
+                    uint8_t *src_endline = src + line_size;
+
+                    // convert first line, use color information
+                    while (src < src_endline)
+                    {
+                        *y_plane++ = *src++;
+                        *cb_plane++ = *src++;
+                        *y_plane++ = *src++;
+                        *cr_plane++ = *src++;
+                    }
+
+                    src_endline = src + line_size;
+
+                    // convert second line, don't use color information
+                    while (src < src_endline)
+                    {
+                        *y_plane++ = *src++;
+                        src++;
+                        *y_plane++ = *src++;
+                        src++;
+                    }
+                }
+
+                BufferIt(conversion_buffer, video_buffer_size);
+            }
+            else
+            {
+                // buffer the frame directly
+                BufferIt(buffers[frame], video_buffer_size);
+            }
+        }
 
         vbuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
         ioctl(fd, VIDIOC_QBUF, &vbuf);
@@ -1558,6 +1737,7 @@ void NuppelVideoRecorder::WriteSeekTable(void)
         curRecording->SetPositionMap(positionMap, MARK_KEYFRAME, db_conn);
         pthread_mutex_unlock(db_lock);
     }
+    delete [] seekbuf;
 }
 
 void NuppelVideoRecorder::WriteKeyFrameAdjustTable(
@@ -1599,9 +1779,9 @@ void NuppelVideoRecorder::WriteKeyFrameAdjustTable(
     delete [] kfa_buf;
 }
 
-void NuppelVideoRecorder::UpdateSeekTable(int frame_num, bool use_db)
+void NuppelVideoRecorder::UpdateSeekTable(int frame_num, bool use_db, long offset)
 {
-    long long position = ringBuffer->GetFileWritePosition();
+    long long position = ringBuffer->GetFileWritePosition() + offset;
     struct seektable_entry ste;
     ste.file_offset = position;
     ste.keyframe_number = frame_num;
