@@ -22,6 +22,14 @@ using namespace std;
 
 #define TFW_BUF_SIZE (2*1024*1024)
 
+#ifndef O_STREAMING
+#define O_STREAMING 0
+#endif
+
+#ifndef O_LARGEFILE
+#define O_LARGEFILE 0
+#endif
+
 class ThreadedFileWriter
 {
 public:
@@ -182,14 +190,19 @@ long long ThreadedFileWriter::Seek(long long pos, int whence)
 
 void ThreadedFileWriter::Sync(void)
 {
+#ifdef _WIN32
+    // Cynwin doesn't have a fdatasync
+    fsync(fd);
+#else
     fdatasync(fd);
+#endif
 }
 
 void ThreadedFileWriter::DiskLoop()
 {
     int size;
-    QDateTime lastsynctime = QDateTime::currentDateTime();
-    QDateTime curtime = lastsynctime;
+    QTime timer;
+    timer.start();
 
     while (!in_dtor || BufUsed() > 0)
     {
@@ -201,11 +214,10 @@ void ThreadedFileWriter::DiskLoop()
             continue;
         }
 
-        curtime = QDateTime::currentDateTime();
-        if (curtime > lastsynctime.addSecs(1))
+        if (timer.elapsed() > 1000)
         {
             Sync();
-            lastsynctime = curtime;
+            timer.restart();
         }
 
         /* cap the max. write size. Prevents the situation where 90% of the
@@ -263,10 +275,12 @@ unsigned ThreadedFileWriter::BufFree()
 
 /**********************************************************************/
 
-RingBuffer::RingBuffer(const QString &lfilename, bool write, bool needevents)
+RingBuffer::RingBuffer(const QString &lfilename, bool write, bool usereadahead)
 {
     Init();
 
+    startreadahead = usereadahead;
+
     normalfile = true;
     filename = (QString)lfilename;
 
@@ -303,9 +317,10 @@ RingBuffer::RingBuffer(const QString &lfilename, bool write, bool needevents)
             is_local = true;
 
         if (is_local)
-            fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE);
+            fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE|O_STREAMING);
         else
-            remotefile = new RemoteFile(filename, needevents);
+            remotefile = new RemoteFile(filename);
+
         writemode = false;
     }
 
@@ -332,11 +347,11 @@ RingBuffer::RingBuffer(const QString &lfilename, long long size,
         tfw = new ThreadedFileWriter(filename.ascii(), 
                                      O_WRONLY|O_CREAT|O_TRUNC|O_LARGEFILE, 
                                      0644);
-        fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE);
+        fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE|O_STREAMING);
     }
     else
     {
-        remotefile = new RemoteFile(filename, false, recorder_num);
+        remotefile = new RemoteFile(filename, recorder_num);
     }
 
     wrapcount = 0;
@@ -345,17 +360,15 @@ RingBuffer::RingBuffer(const QString &lfilename, long long size,
 
 void RingBuffer::Init(void)
 {
+    startreadahead = true;
+
     readaheadrunning = false;
     readaheadpaused = false;
     wantseek = false;
     fill_threshold = -1;
     fill_min = -1;
 
-    readblocksize = 64000; // we only have max. 130k socket buffer size
-                           // we request readblocksize * 2
-                           // averages ~100k/read
-//  readblocksize = 128000;
-    requestedbytes = 0;
+    readblocksize = 128000;
 
     recorder_num = 0;
     remoteencoder = NULL;
@@ -394,13 +407,10 @@ RingBuffer::~RingBuffer(void)
 
 void RingBuffer::Start(void)
 {
-    if (remotefile)
-        remotefile->Start();
-
     if ((normalfile && writemode) || (!normalfile && !recorder_num))
     {
     }
-    else if (!readaheadrunning)
+    else if (!readaheadrunning && startreadahead)
         StartupReadAheadThread();
 }
 
@@ -446,7 +456,7 @@ int RingBuffer::safe_read(int fd, void *data, unsigned sz)
         ret = read(fd, (char *)data + tot, sz - tot);
         if (ret < 0)
         {
-            perror("ERROR: file I/O problem in 'safe_read()'\n");
+            perror("ERROR: file I/O problem in 'safe_read()'");
             errcnt++;
             if (errcnt == 3) 
                 break;
@@ -478,68 +488,17 @@ int RingBuffer::safe_read(int fd, void *data, unsigned sz)
 
 int RingBuffer::safe_read(RemoteFile *rf, void *data, unsigned sz)
 {
-    int ret;
-    unsigned tot = 0;
-    unsigned zerocnt = 0;
-    bool hiteof = false;
-    int reqsize = readblocksize;
+    int ret = 0;
 
-    QSocketDevice *sock = rf->getSocket();
-
-    do 
+    ret = rf->Read(data, sz);
+    if(ret < 0)
     {
-        if (requestedbytes >= reqsize * 2)
-            break;
+        VERBOSE(VB_IMPORTANT, "RemoteFile::Read() failed in RingBuffer::safe_read().");
+        rf->Seek(internalreadpos, SEEK_SET);
+        ret = 0;
+     }
 
-        if (rf->RequestBlock(reqsize))
-        {
-            hiteof = true;
-            VERBOSE(VB_PLAYBACK, "RequestBlock extends beyond EOF.");
-            break;
-        }
-        requestedbytes += reqsize;
-    }
-    while (requestedbytes < reqsize * 2);
-    
-    unsigned int available = sock->bytesAvailable();
-
-    while (available < sz) 
-    {
-        if (hiteof)
-            break;
-
-        zerocnt++;
-        if (zerocnt >= 20)
-        {
-            break;
-        }
-        if (stopreads)
-            break;
-
-        available = sock->bytesAvailable();
-
-        if (available >= sz)
-            break;
-
-        usleep(100);
-    }
-
-    available = sock->bytesAvailable();
-
-    while (available > 0 && tot < sz && !stopreads)
-    {
-        ret = sock->readBlock(((char *)data) + tot, sz - tot);
-        available = sock->bytesAvailable();
-
-        tot += ret;
-    }
-
-    requestedbytes -= tot;
-
-    VERBOSE(VB_NETWORK, QString ("sz: %1 return: %2 requested: %3 avail: %4")
-            .arg(sz).arg(tot).arg(requestedbytes).arg(available));
-
-    return tot;
+    return ret;
 }
 
 #define READ_AHEAD_SIZE (10 * 256000)
@@ -553,6 +512,8 @@ void RingBuffer::CalcReadAheadThresh(int estbitrate)
     fill_threshold = 0;
     fill_min = 0;
 
+    VERBOSE(VB_PLAYBACK, QString("Estimated bitrate = %1").arg(estbitrate));
+
     if (remotefile)
         fill_threshold += 256000;
 
@@ -565,10 +526,8 @@ void RingBuffer::CalcReadAheadThresh(int estbitrate)
     if (estbitrate > 14000)
         fill_threshold += 256000;
 
-/*
     if (estbitrate > 17000)
         readblocksize = 256000;
-*/
 
     fill_min = 1;
 
@@ -620,7 +579,6 @@ void RingBuffer::ResetReadAhead(long long newinternal)
     internalreadpos = newinternal;
     ateof = false;
     readsallowed = false;
-    requestedbytes = 0;
     readAheadLock.unlock();
 }
 
@@ -826,8 +784,7 @@ void RingBuffer::ReadAheadThread(void)
 
         if ((used >= fill_threshold || wantseek) && !pausereadthread)
             usleep(500);
-
-        if (!pausereadthread)
+        else if (!pausereadthread)
             sched_yield();
     }
 
@@ -855,13 +812,28 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
 
     int avail = ReadBufAvail();
 
+    int readErr = 0;
+
     while (avail < count && !stopreads)
     {
         availWaitMutex.lock();
         wanttoread = count;
         if (!availWait.wait(&availWaitMutex, 2000))
+        {
             cerr << "Waited 2 seconds for data to become available, waiting "
                     "again...\n";
+            readErr++;
+            if (readErr > 7)
+            {
+                cerr << "Waited 14 seconds for data to become available, "
+                        "aborting\n";
+                wanttoread = 0;
+                stopreads = true;
+                availWaitMutex.unlock();
+                return 0;
+            }
+        }
+
         wanttoread = 0;
         availWaitMutex.unlock();
 
@@ -908,9 +880,18 @@ int RingBuffer::Read(void *buf, int count)
         {
             if (!readaheadrunning)
             {
-                ret = safe_read(fd2, buf, count);
-                totalreadpos += ret;
-                readpos += ret;
+                if (remotefile)
+                {
+                    ret = safe_read(remotefile, buf, count);
+                    totalreadpos += ret;
+                    readpos += ret;
+                }
+                else
+                {
+                    ret = safe_read(fd2, buf, count);
+                    totalreadpos += ret;
+                    readpos += ret;
+                }
             }
             else
             {
