@@ -24,6 +24,7 @@ Channel::Channel(TVRec *parent, const QString &videodevice)
     isopen = false;
     videofd = -1;
     curList = 0;
+    defaultFreqTable = 1;
     totalChannels = 0;
     usingv4l2 = false;
     videomode = VIDEO_MODE_NTSC;
@@ -111,11 +112,12 @@ void Channel::SetFormat(const QString &format)
         {
             QString msg = QString("Probed: %1 - %2").arg(device)
                             .arg((char *)vin.name);
-            VERBOSE(VB_CHANNEL, msg)
+            VERBOSE(VB_CHANNEL, msg);
             channelnames[vin.index] = (char *)vin.name;
             inputChannel[vin.index] = "";
             inputTuneTo[vin.index] = "";
             externalChanger[vin.index] = "";
+            sourceid[vin.index] = "";
             vin.index++;
 
             capchannels = vin.index;
@@ -139,7 +141,7 @@ void Channel::SetFormat(const QString &format)
             videomode = V4L2_STD_NTSC_M_JP;
 
         pParent->RetrieveInputChannels(inputChannel, inputTuneTo, 
-                                       externalChanger);
+                                       externalChanger, sourceid);
         return;
     }
  
@@ -188,6 +190,7 @@ void Channel::SetFormat(const QString &format)
         inputChannel[i] = "";
         inputTuneTo[i] = "";
         externalChanger[i] = "";
+        sourceid[i] = "";
     }
 
     struct video_channel vc;
@@ -198,10 +201,23 @@ void Channel::SetFormat(const QString &format)
 
     videomode = mode;
 
-    pParent->RetrieveInputChannels(inputChannel, inputTuneTo, externalChanger);
+    pParent->RetrieveInputChannels(inputChannel, inputTuneTo, externalChanger,
+                                   sourceid);
 }
 
-void Channel::SetFreqTable(const QString &name)
+int Channel::SetDefaultFreqTable(const QString &name)
+{
+    defaultFreqTable = SetFreqTable(name);
+    return defaultFreqTable;
+}
+
+void Channel::SetFreqTable(const int index)
+{
+    curList = chanlists[index].list;
+    totalChannels = chanlists[index].count;
+}
+
+int Channel::SetFreqTable(const QString &name)
 {
     int i = 0;
     char *listname = (char *)chanlists[i].name;
@@ -211,19 +227,17 @@ void Channel::SetFreqTable(const QString &name)
     {
         if (name == listname)
         {
-            curList = chanlists[i].list;
-            totalChannels = chanlists[i].count;
-            break;
+            SetFreqTable(i);
+            return i;
         }
         i++;
         listname = (char *)chanlists[i].name;
     }
 
-    if (!curList)
-    {
-        curList = chanlists[1].list;
-        totalChannels = chanlists[1].count;
-    }
+    VERBOSE(VB_CHANNEL, QString("Invalid frequency table name %1, using %2.")
+                        .arg(name).arg((char *)chanlists[1].name));
+    SetFreqTable(1);
+    return 1;
 }
 
 int Channel::GetCurrentChannelNum(const QString &channame)
@@ -321,12 +335,20 @@ bool Channel::SetChannelByString(const QString &chan)
     pthread_mutex_t db_lock;
 
     if (!pParent->CheckChannel(this, chan, db_conn, db_lock))
+    {
+        VERBOSE(VB_IMPORTANT, QString("CheckChannel failed. Please verify "
+                "channel \"%1\" in the \"setup\" Channel Editor.").arg(chan));
         return false;
+    }
 
     pthread_mutex_lock(&db_lock);
-    QString thequery = QString("SELECT finetune, freqid, tvformat "
-                               "FROM channel WHERE channum = \"%1\";")
-                               .arg(chan);
+    QString thequery = QString("SELECT finetune, freqid, tvformat, freqtable "
+                               "FROM channel "
+                               "LEFT JOIN videosource USING (sourceid) "
+                               "WHERE channum = \"%1\" AND "
+                               "channel.sourceid = \"%2\";")
+                               .arg(chan)
+                               .arg(sourceid[currentcapchannel]);
 
     QSqlQuery query = db_conn->exec(thequery);
     if (!query.isActive())
@@ -341,6 +363,15 @@ bool Channel::SetChannelByString(const QString &chan)
     int finetune = query.value(0).toInt();
     QString freqid = query.value(1).toString();
     QString tvformat = query.value(2).toString();
+    QString freqtable = query.value(3).toString();
+
+    if (freqtable == "default" || freqtable.isNull() || freqtable.isEmpty())
+        SetFreqTable(defaultFreqTable);
+    else
+        SetFreqTable(freqtable);
+
+    if (tvformat.isNull() || tvformat.isEmpty())
+        tvformat = "Default";
 
     pthread_mutex_unlock(&db_lock);
 
