@@ -8,7 +8,7 @@ using namespace std;
 
 #include "mythcontext.h"
 
-const QString currentDatabaseVersion = "1019";
+const QString currentDatabaseVersion = "1027";
 
 void UpdateDBVersionNumber(const QString &newnumber)
 {
@@ -430,6 +430,134 @@ void UpgradeTVDatabaseSchema(void)
         performActualUpdate(updates, "1019", dbver);
     }
 
+    if (dbver == "1019")
+    {
+        const QString updates[] = {
+"UPDATE channel set tvformat = 'Default' where tvformat = '';",
+"DELETE FROM keybindings WHERE context = 'TV Playback' and action = 'CONTRASTDOWN';",
+"DELETE FROM keybindings WHERE context = 'TV Playback' and action = 'CONTRASTUP';",
+"DELETE FROM keybindings WHERE context = 'TV Playback' and action = 'BRIGHTDOWN';",
+"DELETE FROM keybindings WHERE context = 'TV Playback' and action = 'BRIGHTUP';",
+"DELETE FROM keybindings WHERE context = 'TV Playback' and action = 'COLORDOWN';",
+"DELETE FROM keybindings WHERE context = 'TV Playback' and action = 'COLORUP';",
+"DELETE FROM keybindings WHERE context = 'TV Playback' and action = 'HUEDOWN';",
+"DELETE FROM keybindings WHERE context = 'TV Playback' and action = 'HUEUP';",
+""
+};
+
+        performActualUpdate(updates, "1020", dbver);
+    }
+
+    if (dbver == "1020")
+    {
+        const QString updates[] = {
+"CREATE TABLE oldprogram ("
+"  oldtitle VARCHAR(128) NOT NULL PRIMARY KEY,"
+"  airdate TIMESTAMP NOT NULL"
+");",
+""
+};
+
+        performActualUpdate(updates, "1021", dbver);
+    }
+
+    if (dbver == "1021")
+    {
+        const QString updates[] = {
+"ALTER TABLE recorded ADD COLUMN commflagged int(10) unsigned NOT NULL default '0';",
+""
+};
+
+        performActualUpdate(updates, "1022", dbver);
+
+        VERBOSE(VB_ALL, QString("This could take a few minutes."));
+
+        QSqlDatabase *db = QSqlDatabase::database();
+
+        QString thequery, chanid, startts;
+        QSqlQuery query;
+        QDateTime recstartts;
+
+        thequery = QString("SELECT distinct recorded.chanid, "
+                           "recorded.starttime "
+                           "FROM recorded, recordedmarkup "
+                           "WHERE recorded.chanid = recordedmarkup.chanid AND "
+                           "recorded.starttime = recordedmarkup.starttime AND "
+                           "(type = 4 OR type = 5) ");
+        query = db->exec(thequery);
+
+        if (query.isActive() && query.numRowsAffected() > 0)
+        {
+            while (query.next())
+            {
+                chanid  = query.value(0).toString();
+                recstartts = query.value(1).toDateTime();
+                startts = recstartts.toString("yyyyMMddhhmm");
+                startts += "00";
+
+                thequery = QString("UPDATE recorded SET commflagged = 1, "
+                                   "starttime = %2 WHERE chanid = '%1' AND "
+                                   "starttime = '%3';").arg(chanid).arg(startts)
+                                                       .arg(startts);
+
+                db->exec(thequery);
+            }  
+        }
+    }
+
+    if (dbver == "1022")
+    {
+        const QString updates[] = {
+"ALTER TABLE dvb_sat CHANGE lnb_lof_switch lnb_lof_switch INTEGER DEFAULT 11700000;",
+""
+};
+        performActualUpdate(updates, "1023", dbver);
+    }
+
+    if (dbver == "1023")
+    {
+        const QString updates[] = {
+QString("ALTER TABLE videosource ADD COLUMN freqtable VARCHAR(16) NOT NULL DEFAULT 'default';"),
+""
+};
+        performActualUpdate(updates, "1024", dbver);
+    }
+
+    if (dbver == "1024")
+    {
+        const QString updates[] = {
+"ALTER TABLE dvb_sat CHANGE sourceid cardid INT;",
+"ALTER TABLE capturecard ADD COLUMN dvb_sat_type INT NOT NULL DEFAULT 0;",
+"ALTER TABLE dvb_channel ADD COLUMN pmtcache BLOB;",
+""
+};
+        performActualUpdate(updates, "1025", dbver);
+    }
+
+    if (dbver == "1025")
+    {
+        const QString updates[] = {
+"CREATE TABLE keyword ("
+"  phrase VARCHAR(128) NOT NULL PRIMARY KEY,"
+"  UNIQUE(phrase)"
+");",
+""
+};
+
+        performActualUpdate(updates, "1026", dbver);
+    }
+
+    if (dbver == "1026")
+    {
+        const QString updates[] = {
+"ALTER TABLE capturecard ADD COLUMN dvb_wait_for_seqstart INT NOT NULL DEFAULT 1;",
+"ALTER TABLE capturecard ADD COLUMN dvb_dmx_buf_size INT NOT NULL DEFAULT 8192;",
+"ALTER TABLE capturecard ADD COLUMN dvb_pkt_buf_size INT NOT NULL DEFAULT 8192;",
+""
+};
+        performActualUpdate(updates, "1027", dbver);
+    }
+
 }
 
 void InitializeDatabase(void)
