@@ -23,7 +23,7 @@ class DecoderBase
 
     void setExactSeeks(bool exact) { exactseeks = exact; }
     void setLiveTVMode(bool live) { livetv = live; }
-    void setWatchingRecording(bool mode) { watchingrecording = mode; }
+    virtual void setWatchingRecording(bool mode) { watchingrecording = mode; }
     void setRecorder(RemoteEncoder *recorder) { nvr_enc = recorder; }
 
     void setLowBuffers(void) { lowbuffers = true; }
@@ -40,14 +40,14 @@ class DecoderBase
     virtual void SetRawVideoState(bool state) = 0;
     virtual bool GetRawVideoState() = 0;
 
-    virtual void UpdateFrameNumber(long frame) = 0;
+    virtual long UpdateStoredFrameNum(long frame) = 0;
 
     virtual void SetPositionMap() = 0;
 
     virtual QString GetEncodingType(void) = 0;
 
-    virtual void InterruptDisplay(void) { }
-       
+    virtual void UpdateFramesPlayed(void) { };
+
   protected:
     NuppelVideoPlayer *m_parent;
     RingBuffer *ringBuffer;
