@@ -51,6 +51,9 @@ using namespace std;
 #include "dvbchannel.h"
 #include "dvbrecorder.h"
 #include "dvbdiseqc.h"
+#include "dvbcam.h"
+#include "dvbsections.h"
+
 
 DVBChannel::DVBChannel(int aCardNum, TVRec *parent)
            : ChannelBase(parent), cardnum(aCardNum)
@@ -59,13 +62,22 @@ DVBChannel::DVBChannel(int aCardNum, TVRec *parent)
     first_tune = true;
     isOpen = false;
     diseqc = NULL;
-    pauseStatusMonitor = false;
+    monitorRunning = false;
+    monitorClients = 0;
+
+    dvbcam = NULL;
+    dvbsct = NULL;
 
     pthread_mutex_init(&chan_opts.lock, NULL);
 }
 
 DVBChannel::~DVBChannel()
 {
+    if (dvbsct)
+        delete dvbsct;
+    if (dvbcam)
+        delete dvbcam;
+
     if (isOpen && (fd_frontend > 0))
         close(fd_frontend);
 }
@@ -76,7 +88,7 @@ bool DVBChannel::Open()
         return true;
 
     fd_frontend = open(dvbdevice(DVB_DEV_FRONTEND, cardnum),
-                        O_RDWR | O_NONBLOCK);
+                       O_RDWR | O_NONBLOCK);
     if(fd_frontend < 0)
     {
         ERRNO("Opening DVB frontend device failed.");
@@ -89,6 +101,13 @@ bool DVBChannel::Open()
         return false;
     }
 
+    GENERAL(QString("Using DVB card %1, with frontend %2.")
+            .arg(cardnum).arg(info.name));
+
+    dvbsct = new DVBSections(cardnum);
+    connect(this, SIGNAL(ChannelChanged(dvb_channel_t&)),
+            dvbsct, SLOT(ChannelChanged(dvb_channel_t&)));
+
     if (info.type == FE_QPSK)
     {
         if (ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_OFF) < 0)
@@ -103,11 +122,15 @@ bool DVBChannel::Open()
             diseqc->DiseqcReset();
         }
     }
-
-    GENERAL(QString("Using DVB card %1, with frontend %2.")
-            .arg(cardnum).arg(info.name));
-
-    pthread_create(&statusMonitorThread, NULL, StatusMonitorHelper, this);
+    
+    dvbcam = new DVBCam(cardnum);
+    if (dvbcam->Open())
+    {
+        connect(this, SIGNAL(ChannelChanged(dvb_channel_t&)),
+                dvbcam, SLOT(ChannelChanged(dvb_channel_t&)));
+        connect(dvbsct, SIGNAL(ChannelChanged(dvb_channel_t&, uint8_t*, int)),
+                dvbcam, SLOT(ChannelChanged(dvb_channel_t&, uint8_t*, int)));
+    }
 
     return isOpen = true;
 }
@@ -137,6 +160,8 @@ bool DVBChannel::SetChannelByString(const QString &chan)
         return false;
     }
 
+    GENERAL(QString("Successfully tuned to channel %1.").arg(chan));
+
     ChannelChanged(chan_opts);
 
     curchannelname = chan;
@@ -166,20 +191,20 @@ bool DVBChannel::GetChannelOptions(QString channum)
     QSqlDatabase    *db_conn;
     pthread_mutex_t db_lock;
 
-    int chnumber = channum.toInt();
-
-    if (!pParent->CheckChannel( (ChannelBase*)this, channum,
-                                db_conn, db_lock))
+    if (!pParent->CheckChannel((ChannelBase*)this, channum,
+                               db_conn, db_lock))
     {
         ERROR("Failed to verify channel integrity.");
         return false;
     }
 
+    dvbcam->SetDatabase(db_conn, db_lock);
+
     pthread_mutex_lock(&db_lock);
     MythContext::KickDatabase(db_conn);
 
     thequery = QString("SELECT chanid FROM channel WHERE channum='%1'")
-                       .arg(chnumber);
+                       .arg(channum);
     query = db_conn->exec(thequery);
 
     if (!query.isActive())
@@ -226,8 +251,34 @@ bool DVBChannel::GetChannelOptions(QString channum)
 
     if (query.numRowsAffected() <= 0)
     {
-        ERROR(QString("Could not find dvb tuning parameters for channel %1"
-                      " with id %2.").arg(channum).arg(chanid));
+        thequery += QString("SELECT satid FROM dvb_channel WHERE chanid='%1'")
+                    .arg(chanid);
+        query = db_conn->exec(thequery);
+
+        if (!query.isActive())
+            MythContext::DBError("GetChannelOptions - Channel Check", query);
+
+        if (query.numRowsAffected() <= 0)
+        {
+            ERROR(QString("Could not find dvb tuning parameters for channel %1"
+                          " with id %2.").arg(channum).arg(chanid));
+        }
+        else if (info.type == FE_QPSK)
+        {
+            int satid = query.value(0).toInt();
+            thequery += QString("SELECT * FROM dvb_sat WHERE satid='%1'")
+                        .arg(satid);
+            query = db_conn->exec(thequery);
+
+            if (!query.isActive())
+                MythContext::DBError("GetChannelOptions - Sat Check", query);
+
+            if (query.numRowsAffected() <= 0)
+                ERROR(QString("Channel %1 is configured with satid=%2, but this"
+                              " satellite does not exist, check your setup.")
+                              .arg(channum).arg(satid));
+        }
+
         pthread_mutex_unlock(&db_lock);
         return false;
     }
@@ -457,8 +508,8 @@ void DVBChannel::CheckOptions()
     else
         frequency = t.params.frequency;
 
-    if (frequency < info.frequency_min ||
-        frequency > info.frequency_max)
+    if ((info.frequency_min > 0 && info.frequency_max > 0) &&
+       (frequency < info.frequency_min || frequency > info.frequency_max))
         WARNING(QString("Your frequency setting (%1) is"
                         " out of range. (min/max:%2/%3)")
                         .arg(frequency)
@@ -555,6 +606,20 @@ bool DVBChannel::CheckModulation(fe_modulation_t& modulation)
     return false;
 }
 
+bool DVBChannel::FillFrontendStats(dvb_stats_t& stats)
+{
+    if (fd_frontend <= 0)
+        return false;
+
+    ioctl(fd_frontend, FE_READ_SNR, &stats.snr);
+    ioctl(fd_frontend, FE_READ_SIGNAL_STRENGTH, &stats.ss);
+    ioctl(fd_frontend, FE_READ_BER, &stats.ber);
+    ioctl(fd_frontend, FE_READ_UNCORRECTED_BLOCKS, &stats.ub);
+    ioctl(fd_frontend, FE_READ_STATUS, &stats.status);
+
+    return true;
+}
+
 void* DVBChannel::StatusMonitorHelper(void* self)
 {
     ((DVBChannel*)self)->StatusMonitorLoop();
@@ -563,31 +628,74 @@ void* DVBChannel::StatusMonitorHelper(void* self)
 
 void DVBChannel::StatusMonitorLoop()
 {
-    unsigned int snr=1, ss=1, ber=1, ub=1;
-    fe_status_t status;
+    dvb_stats_t stats;
 
-    while (true)
+    monitorRunning = true;
+    GENERAL("StatusMonitor Starting");
+
+    while (monitorClients > 0)
     {
-        while (pauseStatusMonitor)
-            usleep(500*1000);
+        if (!isOpen)
+        {
+            usleep(250*1000);
+            continue;
+        }
         
-        ioctl(fd_frontend, FE_READ_SNR, &snr);
-        emit StatusSignalToNoise(snr);
-        
-        ioctl(fd_frontend, FE_READ_SIGNAL_STRENGTH, &ss);
-        emit StatusSignalStrength(ss);
+        if (FillFrontendStats(stats))
+        {
+            emit Status(stats);
+            emit StatusSignalToNoise(stats.snr);
+            emit StatusSignalStrength(stats.ss);
+            emit StatusBitErrorRate(stats.ber);
+            emit StatusUncorrectedBlocks(stats.ub);
+
+            QString str = "";
+            if (stats.status & FE_TIMEDOUT)
+            {
+                str = "Timed out waiting for signal.";
+            }
+            else
+            {
+                if (stats.status & FE_HAS_SIGNAL)  str += "SIGNAL ";
+                if (stats.status & FE_HAS_CARRIER) str += "CARRIER ";
+                if (stats.status & FE_HAS_VITERBI) str += "VITERBI ";
+                if (stats.status & FE_HAS_SYNC) str += "SYNC ";
+                if (stats.status & FE_HAS_LOCK) str += "LOCK ";
+            }
+            emit Status(str);
+        }
 
-        ioctl(fd_frontend, FE_READ_BER, &ber);
-        emit StatusBitErrorRate(ber);
-        
-        ioctl(fd_frontend, FE_READ_UNCORRECTED_BLOCKS, &ub);
-        emit StatusUncorrectedBlocks(ub);
-        
-        ioctl(fd_frontend, FE_READ_STATUS, &status);
-        emit Status(status);
-        
         usleep(250*1000);
     }
+
+    monitorRunning = false;
+    GENERAL("StatusMonitor Stopping");
+}
+
+void DVBChannel::connectNotify(const char* signal)
+{
+    QString sig = signal;
+    if (sig == SIGNAL(Status(dvb_stats_t &)) ||
+        sig == SIGNAL(Status(QString)))
+    {
+        monitorClients++;
+        if (!monitorRunning)
+            pthread_create(&statusMonitorThread, NULL,
+                           StatusMonitorHelper, this);
+    }
+}
+
+void DVBChannel::disconnectNotify(const char* signal)
+{
+    QString sig = signal;
+    if (sig == SIGNAL(Status(dvb_stats_t &)) ||
+        sig == SIGNAL(Status(QString)))
+    {
+        monitorClients--;
+        if (monitorClients < 1)
+            while(monitorRunning)
+                usleep(1000);
+    }
 }
 
 /*****************************************************************************
