@@ -21,6 +21,8 @@ using namespace std;
 #include "dvbdiseqc.h"
 
 class TVRec;
+class DVBCam;
+class DVBSections;
 
 class DVBChannel : public QObject, public ChannelBase
 {
@@ -63,14 +65,22 @@ public:
                    const QString& hierarchy, dvb_tuning_t& p);
 
     void RecorderStarted();
+    bool FillFrontendStats(dvb_stats_t &stats);
 
 signals:
     void ChannelChanged(dvb_channel_t& chan);
-    void StatusSignalToNoise(int val);
-    void StatusSignalStrength(int val);
-    void StatusBitErrorRate(int val);
-    void StatusUncorrectedBlocks(int val);
-    void Status(fe_status_t status);
+
+    void Status(dvb_stats_t &stats);
+    void Status(QString val);
+
+    void StatusSignalToNoise(int);
+    void StatusSignalStrength(int);
+    void StatusBitErrorRate(int);
+    void StatusUncorrectedBlocks(int);
+
+protected:
+    void connectNotify(const char* signal);
+    void disconnectNotify(const char* signal);
 
 private:
     static void* StatusMonitorHelper(void*);
@@ -104,9 +114,13 @@ private:
     bool isOpen;
 
     pthread_t statusMonitorThread;
-    bool pauseStatusMonitor;
+    bool monitorRunning;
+    int monitorClients;
 
     DVBDiSEqC* diseqc;
+
+    DVBCam*      dvbcam;
+    DVBSections* dvbsct;
 };
 
 #endif
