@@ -29,7 +29,8 @@ using namespace std;
 #include "util.h"
 #include "remoteutil.h"
 
-QString RunProgramGuide(QString startchannel, bool thread, TV *player)
+QString RunProgramGuide(QString startchannel, bool thread, TV *player,
+                        bool allowsecondaryepg)
 {
     QString chanstr;
    
@@ -40,7 +41,7 @@ QString RunProgramGuide(QString startchannel, bool thread, TV *player)
         startchannel = "";
 
     GuideGrid *gg = new GuideGrid(gContext->GetMainWindow(), startchannel, 
-                                  player, "guidegrid");
+                                  player, allowsecondaryepg, "guidegrid");
 
     gg->Show();
 
@@ -71,17 +72,13 @@ QString RunProgramGuide(QString startchannel, bool thread, TV *player)
 }
 
 GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
-                     const char *name)
+                     bool allowsecondaryepg, const char *name)
          : MythDialog(parent, name)
 {
-    updateLock = new QMutex(true);
-    
     desiredDisplayChans = DISPLAY_CHANS = 6;
     DISPLAY_TIMES = 30;
     int maxchannel = 0;
     m_currentStartChannel = 0;
-    ignoreevents = false;
-    doProgramInfoUpdate = false;
 
     m_player = player;
     m_db = QSqlDatabase::database();
@@ -102,18 +99,28 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     theme = new XMLParse();
     theme->SetWMult(wmult);
     theme->SetHMult(hmult);
-    if (m_player && m_player->IsRunning())
-    {
+    if (m_player && m_player->IsRunning() && allowsecondaryepg)
         theme->LoadTheme(xmldata, "programguide-video");
-    }
     else 
-    {
         theme->LoadTheme(xmldata, "programguide");
-    }
+
     LoadWindow(xmldata);
 
+    if (m_player && m_player->IsRunning() && !allowsecondaryepg)
+        videoRect = QRect(0, 0, 1, 1);
+
     showFavorites = gContext->GetNumSetting("EPGShowFavorites", 0);
-    gridfilltype = gContext->GetNumSetting("EPGFillType", 6);
+    gridfilltype = gContext->GetNumSetting("EPGFillType", UIGuideType::Alpha);
+    if (gridfilltype < (int)UIGuideType::Alpha)
+    { // update old settings to new fill types
+        if (gridfilltype == 5)
+            gridfilltype = UIGuideType::Dense;
+        else
+            gridfilltype = UIGuideType::Alpha;
+            
+        gContext->SaveSetting("EPGFillType", gridfilltype);
+    }
+
     scrolltype = gContext->GetNumSetting("EPGScrollType", 1);
 
     LayerSet *container = NULL;
@@ -121,8 +128,16 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     if (container)
     {
         UIGuideType *type = (UIGuideType *)container->GetType("guidegrid");
-        if (type)
+        if (type) 
+        {
             type->SetFillType(gridfilltype);
+            type->SetShowCategoryColors(
+                   gContext->GetNumSetting("EPGShowCategoryColors", 1));
+            type->SetShowCategoryText(
+                   gContext->GetNumSetting("EPGShowCategoryText", 1));
+        }
+        if (gridfilltype == UIGuideType::Eco)    
+            container->SetDrawFontShadow(false);
     }
 
     timeformat = gContext->GetSetting("TimeFormat", "h:mm AP");
@@ -136,6 +151,15 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
         UITextType *type = (UITextType *)container->GetType("time");
         if (type)
             type->SetText(curTime);
+        if (gridfilltype == UIGuideType::Eco)    
+            container->SetDrawFontShadow(false);
+    }
+    
+    container = theme->GetSet("program_info");
+    if (container)
+    {
+        if (gridfilltype == UIGuideType::Eco)    
+            container->SetDrawFontShadow(false);
     }
 
     channelOrdering = gContext->GetSetting("ChannelOrdering", "channum + 0");
@@ -155,6 +179,8 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
         type = (UIBarType *)container->GetType("chans");
         if (type)
             type->SetSize(dNum);
+        if (gridfilltype == UIGuideType::Eco)    
+            container->SetDrawFontShadow(false);
     }
 
     container = theme->GetSet("timebar");
@@ -168,6 +194,8 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
         type = (UIBarType *)container->GetType("times");
         if (type)
             type->SetSize(dNum);
+        if (gridfilltype == UIGuideType::Eco)    
+            container->SetDrawFontShadow(false);
     }
     m_originalStartTime = QDateTime::currentDateTime();
 
@@ -212,7 +240,6 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     timeCheck->start(200);
 
     selectState = false;
-    showInfo = false;
 
     updateBackground();
 
@@ -248,7 +275,6 @@ GuideGrid::~GuideGrid()
     }
 
     delete theme;
-    delete updateLock;
 }
 
 void GuideGrid::keyPressEvent(QKeyEvent *e)
@@ -260,9 +286,6 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
     // setFocusPolicy(QWidget::ClickFocus) in constructor is important 
     // or keyRelease events will not be received after a refocus.
     
-    if (ignoreevents)
-        return;
-
     bool handled = false;
 
     QStringList actions;
@@ -275,9 +298,6 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
     if (e->key() != Key_Control)
         keyDown = true;    
 
-    ignoreevents = true;  //prevent customEvent from updating
-    updateLock->lock();    //wait if customEvent is updating 
-    
     if (e->state() == Qt::ControlButton)
     {
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
@@ -350,17 +370,6 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
 
     if (!handled)
         MythDialog::keyPressEvent(e);
-
-    if (doProgramInfoUpdate) 
-    {
-        fillRecordInfos();
-        fillProgramInfos();
-        update(fullRect);
-    }
-    
-    ignoreevents = false;
-
-    updateLock->unlock();
 }
 
 void GuideGrid::keyReleaseEvent(QKeyEvent *e)
@@ -406,7 +415,7 @@ void GuideGrid::LoadWindow(QDomElement &element)
             else
             {
                 cerr << "Unknown element: " << e.tagName() << endl;
-                exit(0);
+                continue;
             }
         }
     }
@@ -469,7 +478,7 @@ void GuideGrid::timeout()
                                   videoRect.width(), videoRect.height());
     }
 
-    update(curInfoRect);
+    repaint(curInfoRect, false);
 }
 
 void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
@@ -619,9 +628,10 @@ void GuideGrid::fillProgramInfos(void)
         type = (UIGuideType *)container->GetType("guidegrid");
         if (type)
         {
-            type->ResetData();
             type->SetWindow(this);
             type->SetScreenLocation(programRect.topLeft());
+            type->SetNumRows(DISPLAY_CHANS);
+            type->ResetData();
         }
     }
 
@@ -629,8 +639,6 @@ void GuideGrid::fillProgramInfos(void)
     {
         fillProgramRowInfos(y);
     }
-    
-    doProgramInfoUpdate = false;        
 }
 
 void GuideGrid::fillProgramRowInfos(unsigned int row)
@@ -640,6 +648,9 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
     UIGuideType *type = NULL;
     if (container)
         type = (UIGuideType *)container->GetType("guidegrid");
+
+    if (type)
+        type->ResetRow(row);
  
     QPtrList<ProgramInfo> *proglist;
     ProgramInfo *program;
@@ -838,12 +849,9 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
                 else
                     recStat = 2;
 
-                type->SetProgramInfo(row + 1, cnt, tempRect, proginfo->title,
+                type->SetProgramInfo(row, cnt, tempRect, proginfo->title,
                                      proginfo->category, arrow, recFlag, 
-                                     recStat);
-
-                if (isCurrent == true)
-                    type->SetCurrentArea(tempRect);
+                                     recStat, isCurrent);
 
                 cnt++;
             }
@@ -862,17 +870,8 @@ void GuideGrid::customEvent(QCustomEvent *e)
 
         if (message == "SCHEDULE_CHANGE")
         {
-            if (ignoreevents)
-            {
-                doProgramInfoUpdate = true;
-                return;
-            }    
-            updateLock->lock();
-            ignoreevents = true; 
             fillRecordInfos();
             fillProgramInfos();
-            ignoreevents = false;
-            updateLock->unlock();
             update(fullRect);
         }
     }
@@ -880,26 +879,25 @@ void GuideGrid::customEvent(QCustomEvent *e)
 
 void GuideGrid::paintEvent(QPaintEvent *e)
 {
+    qApp->lock();
+
     QRect r = e->rect();
     QPainter p(this);
 
-    if (!showInfo)
-    {
-        updateLock->lock(); 
-        if (r.intersects(infoRect))
-            paintInfo(&p);
-        if (r.intersects(dateRect))
-            paintDate(&p);
-        if (r.intersects(channelRect))
-            paintChannels(&p);
-        if (r.intersects(timeRect))
-            paintTimes(&p);
-        if (r.intersects(programRect))
-            paintPrograms(&p);
-        if (r.intersects(curInfoRect))
-            paintCurrentInfo(&p);
-        updateLock->unlock();
-    }
+    if (r.intersects(infoRect))
+        paintInfo(&p);
+    if (r.intersects(dateRect))
+        paintDate(&p);
+    if (r.intersects(channelRect))
+        paintChannels(&p);
+    if (r.intersects(timeRect))
+        paintTimes(&p);
+    if (r.intersects(programRect))
+        paintPrograms(&p);
+    if (r.intersects(curInfoRect))
+        paintCurrentInfo(&p);
+
+    qApp->unlock();
 }
 
 void GuideGrid::paintDate(QPainter *p)
@@ -1101,7 +1099,7 @@ void GuideGrid::paintInfo(QPainter *p)
     if (!pginfo)
         return;
 
-    QMap<QString, QString> regexpMap;
+    QMap<QString, QString> infoMap;
     QRect pr = infoRect;
     QPixmap pix(pr.size());
     pix.fill(this, pr.topLeft());
@@ -1115,13 +1113,13 @@ void GuideGrid::paintInfo(QPainter *p)
 
     ChannelInfo *chinfo = &(m_channelInfos[chanNum]);
 
-    pginfo->ToMap(m_db, regexpMap);
+    pginfo->ToMap(m_db, infoMap);
 
     LayerSet *container = theme->GetSet("program_info");
     if (container)
     {
         container->ClearAllText();
-        container->SetTextByRegexp(regexpMap);
+        container->SetText(infoMap);
 
         UITextType *type;
 
@@ -1213,7 +1211,7 @@ void GuideGrid::toggleChannelFavorite()
         if (DISPLAY_CHANS > maxchannel)
             DISPLAY_CHANS = maxchannel;
 
-        update(channelRect);
+        repaint(channelRect, false);
     }
 }
 
@@ -1223,7 +1221,7 @@ void GuideGrid::cursorLeft()
     
     if (!test)
     {
-        emit scrollLeft();
+        scrollLeft();
         return;
     }
 
@@ -1233,7 +1231,7 @@ void GuideGrid::cursorLeft()
     if (m_currentCol < 0)
     {
         m_currentCol = 0;
-        emit scrollLeft();
+        scrollLeft();
     }
     else
     {
@@ -1250,7 +1248,7 @@ void GuideGrid::cursorRight()
 
     if (!test)
     {
-        emit scrollRight();
+        scrollRight();
         return;
     }
 
@@ -1262,7 +1260,7 @@ void GuideGrid::cursorRight()
     if (m_currentCol > DISPLAY_TIMES - 1)
     {
         m_currentCol = DISPLAY_TIMES - 1;
-        emit scrollRight();
+        scrollRight();
     }
     else
     {
@@ -1282,7 +1280,7 @@ void GuideGrid::cursorDown()
         if (m_currentRow > DISPLAY_CHANS - 1)
         {
             m_currentRow = DISPLAY_CHANS - 1;
-            emit scrollDown();
+            scrollDown();
         }
         else
         {
@@ -1292,7 +1290,7 @@ void GuideGrid::cursorDown()
         }
     }
     else
-        emit scrollDown();
+        scrollDown();
 }
 
 void GuideGrid::cursorUp()
@@ -1304,7 +1302,7 @@ void GuideGrid::cursorUp()
         if (m_currentRow < 0)
         {
             m_currentRow = 0;
-            emit scrollUp();
+            scrollUp();
         }
         else
         {
@@ -1314,7 +1312,7 @@ void GuideGrid::cursorUp()
         }
     }
     else
-        emit scrollUp();
+        scrollUp();
 }
 
 void GuideGrid::scrollLeft()
@@ -1490,12 +1488,8 @@ void GuideGrid::pageUp()
  
 void GuideGrid::showProgFinder()
 {
-    showInfo = 1;
-
     RunProgramFind();
 
-    showInfo = 0;
-
     setActiveWindow();
     setFocus();
 
@@ -1515,7 +1509,7 @@ void GuideGrid::enter()
 
     unsetCursor();
     selectState = 1;
-    emit accept();
+    accept();
 }
 
 void GuideGrid::escape()
@@ -1528,7 +1522,7 @@ void GuideGrid::escape()
     }
 
     unsetCursor();
-    emit accept();
+    accept();
 }
 
 void GuideGrid::quickRecord()
@@ -1545,8 +1539,8 @@ void GuideGrid::quickRecord()
 
     fillRecordInfos();
     fillProgramInfos();
-    update(programRect);
-    update(infoRect);
+    repaint(programRect, false);
+    repaint(infoRect, false);
 }
 
 void GuideGrid::editRecording()
@@ -1559,21 +1553,21 @@ void GuideGrid::editRecording()
     if (pginfo->title == unknownTitle)
         return;
 
-    showInfo = 1;
-
     FocusPolicy storeFocus = focusPolicy();
     setFocusPolicy(QWidget::NoFocus);
-    pginfo->EditRecording(m_db);
-    setFocusPolicy(storeFocus);
 
-    showInfo = 0;
+    ProgramInfo *temppginfo = new ProgramInfo(*pginfo);
+    temppginfo->EditRecording(m_db);
+    delete temppginfo;
+
+    setFocusPolicy(storeFocus);
 
     setActiveWindow();
     setFocus();
 
     fillRecordInfos();
     fillProgramInfos();
-    update(fullRect);
+    repaint(fullRect, false);
 }
 
 void GuideGrid::editScheduled()
@@ -1586,21 +1580,21 @@ void GuideGrid::editScheduled()
     if (pginfo->title == unknownTitle)
         return;
 
-    showInfo = 1;
-
     FocusPolicy storeFocus = focusPolicy();
     setFocusPolicy(QWidget::NoFocus);
-    pginfo->EditScheduled(m_db);
-    setFocusPolicy(storeFocus);
 
-    showInfo = 0;
+    ProgramInfo *temppginfo = new ProgramInfo(*pginfo);
+    temppginfo->EditScheduled(m_db);
+    delete temppginfo;
+
+    setFocusPolicy(storeFocus);
 
     setActiveWindow();
     setFocus();
 
     fillRecordInfos();
     fillProgramInfos();
-    update(fullRect);
+    repaint(fullRect, false);
 }
 
 void GuideGrid::channelUpdate(void)
