@@ -28,7 +28,7 @@ class QWidget;
 
 // Use this function to instantiate a guidegrid instance.
 QString RunProgramGuide(QString startchannel, bool thread = false, 
-                        TV *player = NULL);
+                        TV *player = NULL, bool allowsecondaryepg = true);
 
 
 class GuideGrid : public MythDialog
@@ -36,7 +36,7 @@ class GuideGrid : public MythDialog
     Q_OBJECT
   public:
     GuideGrid(MythMainWindow *parent, const QString &channel, TV *player = NULL,
-              const char *name = 0);
+              bool allowsecondaryepg = true, const char *name = 0);
    ~GuideGrid();
 
     QString getLastChannel(void);
@@ -139,7 +139,6 @@ class GuideGrid : public MythDialog
     int m_currentRow;
     int m_currentCol;
 
-    bool showInfo;
     bool selectState;
     bool showFavorites;
     bool displaychannum;
@@ -164,11 +163,7 @@ class GuideGrid : public MythDialog
 
     QSqlDatabase *m_db;
 
-    bool ignoreevents;
-    bool doProgramInfoUpdate;
     bool keyDown;
-    
-    QMutex *updateLock;
 };
 
 #endif
