@@ -45,15 +45,15 @@
 
 #include <iostream>
 #include <pthread.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
-#include <time.h>
+#include <ctime>
 #include "videodev_myth.h"
 
 using namespace std;
@@ -69,6 +69,7 @@ extern "C" {
 #include "../libavformat/mpegts.h"
 }
 
+// n.b. at 19 Mbits/sec, default buffer size is approx. 1/10 sec.
 #define PACKETS (HD_BUFFER_SIZE / 188)
 
 #define SYNC_BYTE 0x47
@@ -86,12 +87,16 @@ HDTVRecorder::HDTVRecorder()
     recording = false;
 
     framesWritten = 0;
+    framesSeen = 0;
 
     chanfd = -1; 
 
     keyframedist = 30;
     gopset = false;
-    pict_start_is_gop = false;
+    m_in_mpg_headers = false;
+    seq_start_is_gop = false;
+    seq_count = 0;
+    m_header_sync = 0;
 
     firstgoppos = 0;
 
@@ -120,6 +125,21 @@ void HDTVRecorder::SetOption(const QString &opt, int value)
     (void)value;
 }
 
+void HDTVRecorder::SetOptionsFromProfile(RecordingProfile *profile,
+                                         const QString &videodev,
+                                         const QString &audiodev,
+                                         const QString &vbidev, int ispip)
+{
+    (void)audiodev;
+    (void)vbidev;
+    (void)profile;
+    (void)ispip;
+
+    SetOption("videodevice", videodev);
+    SetOption("tvformat", gContext->GetSetting("TVFormat"));
+    SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
+}
+
 void HDTVRecorder::StartRecording(void)
 {
     uint8_t * buf;
@@ -259,7 +279,7 @@ void HDTVRecorder::FinishRecording(void)
     {
         pthread_mutex_lock(db_lock);
         MythContext::KickDatabase(db_conn);
-        curRecording->SetPositionMap(positionMap, MARK_GOP_START, db_conn);
+        curRecording->SetPositionMap(positionMap, MARK_GOP_BYFRAME, db_conn);
         pthread_mutex_unlock(db_lock);
     }
 }
@@ -304,57 +324,62 @@ void HDTVRecorder::FindKeyframes(const unsigned char *buffer,
         if (payload_unit_start_indicator)
         {
             // packet contains start of PES packet
-
+            m_in_mpg_headers = true;
+            m_header_sync = 0;
+        }
+        if (m_in_mpg_headers) {
             // Scan for PES header codes; specifically picture_start
             // and group_start (of_pictures).  These should be within 
             // this first TS packet of the PES packet.
             //   00 00 01 00: picture_start_code
             //   00 00 01 B8: group_start_code
             //   (there are others that we don't care about)
-            int sync = 0;
             payload_size = packet_end_pos - pkt_start;
-            for (int i=pos; i < payload_size+pos; i++)
+            for (int i=pkt_start; i < packet_end_pos && m_in_mpg_headers; i++)
             {
                 char k = buffer[i];
-                switch (sync)
+                switch (m_header_sync)
                 {
                 case 0:
                     if (k == 0x00)
-                        sync = 1;
+                        m_header_sync = 1;
                     break;
                 case 1:
                     if (k == 0x00)
-                        sync = 2;
+                        m_header_sync = 2;
                     else
-                        sync = 0;
+                        m_header_sync = 0;
                     break;
                 case 2:
                     if (k != 0x01)
                     {
                         if (k != 0x00)
-                            sync = 0;
+                            m_header_sync = 0;
                     }
                     else 
                     {
                         if (buffer[i+1] == 0x00) // picture_start
                         {
-                            framesWritten++;
-                            if (framesWritten >= 30 && firstgoppos <= 0)
+                            if (gopset || firstgoppos > 0)
+                                framesWritten++;
+                            framesSeen++;
+                        }
+
+                        if (buffer[i+1] == 0xB3)
+                        {
+                            seq_count ++;
+                            if (seq_count > 1 && !gopset) 
                             {
-                                // seen 30 frames with no GOP; assume that
-                                // each GOP only contains one I-frame, and
-                                // treat the I-frame as the beginning of the
-                                // GOP.
-                                pict_start_is_gop = true;
+                                seq_start_is_gop = true;
                             }
                         }
                         if (buffer[i+1] == 0xB8 || 
-                            (pict_start_is_gop && buffer[i+1] == 0x00))
+                            (seq_start_is_gop && buffer[i+1] == 0xB3))
                         {
                             // group_of_pictures
                             int frameNum = framesWritten - 1;
                             
-                            if (!gopset && frameNum > 0)
+                            if (!gopset && framesSeen > 0)
                             {
                                 if (firstgoppos > 0)
                                 {
@@ -366,15 +391,14 @@ void HDTVRecorder::FindKeyframes(const unsigned char *buffer,
                                     gopset = true;
                                 }
                                 else
-                                    firstgoppos = frameNum;
+                                    firstgoppos = (frameNum > 0) ? 
+                                        frameNum : 1;
                             }
                             
                             long long startpos = 
                                 ringBuffer->GetFileWritePosition();
 
-                            long long keyCount = frameNum/keyframedist;
-                            
-                            positionMap[keyCount] = startpos;
+                            positionMap[frameNum] = startpos;
                             
                             if (curRecording && db_lock && db_conn &&
                                 ((positionMap.size() % 30) == 0))
@@ -382,15 +406,21 @@ void HDTVRecorder::FindKeyframes(const unsigned char *buffer,
                                 pthread_mutex_lock(db_lock);
                                 MythContext::KickDatabase(db_conn);
                                 curRecording->SetPositionMap(
-                                    positionMap, MARK_GOP_START,
+                                    positionMap, MARK_GOP_BYFRAME,
                                     db_conn, prev_gop_save_pos, 
-                                    keyCount);
+                                    frameNum);
                                 pthread_mutex_unlock(db_lock);
-                                prev_gop_save_pos = keyCount + 1;
+                                prev_gop_save_pos = frameNum + 1;
                             }
                             
                         }
-                        sync = 0;
+                        if (buffer[i+1] >= 0x01 && buffer[i+1] <= 0xAF)
+                        {
+                            // video slice ... end of "interesting"
+                            // headers
+                            m_in_mpg_headers = false;
+                        }
+                        m_header_sync = 0;
                     }
                     break;
                 }
@@ -653,7 +683,7 @@ int HDTVRecorder::ProcessData(unsigned char *buffer, int len)
               5 bits unused flags
             */
             unsigned char adaptation_length;
-            adaptation_length = buffer[pos];
+            adaptation_length = buffer[pos++];
             pos += adaptation_length;
         }
 
@@ -782,7 +812,12 @@ void HDTVRecorder::StopRecording(void)
 void HDTVRecorder::Reset(void)
 {
     framesWritten = 0;
+    framesSeen = 0;
     gopset = false;
+    m_in_mpg_headers = false;
+    seq_start_is_gop = false;
+    seq_count = 0;
+    m_header_sync = 0;
     firstgoppos = 0;
     ts_packets = 0;
     pmt_pid = -1;
