@@ -18,7 +18,7 @@ class ChannelInfo
     QString chanstr;
     int chanid;
     int favid;
-    QString rank;
+    QString recpriority;
 
     QPixmap *icon;
 };
