@@ -1,15 +1,16 @@
 #include <iostream>
 #include <pthread.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
-#include <time.h>
+#include <ctime>
 #include "videodev_myth.h"
+
 extern "C" {
 #include <inttypes.h>
 #include "ivtv-ext-api.h"
@@ -21,6 +22,7 @@ using namespace std;
 #include "RingBuffer.h"
 #include "mythcontext.h"
 #include "programinfo.h"
+#include "recordingprofile.h"
 
 extern "C" {
 #include "../libavcodec/avcodec.h"
@@ -168,6 +170,43 @@ void MpegRecorder::SetOption(const QString &opt, const QString &value)
 
 }
 
+void MpegRecorder::SetOptionsFromProfile(RecordingProfile *profile, 
+                                         const QString &videodev, 
+                                         const QString &audiodev,
+                                         const QString &vbidev, int ispip)
+{
+    (void)audiodev;
+    (void)vbidev;
+
+    SetOption("videodevice", videodev);
+    SetOption("tvformat", gContext->GetSetting("TVFormat"));
+    SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
+
+    SetIntOption(profile, "mpeg2bitrate");
+    SetIntOption(profile, "mpeg2maxbitrate");
+    SetOption("mpeg2streamtype",
+              profile->byName("mpeg2streamtype")->getValue());
+    SetOption("mpeg2aspectratio",
+              profile->byName("mpeg2aspectratio")->getValue());
+
+    SetIntOption(profile, "samplerate");
+    SetOption("mpeg2audtype", profile->byName("mpeg2audtype")->getValue());
+    SetIntOption(profile, "mpeg2audbitratel1");
+    SetIntOption(profile, "mpeg2audbitratel2");
+    SetIntOption(profile, "mpeg2audvolume");
+
+    SetIntOption(profile, "width");
+    SetIntOption(profile, "height");
+
+    if (ispip)
+    {
+        SetOption("width", 320);
+        SetOption("height", 240);
+        SetOption("mpeg2bitrate", 1000);
+        SetOption("mpeg2maxbitrate", 2000);
+    }
+}
+
 void MpegRecorder::StartRecording(void)
 {
     chanfd = open(videodevice.ascii(), O_RDWR);
@@ -247,6 +286,9 @@ void MpegRecorder::StartRecording(void)
     ivtvcodec.bitrate = bitrate * 1000;
     ivtvcodec.bitrate_peak = maxbitrate * 1000;
 
+    if (ivtvcodec.bitrate > ivtvcodec.bitrate_peak)
+        ivtvcodec.bitrate = ivtvcodec.bitrate_peak;
+
     // framerate (1 = 25fps, 0 = 30fps)
     ivtvcodec.framerate = (!ntsc);
     ivtvcodec.stream_type = streamtype;
