@@ -76,6 +76,11 @@ OSDSet::OSDSet(const OSDSet &other)
     m_xoff = other.m_xoff;
     m_yoff = other.m_yoff;
     m_allowfade = other.m_allowfade;
+    m_wantsupdates = other.m_wantsupdates;
+    m_lastupdate = other.m_lastupdate;
+    m_needsupdate = other.m_needsupdate;
+    m_draweveryframe = other.m_draweveryframe;
+
     currentOSDFunctionalType = 0;
 
     allTypes = new vector<OSDType *>;
@@ -229,7 +234,7 @@ void OSDSet::ClearAllText(void)
     }
 }
 
-void OSDSet::SetTextByRegexp(QMap<QString, QString> &regexpMap)
+void OSDSet::SetText(QMap<QString, QString> &infoMap)
 {
     vector<OSDType *>::iterator iter = allTypes->begin();
     for (; iter != allTypes->end(); iter++)
@@ -237,17 +242,17 @@ void OSDSet::SetTextByRegexp(QMap<QString, QString> &regexpMap)
         OSDType *type = (*iter);
         if (OSDTypeText *item = dynamic_cast<OSDTypeText *>(type))
         {
-            QMap<QString, QString>::Iterator riter = regexpMap.begin();
+            QMap<QString, QString>::Iterator riter = infoMap.begin();
             QString new_text = item->GetDefaultText();
             QString full_regex;
 
             if ((new_text == "") &&
-                (regexpMap.contains(item->Name())))
+                (infoMap.contains(item->Name())))
             {
-                new_text = regexpMap[item->Name()];
+                new_text = infoMap[item->Name()];
             }
             else if (new_text.contains(QRegExp("%.*%")))
-                for (; riter != regexpMap.end(); riter++)
+                for (; riter != infoMap.end(); riter++)
                 {
                    full_regex = "%" + riter.key().upper() + 
                      "(\\|([^%|]*))?" + "(\\|([^%|]*))?" + "(\\|([^%]*))?%";
@@ -504,7 +509,7 @@ void OSDTypeText::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
         tmp_msg.replace(QRegExp("%BR%"), "\n");
         tmp_msg.replace(QRegExp("\n")," \n ");
 
-        QStringList wordlist = QStringList::split(" ", tmp_msg.ascii());
+        QStringList wordlist = QStringList::split(" ", tmp_msg);
         int length = 0;
         int lines = 0;
 
@@ -1827,8 +1832,8 @@ void OSDTypeCC::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
             }
             else
             {
-                x = (cc->x + 1) * displaywidth / 34 + xoffset;
-                y = cc->y * displayheight / 18 + yoffset;
+                x = (cc->x + 3) * displaywidth / 36 + xoffset;
+                y = cc->y * displayheight / 19 + yoffset;
             }
 
             int maxx = x + textlength;
