@@ -37,7 +37,7 @@ ProgramInfo::ProgramInfo(void)
     conflicting = false;
     recording = false;
     override = 0;
-    norecord = nrUnknown;
+    recstatus = rsUnknown;
     recordid = 0;
     rectype = kNotRecording;
     recdups = kRecordDupsNever;
@@ -77,7 +77,7 @@ ProgramInfo::ProgramInfo(const ProgramInfo &other)
     conflicting = other.conflicting;
     recording = other.recording;
     override = other.override;
-    norecord = other.norecord;
+    recstatus = other.recstatus;
     recordid = other.recordid;
     rectype = other.rectype;
     recdups = other.recdups;
@@ -87,19 +87,66 @@ ProgramInfo::ProgramInfo(const ProgramInfo &other)
     cardid = other.cardid;
     schedulerid = other.schedulerid;
     recpriority = other.recpriority;
+    programflags = other.programflags;
 
     repeat = other.repeat;
 
     record = NULL;
 }
 
+ProgramInfo &ProgramInfo::operator=(const ProgramInfo &other)
+{
+    if (record)
+        delete record;
+
+    title = other.title;
+    subtitle = other.subtitle;
+    description = other.description;
+    category = other.category;
+    chanid = other.chanid;
+    chanstr = other.chanstr;
+    chansign = other.chansign;
+    channame = other.channame;
+    pathname = other.pathname;
+    filesize = other.filesize;
+    hostname = other.hostname;
+
+    startts = other.startts;
+    endts = other.endts;
+    recstartts = other.recstartts;
+    recendts = other.recendts;
+    spread = other.spread;
+    startCol = other.startCol;
+
+    conflicting = other.conflicting;
+    recording = other.recording;
+    override = other.override;
+    recstatus = other.recstatus;
+    recordid = other.recordid;
+    rectype = other.rectype;
+    recdups = other.recdups;
+
+    sourceid = other.sourceid;
+    inputid = other.inputid;
+    cardid = other.cardid;
+    schedulerid = other.schedulerid;
+    recpriority = other.recpriority;
+    programflags = other.programflags;
+
+    repeat = other.repeat;
+
+    record = NULL;
+
+    return *this;
+}
+
 ProgramInfo::~ProgramInfo() 
 {
     if (record != NULL)
         delete record;
 }
 
-QString ProgramInfo::MakeUniqueKey(void)
+QString ProgramInfo::MakeUniqueKey(void) const
 {
     return title + ":" + chanid + ":" + startts.toString(Qt::ISODate);
 }
@@ -126,51 +173,60 @@ void ProgramInfo::ToStringList(QStringList &list)
     list << QString::number(cardid);
     list << QString::number(inputid);
     list << ((recpriority != "") ? recpriority : QString(" "));
-    list << QString::number(norecord);
+    list << QString::number(recstatus);
     list << QString::number(recordid);
     list << QString::number(rectype);
     list << QString::number(recdups);
     list << recstartts.toString(Qt::ISODate);
     list << recendts.toString(Qt::ISODate);
     list << QString::number(repeat);
+    list << QString::number(programflags);
 }
 
 void ProgramInfo::FromStringList(QStringList &list, int offset)
 {
-    if (offset + NUMPROGRAMLINES > (int)list.size())
+    QStringList::iterator it = list.at(offset);
+    FromStringList(list, it);
+}
+
+void ProgramInfo::FromStringList(QStringList &list, QStringList::iterator &it)
+{
+    QStringList::iterator checkit = it;
+    checkit += (NUMPROGRAMLINES - 1);
+    if (checkit == list.end())
     {
-        cerr << "offset is: " << offset << " but size is " << list.size() 
-             << endl;
+        cerr << "Not enough items in list for ProgramInfo object" << endl;
         return;
     }
 
-    title = list[offset];
-    subtitle = list[offset + 1];
-    description = list[offset + 2];
-    category = list[offset + 3];
-    chanid = list[offset + 4];
-    chanstr = list[offset + 5];
-    chansign = list[offset + 6];
-    channame = list[offset + 7];
-    pathname = list[offset + 8];
-    filesize = decodeLongLong(list, offset + 9);
-    startts = QDateTime::fromString(list[offset + 11], Qt::ISODate);
-    endts = QDateTime::fromString(list[offset + 12], Qt::ISODate);
-    conflicting = list[offset + 13].toInt();
-    recording = list[offset + 14].toInt();
-    override = list[offset + 15].toInt();
-    hostname = list[offset + 16];
-    sourceid = list[offset + 17].toInt();
-    cardid = list[offset + 18].toInt();
-    inputid = list[offset + 19].toInt();
-    recpriority = list[offset + 20];
-    norecord = NoRecordType(list[offset + 21].toInt());
-    recordid = list[offset + 22].toInt();
-    rectype = RecordingType(list[offset + 23].toInt());
-    recdups = RecordingDupsType(list[offset + 24].toInt());
-    recstartts = QDateTime::fromString(list[offset + 25], Qt::ISODate);
-    recendts = QDateTime::fromString(list[offset + 26], Qt::ISODate);
-    repeat = list[offset + 27].toInt();
+    title = *(it++);
+    subtitle = *(it++);
+    description = *(it++);
+    category = *(it++);
+    chanid = *(it++);
+    chanstr = *(it++);
+    chansign = *(it++);
+    channame = *(it++);
+    pathname = *(it++);
+    filesize = decodeLongLong(list, it);
+    startts = QDateTime::fromString(*(it++), Qt::ISODate);
+    endts = QDateTime::fromString(*(it++), Qt::ISODate);
+    conflicting = (*(it++)).toInt();
+    recording = (*(it++)).toInt();
+    override = (*(it++)).toInt();
+    hostname = *(it++);
+    sourceid = (*(it++)).toInt();
+    cardid = (*(it++)).toInt();
+    inputid = (*(it++)).toInt();
+    recpriority = *(it++);
+    recstatus = RecStatusType((*(it++)).toInt());
+    recordid = (*(it++)).toInt();
+    rectype = RecordingType((*(it++)).toInt());
+    recdups = RecordingDupsType((*(it++)).toInt());
+    recstartts = QDateTime::fromString(*(it++), Qt::ISODate);
+    recendts = QDateTime::fromString(*(it++), Qt::ISODate);
+    repeat = (*(it++)).toInt();
+    programflags = (*(it++)).toInt();
 
     if (title == " ")
         title = "";
@@ -243,12 +299,18 @@ void ProgramInfo::ToMap(QSqlDatabase *db, QMap<QString, QString> &progMap)
     length.sprintf("%d:%02d", hours, minutes);
     progMap["lentime"] = length;
 
-    progMap["rec_type"] = RecordingChar();
-    progMap["rec_str"] = RecordingText();
+    progMap["rec_type"] = RecTypeChar();
+    progMap["rec_str"] = RecTypeText();
+    if (rectype != kNotRecording)
+    {
+        progMap["rec_str"] += " - ";
+        progMap["rec_str"] += RecStatusText();
+    }
     progMap["recordingstatus"] = progMap["rec_str"];
     progMap["type"] = progMap["rec_str"];
 
     progMap["recpriority"] = recpriority;
+    progMap["programflags"] = programflags;
 
     progMap["timedate"] = recstartts.date().toString(dateFormat) + ", " +
                           recstartts.time().toString(timeFormat) + " - " +
@@ -273,53 +335,7 @@ void ProgramInfo::ToMap(QSqlDatabase *db, QMap<QString, QString> &progMap)
         if (query.next())
             progMap["iconpath"] = query.value(0).toString();
 
-    QString recstatus = "";
-    if (recording)
-    {
-        recstatus = QObject::tr("Recording on cardid: ") +
-                    QString::number(cardid);
-    }
-    else
-    {
-        recstatus = QObject::tr("Not Recording: ");
-        switch (norecord)
-        {
-            case nrManualOverride:
-                    recstatus += QObject::tr("Manual Override");
-                    break;
-            case nrPreviousRecording:
-                    recstatus += QObject::tr("Previously Recorded");
-                    break;
-            case nrCurrentRecording:
-                    recstatus += QObject::tr("Currently Recording");
-                    break;
-            case nrOtherShowing:
-                    recstatus += QObject::tr("Other Showing");
-                    break;
-            case nrTooManyRecordings:
-                    recstatus += QObject::tr("Too Many Recordings");
-                    break;
-            case nrDontRecordList:
-                    recstatus += QObject::tr("Don't Record List");
-                    break;
-            case nrLowerRecPriority:
-                    recstatus += QObject::tr("Lower Rec Priority");
-                    break;
-            case nrManualConflict:
-                    recstatus += QObject::tr("Manual Conflict");
-                    break;
-            case nrAutoConflict:
-                    recstatus += QObject::tr("Auto Conflict");
-                    break;
-            case nrOverlap:
-                    recstatus += QObject::tr("Overlap");
-                    break;
-            case nrUnknown:
-                    recstatus += QObject::tr("Reason Unknown");
-                    break;
-        }
-    }
-    progMap["RECSTATUS"] = recstatus;
+    progMap["RECSTATUS"] = RecStatusText();
 
     if (repeat)
         progMap["REPEAT"] = QObject::tr("Repeat");
@@ -338,9 +354,10 @@ void ProgramInfo::GetProgramListByQuery(QSqlDatabase *db,
 {
     QString thequery;
 
-    thequery = QString("SELECT channel.chanid,starttime,endtime,title,subtitle,"
-                       "description,category,channel.channum,channel.callsign, "
-                       "channel.name,previouslyshown FROM program,channel ")
+    thequery = QString("SELECT channel.chanid,starttime,endtime,title,"
+                       "subtitle,description,category,channel.channum,"
+                       "channel.callsign,channel.name,previouslyshown "
+                       "FROM program,channel ")
                        + where;
 
     QSqlQuery query = db->exec(thequery);
@@ -391,8 +408,8 @@ void ProgramInfo::GetProgramRangeDateTime(QSqlDatabase *db,
     QString where;
 
     where = QString("WHERE program.chanid = %1 AND endtime >= %1 AND "
-                   "starttime <= %3 AND program.chanid = channel.chanid "
-                   "ORDER BY starttime;")
+                    "starttime <= %3 AND program.chanid = channel.chanid "
+                    "ORDER BY starttime;")
                     .arg(channel).arg(ltime).arg(rtime);
 
     GetProgramListByQuery(db, proglist, where);
@@ -477,13 +494,13 @@ ProgramInfo *ProgramInfo::GetProgramFromRecorded(QSqlDatabase *db,
     QString thequery;
    
     thequery = QString("SELECT recorded.chanid,starttime,endtime,title, "
-                               "subtitle,description,channel.channum, "
-                               "channel.callsign,channel.name "
-                           "FROM recorded "
-                           "LEFT JOIN channel "
-                               "ON recorded.chanid = channel.chanid "
-                           "WHERE recorded.chanid = %1 "
-                               "AND starttime = %2;")
+                       "subtitle,description,channel.channum, "
+                       "channel.callsign,channel.name "
+                       "FROM recorded "
+                       "LEFT JOIN channel "
+                       "ON recorded.chanid = channel.chanid "
+                       "WHERE recorded.chanid = %1 "
+                       "AND starttime = %2;")
                        .arg(channel).arg(starttime);
 
     QSqlQuery query = db->exec(thequery);
@@ -525,6 +542,8 @@ ProgramInfo *ProgramInfo::GetProgramFromRecorded(QSqlDatabase *db,
         if (proginfo->channame == QString::null)
             proginfo->channame = "";
 
+        proginfo->programflags = proginfo->getProgramFlags(db);
+
         return proginfo;
     }
 
@@ -757,9 +776,10 @@ bool ProgramInfo::IsSameTimeslot(const ProgramInfo& other) const
 bool ProgramInfo::IsSameProgramTimeslot(const ProgramInfo &other) const
 {
     if (chanid == other.chanid &&
-        startts == other.startts &&
-        endts == other.endts &&
-        sourceid == other.sourceid)
+        startts < other.endts &&
+        endts > other.startts &&
+        (sourceid == -1 || other.sourceid == -1 ||
+         sourceid == other.sourceid))
         return true;
     return false;
 }
@@ -943,6 +963,70 @@ void ProgramInfo::SetEditing(bool edit, QSqlDatabase *db)
         MythContext::DBError("Edit status update", querystr);
 }
 
+bool ProgramInfo::IsCommFlagged(QSqlDatabase *db)
+{
+    MythContext::KickDatabase(db);
+
+    bool result = false;
+
+    QString starts = recstartts.toString("yyyyMMddhhmm");
+    starts += "00";
+
+    QString querystr = QString("SELECT commflagged FROM recorded WHERE "
+                               "chanid = '%1' AND starttime = '%2';")
+                              .arg(chanid).arg(starts);
+
+    QSqlQuery query = db->exec(querystr);
+    if (query.isActive() && query.numRowsAffected() > 0)
+    {
+        query.next();
+
+        result = (query.value(0).toInt() == 1);
+    }
+
+    return result;
+}
+
+void ProgramInfo::SetCommFlagged(int flag, QSqlDatabase *db)
+{
+    MythContext::KickDatabase(db);
+
+    QString starts = recstartts.toString("yyyyMMddhhmm");
+    starts += "00";
+
+    QString querystr = QString("UPDATE recorded SET commflagged = '%1', "
+                               "starttime = '%2' WHERE chanid = '%3' AND "
+                               "starttime = '%4';").arg(flag).arg(starts)
+                                                   .arg(chanid).arg(starts);
+    QSqlQuery query = db->exec(querystr);
+    if (!query.isActive())
+        MythContext::DBError("Commercial Flagged status update", querystr);
+}
+
+bool ProgramInfo::IsCommProcessing(QSqlDatabase *db)
+{
+    MythContext::KickDatabase(db);
+
+    bool result = false;
+
+    QString starts = recstartts.toString("yyyyMMddhhmm");
+    starts += "00";
+
+    QString querystr = QString("SELECT commflagged FROM recorded WHERE "
+                               "chanid = '%1' AND starttime = '%2';")
+                              .arg(chanid).arg(starts);
+
+    QSqlQuery query = db->exec(querystr);
+    if (query.isActive() && query.numRowsAffected() > 0)
+    {
+        query.next();
+
+        result = (query.value(0).toInt() == 2);
+    }
+
+    return result;
+}
+
 void ProgramInfo::SetAutoExpire(bool autoExpire, QSqlDatabase *db)
 {
     MythContext::KickDatabase(db);
@@ -1388,67 +1472,7 @@ void ProgramInfo::setOverride(QSqlDatabase *db, int override)
     }
 }
 
-QString ProgramInfo::NoRecordText(void)
-{
-    switch (norecord)
-    {
-    case nrManualOverride:
-        return "it was manually set to not record";
-    case nrPreviousRecording:
-        return "this episode was previously recorded according to the duplicate policy chosen for this title";
-    case nrCurrentRecording:
-        return "this episode was previously recorded and is still available in the list of recordings";
-    case nrOtherShowing:
-        return "this episode will be recorded at another time instead";
-    case nrTooManyRecordings:
-        return "too many recordings of this program have already been recorded";
-    case nrDontRecordList:
-        return "it is currently being recorded or was manually canceled";
-    case nrLowerRecPriority:
-        return "another program with a higher recording priority will be recorded";
-    case nrManualConflict:
-        return "another program was manually chosen to be recorded instead";
-    case nrAutoConflict:
-        return "another program was automatically chosen to be recorded instead";
-    case nrOverlap:
-        return "it is covered by another scheduled recording for the same program";
-    case nrUnknown:
-    default:
-        return "you should never see this";
-    }
-}
-
-QString ProgramInfo::NoRecordChar(void)
-{
-    switch (norecord)
-    {
-    case nrManualOverride:
-        return "X";
-    case nrPreviousRecording:
-        return "P";
-    case nrCurrentRecording:
-        return "C";
-    case nrOtherShowing:
-        return "O";
-    case nrTooManyRecordings:
-        return "T";
-    case nrDontRecordList:
-        return "D";
-    case nrLowerRecPriority:
-        return "R";
-    case nrManualConflict:
-        return "M";
-    case nrAutoConflict:
-        return "A";
-    case nrOverlap:
-        return "V";
-    case nrUnknown:
-    default:
-        return "-";
-    }
-}
-
-QString ProgramInfo::RecordingChar(void)
+QString ProgramInfo::RecTypeChar(void)
 {
     QString recstring = "";
 
@@ -1478,7 +1502,7 @@ QString ProgramInfo::RecordingChar(void)
     return recstring;
 }
 
-QString ProgramInfo::RecordingText(void)
+QString ProgramInfo::RecTypeText(void)
 {
     QString recstring;
 
@@ -1499,25 +1523,203 @@ QString ProgramInfo::RecordingText(void)
     case kAllRecord:
         recstring = QObject::tr("All Recording");
         break;
-    case kNotRecording:
     default:
         recstring = QObject::tr("Not Recording");
         break;
     }
 
-    if (rectype != kNotRecording)
+    return recstring;
+}
+
+QString ProgramInfo::RecStatusChar(void)
+{
+    switch (recstatus)
     {
-        recstring += " - ";
-        if (conflicting)
-            recstring += QObject::tr("Conflicting");
-        else if (recording)
-            recstring += QObject::tr("Will Record");
-        else
-            recstring += QObject::tr("Won't Record") +
-                " (" + NoRecordChar() + ")";
+    case rsDeleted:
+        return "D";
+    case rsStopped:
+        return "S";
+    case rsWillRecord:
+        return QString::number(cardid);
+    case rsRecording:
+        return QString::number(cardid);
+    case rsManualOverride:
+        return "X";
+    case rsPreviousRecording:
+        return "P";
+    case rsCurrentRecording:
+        return "C";
+    case rsOtherShowing:
+        return "O";
+    case rsTooManyRecordings:
+        return "T";
+    case rsCancelled:
+        return "N";
+    case rsLowerRecPriority:
+        return "R";
+    case rsManualConflict:
+        return "M";
+    case rsAutoConflict:
+        return "A";
+    case rsOverlap:
+        return "V";
+    case rsLowDiskSpace:
+        return "K";
+    case rsTunerBusy:
+        return "B";
+    default:
+        return "-";
+    }
+}
+
+QString ProgramInfo::RecStatusText(void)
+{
+    if (rectype == kNotRecording)
+        return QObject::tr("Not Recording");
+    else if (conflicting)
+        return QObject::tr("Conflicting");
+    else
+    {
+        switch (recstatus)
+        {
+        case rsDeleted:
+            return QObject::tr("Deleted");
+        case rsStopped:
+            return QObject::tr("Stopped");
+        case rsRecorded:
+            return QObject::tr("Recorded");
+        case rsRecording:
+            return QObject::tr("Recording");
+        case rsWillRecord:
+            return QObject::tr("Will Record");
+        case rsManualOverride:
+            return QObject::tr("Manual Override");
+        case rsPreviousRecording:
+            return QObject::tr("Previous Recording");
+        case rsCurrentRecording:
+            return QObject::tr("Current Recording");
+        case rsOtherShowing:
+            return QObject::tr("Other Showing");
+        case rsTooManyRecordings:
+            return QObject::tr("Max Recordings");
+        case rsCancelled:
+            return QObject::tr("Manual Cancel");
+        case rsLowerRecPriority:
+            return QObject::tr("Low Priority");
+        case rsManualConflict:
+            return QObject::tr("Manual Conflict");
+        case rsAutoConflict:
+            return QObject::tr("Auto Conflict");
+        case rsOverlap:
+            return QObject::tr("Overlap");
+        case rsLowDiskSpace:
+            return QObject::tr("Low Disk Space");
+        case rsTunerBusy:
+            return QObject::tr("Tuner Busy");
+        default:
+            return QObject::tr("Unknown");
+        }
     }
 
-    return recstring;
+    return QObject::tr("Unknown");;
+}
+
+QString ProgramInfo::RecStatusDesc(void)
+{
+    QString message;
+    QDateTime now = QDateTime::currentDateTime();
+
+    if (conflicting)
+        message += QObject::tr("This showing conflicts with one or more other "
+                               "scheduled programs.");
+    else if (recording)
+    {
+        switch (recstatus)
+        {
+        case rsWillRecord:
+            message = QObject::tr("This showing will be recorded.");
+            break;
+        case rsRecording:
+            message = QObject::tr("This showing is being recorded.");
+            break;
+        case rsRecorded:
+            message = QObject::tr("This showing was recorded.");
+            break;
+        case rsStopped:
+            message = QObject::tr("This showing was recorded but was stopped "
+                                   "before recording was completed.");
+            break;
+        case rsDeleted:
+            message = QObject::tr("This showing was recorded but was deleted "
+                                   "before recording was completed.");
+            break;
+        default:
+            message = QObject::tr("The status of this showing is unknown.");
+            break;
+        }
+    }
+    else
+    {
+        if (recstartts > now)
+            message = QObject::tr("This showing will not be recorded because ");
+        else
+            message = QObject::tr("This showing was not recorded because ");
+
+        switch (recstatus)
+        {
+        case rsManualOverride:
+            message += QObject::tr("it was manually set to not record.");
+            break;
+        case rsPreviousRecording:
+            message += QObject::tr("this episode was previously recorded "
+                                   "according to the duplicate policy chosen "
+                                   "for this title.");
+            break;
+        case rsCurrentRecording:
+            message += QObject::tr("this episode was previously recorded and "
+                                   "is still available in the list of "
+                                   "recordings.");
+            break;
+        case rsOtherShowing:
+            message += QObject::tr("this episode will be recorded at another "
+                                   "time instead.");
+            break;
+        case rsTooManyRecordings:
+            message += QObject::tr("too many recordings of this program have "
+                                   "already been recorded.");
+            break;
+        case rsCancelled:
+            message += QObject::tr("it was manually cancelled.");
+            break;
+        case rsLowerRecPriority:
+            message += QObject::tr("another program with a higher recording "
+                                   "priority will be recorded.");
+            break;
+        case rsManualConflict:
+            message += QObject::tr("another program was manually chosen to be "
+                                    "recorded instead.");
+            break;
+        case rsAutoConflict:
+            message += QObject::tr("another program was automatically chosen "
+                                   "to be recorded instead.");
+            break;
+        case rsOverlap:
+            message += QObject::tr("it is covered by another scheduled "
+                                   "recording for the same program.");
+            break;
+        case rsLowDiskSpace:
+            message += QObject::tr("there wasn't enough disk space available.");
+            break;
+        case rsTunerBusy:
+            message += QObject::tr("the tuner card was already being used.");
+            break;
+        default:
+            message += QObject::tr("you should never see this.");
+            break;
+        }
+    }
+
+    return message;
 }
 
 void ProgramInfo::FillInRecordInfo(vector<ProgramInfo *> &reclist)
@@ -1551,7 +1753,7 @@ void ProgramInfo::FillInRecordInfo(vector<ProgramInfo *> &reclist)
         conflicting = found->conflicting;
         recording = found->recording;
         override = found->override;
-        norecord = found->norecord;
+        recstatus = found->recstatus;
         recordid = found->recordid;
         rectype = found->rectype;
         recdups = found->recdups;
@@ -1562,43 +1764,27 @@ void ProgramInfo::FillInRecordInfo(vector<ProgramInfo *> &reclist)
 
 void ProgramInfo::Save(QSqlDatabase *db)
 {
-    QString querystr;
-    QString escaped_title;
-    QString escaped_subtitle;
-    QString escaped_description;
-    QString escaped_category;
+    QSqlQuery query(QString::null, db);
 
-    escaped_title = title;
-    escaped_title.replace(QRegExp("\""), QString("\\\""));
+    query.prepare("REPLACE INTO program (chanid,starttime,endtime,"
+                  "title,subtitle,description,category,airdate,"
+                  "stars) VALUES(:CHANID,:STARTTIME,:ENDTIME,:TITLE,"
+                  ":SUBTITLE,:DESCRIPTION,:CATEGORY,:AIRDATE,:STARS);");
+    query.bindValue(":CHANID", chanid.toInt());
+    query.bindValue(":STARTTIME", startts.toString("yyyyMMddhhmmss"));
+    query.bindValue(":ENDTIME", endts.toString("yyyyMMddhhmmss"));
+    query.bindValue(":TITLE", title.utf8());
+    query.bindValue(":SUBTITLE", subtitle.utf8());
+    query.bindValue(":DESCRIPTION", description.utf8());
+    query.bindValue(":CATEGORY", category.utf8());
+    query.bindValue(":AIRDATE", "0");
+    query.bindValue(":STARS", "0");
 
-    escaped_subtitle = subtitle;
-    escaped_subtitle.replace(QRegExp("\""), QString("\\\""));
-
-    escaped_description = description;
-    escaped_description.replace(QRegExp("\""), QString("\\\""));
-
-    escaped_category = category;
-    escaped_category.replace(QRegExp("\""), QString("\\\""));
-
-    querystr.sprintf("INSERT INTO program (chanid,starttime,endtime,"
-                    "title,subtitle,description,category,airdate,"
-                    "stars) VALUES(%d,\"%s\",\"%s\",\"%s\",\"%s\", "
-                    "\"%s\",\"%s\",\"%s\",\"%s\");",
-                    chanid.toInt(),
-                    startts.toString("yyyyMMddhhmmss").ascii(),
-                    endts.toString("yyyyMMddhhmmss").ascii(),
-                    escaped_title.utf8().data(),
-                    escaped_subtitle.utf8().data(),
-                    escaped_description.utf8().data(),
-                    escaped_category.utf8().data(),
-                    "0",
-                    "0");
-
-    QSqlQuery query = db->exec(querystr.utf8().data());
+    if (!query.exec())
+        MythContext::DBError("Saving program", query);
 }
 
-QGridLayout* ProgramInfo::DisplayWidget(ScheduledRecording* rec,
-                                        QWidget *parent)
+QGridLayout* ProgramInfo::DisplayWidget(QWidget *parent)
 {
     int row = 0;
     float wmult, hmult;
@@ -1621,36 +1807,6 @@ QGridLayout* ProgramInfo::DisplayWidget(ScheduledRecording* rec,
     airDateLabel->setBackgroundOrigin(QWidget::WindowOrigin);
     QLabel* airDateField = new QLabel(airDateText,parent);
     airDateField->setBackgroundOrigin(QWidget::WindowOrigin);
-
-    QString recDateText = "";
-    QLabel *recDateLabel = NULL;
-    QLabel *recDateField = NULL;
-    if (rec)
-    {
-        switch (rec->getRecordingType()) {
-        case kSingleRecord:
-            recDateText += startts.toString(dateFormat + " " + timeFormat);
-            break;
-        case kWeekslotRecord:
-            recDateText += startts.toString("dddd")+"s ";
-        case kTimeslotRecord:
-            recDateText += QString("%1 - %2")
-                .arg(startts.toString(timeFormat))
-                .arg(endts.toString(timeFormat));
-            break;
-        case kChannelRecord:
-        case kAllRecord:
-        case kNotRecording:
-            recDateText += "(any)";
-            break;
-        }
-
-        recDateLabel = new QLabel(QObject::tr("Record Schedule:"), parent);
-        recDateLabel->setBackgroundOrigin(QWidget::WindowOrigin);
-        recDateField = new QLabel(recDateText,parent);
-        recDateField->setBackgroundOrigin(QWidget::WindowOrigin);
-    }
-
     QLabel *subtitlelabel = new QLabel(QObject::tr("Episode:"), parent);
     subtitlelabel->setBackgroundOrigin(QWidget::WindowOrigin);
     QLabel *subtitlefield = new QLabel(subtitle, parent);
@@ -1665,7 +1821,7 @@ QGridLayout* ProgramInfo::DisplayWidget(ScheduledRecording* rec,
 
     descriptionlabel->polish();
 
-    int descwidth = (int)(700 * wmult) - descriptionlabel->width();
+    int descwidth = (int)(740 * wmult) - descriptionlabel->width();
     int titlewidth = (int)(760 * wmult);
 
     titlefield->setMinimumWidth(titlewidth);
@@ -1682,12 +1838,6 @@ QGridLayout* ProgramInfo::DisplayWidget(ScheduledRecording* rec,
     grid->addWidget(airDateLabel, row, 0, Qt::AlignLeft | Qt::AlignTop);
     grid->addWidget(airDateField, row, 1, Qt::AlignLeft);
     row++;
-    if (rec)
-    {
-        grid->addWidget(recDateLabel, row, 0, Qt::AlignLeft | Qt::AlignTop);
-        grid->addWidget(recDateField, row, 1, Qt::AlignLeft);
-        row++;
-    }
     grid->addWidget(subtitlelabel, row, 0, Qt::AlignLeft | Qt::AlignTop);
     grid->addWidget(subtitlefield, row, 1, Qt::AlignLeft | Qt::AlignTop);
     row++;
@@ -1735,6 +1885,36 @@ void ProgramInfo::EditScheduled(QSqlDatabase *db)
     ScheduledRecording::signalChange(db);
 }
 
+int ProgramInfo::getProgramFlags(QSqlDatabase *db)
+{
+    int flags = 0;
+
+    MythContext::KickDatabase(db);
+        
+    QString starts = recstartts.toString("yyyyMMddhhmm");
+    starts += "00";
+    
+    QString querystr = QString("SELECT commflagged, cutlist, autoexpire, "
+                               "editing, bookmark FROM recorded WHERE "
+                               "chanid = '%1' AND starttime = '%2';")
+                              .arg(chanid).arg(starts);
+    
+    QSqlQuery query = db->exec(querystr);
+    if (query.isActive() && query.numRowsAffected() > 0)
+    {
+        query.next();
+
+        flags |= (query.value(0).toInt() == 1) ? FL_COMMFLAG : 0;
+        flags |= query.value(1).toString().length() > 1 ? FL_CUTLIST : 0;
+        flags |= query.value(2).toInt() ? FL_AUTOEXP : 0;
+        if (query.value(3).toInt() || (query.value(0).toInt() == 2))
+            flags |= FL_EDITING;
+        flags |= query.value(4).toString().length() > 1 ? FL_BOOKMARK : 0;
+    }
+
+    return flags;
+}
+
 void ProgramInfo::handleRecording(QSqlDatabase *db)
 {
     QString message = title;
@@ -1742,14 +1922,18 @@ void ProgramInfo::handleRecording(QSqlDatabase *db)
     if (subtitle != "")
         message += QString(" - \"%1\"").arg(subtitle);
 
-    message += "\n\nThis showing will be recorded.";
+    message += "\n\n";
+    message += RecStatusDesc();
 
-    DialogBox diag(gContext->GetMainWindow(), QObject::tr(message));
+    DialogBox diag(gContext->GetMainWindow(), message);
     diag.AddButton(QObject::tr("OK"));
-    diag.AddButton(QObject::tr("Don't record it"));
 
-    if (subtitle != "" && description != "")
-        diag.AddButton(QObject::tr("Never record this episode"));
+    if (recstatus == rsWillRecord)
+    {
+        diag.AddButton(QObject::tr("Don't record it"));
+        if (subtitle != "" && description != "")
+            diag.AddButton(QObject::tr("Never record this episode"));
+    }
 
     int ret = diag.exec();
 
@@ -1776,16 +1960,21 @@ void ProgramInfo::handleNotRecording(QSqlDatabase *db)
     if (subtitle != "")
         message += QString(" - \"%1\"").arg(subtitle);
 
-    message += QString("\n\nThis showing will not be recorded because %1.")
-        .arg(NoRecordText());
+    message += "\n\n";
+    message += RecStatusDesc();
 
     DialogBox diag(gContext->GetMainWindow(), QObject::tr(message));
     diag.AddButton(QObject::tr("OK"));
 
-    if (norecord != nrTooManyRecordings &&
-        norecord != nrDontRecordList &&
-        norecord != nrLowerRecPriority &&
-        norecord != nrOverlap)
+    QDateTime now = QDateTime::currentDateTime();
+
+    if (recstartts > now &&
+        recstatus != rsTooManyRecordings &&
+        recstatus != rsCancelled &&
+        recstatus != rsLowerRecPriority &&
+        recstatus != rsOverlap &&
+        recstatus != rsLowDiskSpace &&
+        recstatus != rsTunerBusy)
         diag.AddButton(QObject::tr("Record it anyway"));
 
     int ret = diag.exec();
@@ -1793,8 +1982,8 @@ void ProgramInfo::handleNotRecording(QSqlDatabase *db)
     if (ret <= 1)
         return;
 
-    if (norecord != nrManualConflict &&
-        norecord != nrAutoConflict)
+    if (recstatus != rsManualConflict &&
+        recstatus != rsAutoConflict)
     {
         setOverride(db, 1);
         return;
@@ -2022,3 +2211,150 @@ void ProgramInfo::handleConflicting(QSqlDatabase *db)
         delete (*iter);
     delete conflictlist;
 }
+
+PGInfoCon::PGInfoCon()
+{
+    init();
+}
+
+PGInfoCon::~PGInfoCon() 
+{
+    init();
+}
+
+void PGInfoCon::updateAll(const vector<ProgramInfo *> &pginfoList)
+{
+    ProgramInfo *tmp = allData.at(selectedPos);
+    ProgramInfo org;
+
+    if (tmp)
+        org = *tmp;
+    else
+        selectedPos = -1;
+
+    allData.clear();
+    vector<ProgramInfo *>::const_iterator i;
+    for (i = pginfoList.begin(); i != pginfoList.end(); i++)
+        allData.append(*i);
+
+    if (selectedPos >= 0)
+    {
+        tmp = allData.at(selectedPos);
+        if (tmp && tmp->MakeUniqueKey() == org.MakeUniqueKey())
+            ;
+        else
+        {
+            tmp = find(org);
+            if (!tmp)
+                tmp = findNearest(org);
+
+            if (tmp)
+                selectedPos = allData.find(tmp);
+        }
+    }
+
+    checkSetIndex();
+}
+
+bool PGInfoCon::update(const ProgramInfo &pginfo)
+{
+    ProgramInfo *oldpginfo = find(pginfo);
+
+    if (oldpginfo)
+    {
+        *oldpginfo = pginfo;
+        return true;
+    }    
+
+    return false;    
+}
+
+int PGInfoCon::count()
+{
+    return allData.count();
+}
+
+void PGInfoCon::clear()
+{
+    init();
+}
+
+PGInfoCon::PGInfoConIt PGInfoCon::iterator()
+{
+    PGInfoConIt it(allData);
+    return it;
+}
+
+int PGInfoCon::selectedIndex()
+{
+    return selectedPos;
+}
+
+void PGInfoCon::setSelected(int delta)
+{
+    selectedPos += delta;
+    checkSetIndex();        
+}
+
+bool PGInfoCon::getSelected(ProgramInfo &pginfo)
+{
+    ProgramInfo *current = allData.at(selectedPos);
+    if (current)
+    {
+        pginfo = *current;
+        return true;
+    }    
+
+    return false;    
+}
+
+void PGInfoCon::init()
+{
+    selectedPos = -1;
+    allData.setAutoDelete(true);
+    allData.clear();
+}
+
+void PGInfoCon::checkSetIndex()
+{
+    if (selectedPos < 0)
+        selectedPos = 0;
+    else if (selectedPos > count() - 1)
+        selectedPos = count() - 1;
+
+    if (count() == 0)
+        selectedPos = -1;
+}
+
+ProgramInfo *PGInfoCon::find(const ProgramInfo &pginfo)
+{
+    QString key(pginfo.MakeUniqueKey());
+    return findFromKey(key);
+}
+
+ProgramInfo *PGInfoCon::findFromKey(const QString &key)
+{
+    if (key == "")
+        return NULL;
+
+    ProgramInfo *pginfo;
+    for (PGInfoConIt it(allData); (pginfo = it.current()) != 0; ++it)
+    {
+        if (pginfo->MakeUniqueKey() == key)
+            return pginfo;
+    }        
+
+    return NULL;                
+}
+
+ProgramInfo *PGInfoCon::findNearest(const ProgramInfo &pginfo)
+{
+    ProgramInfo *tmppginfo;
+    for (PGInfoConIt it(allData); (tmppginfo = it.current()) != 0; ++it)
+    {
+        if (tmppginfo->recstartts >= pginfo.recstartts)
+            return tmppginfo;
+    }
+    return allData.last();
+}
+
