@@ -7,7 +7,7 @@
 #include <qmap.h>
 #include "scheduledrecording.h"
 
-#define NUMPROGRAMLINES 28
+#define NUMPROGRAMLINES 29
 
 enum MarkTypes {
     MARK_UPDATED_CUT = -3,
@@ -21,7 +21,8 @@ enum MarkTypes {
     MARK_COMM_END = 5,
     MARK_GOP_START = 6,
     MARK_KEYFRAME = 7,
-    MARK_SCENE_CHANGE = 8
+    MARK_SCENE_CHANGE = 8,
+    MARK_GOP_BYFRAME = 9
 };
 
 enum TranscoderStatus {
@@ -36,18 +37,33 @@ enum TranscoderStatus {
     TRANSCODE_FLAGS       = 0xF0
 };
 
-enum NoRecordType {
-    nrUnknown = 0,
-    nrManualOverride = 1,
-    nrPreviousRecording = 2,
-    nrCurrentRecording = 3,
-    nrOtherShowing = 4,
-    nrTooManyRecordings = 5,
-    nrDontRecordList = 6,
-    nrLowerRecPriority = 7,
-    nrManualConflict = 8,
-    nrAutoConflict = 9,
-    nrOverlap = 10
+enum FlagMask {
+    FL_COMMFLAG  = 0x01,
+    FL_CUTLIST   = 0x02,
+    FL_AUTOEXP   = 0x04,
+    FL_EDITING   = 0x08,
+    FL_BOOKMARK  = 0x10,
+};
+
+enum RecStatusType {
+    rsDeleted = -5,
+    rsStopped = -4,
+    rsRecorded = -3,
+    rsRecording = -2,
+    rsWillRecord = -1,
+    rsUnknown = 0,
+    rsManualOverride = 1,
+    rsPreviousRecording = 2,
+    rsCurrentRecording = 3,
+    rsOtherShowing = 4,
+    rsTooManyRecordings = 5,
+    rsCancelled = 6,
+    rsLowerRecPriority = 7,
+    rsManualConflict = 8,
+    rsAutoConflict = 9,
+    rsOverlap = 10,
+    rsLowDiskSpace = 11,
+    rsTunerBusy = 12
 };
 
 class QSqlDatabase;
@@ -59,6 +75,8 @@ class ProgramInfo
     ProgramInfo(const ProgramInfo &other);
     ~ProgramInfo();
 
+    ProgramInfo& operator=(const ProgramInfo &other);
+
     // returns 0 for one-time, 1 for weekdaily, 2 for weekly
     int IsProgramRecurring(void);
 
@@ -94,24 +112,28 @@ class ProgramInfo
     void StartedRecording(QSqlDatabase *db);
     void FinishedRecording(QSqlDatabase* db);
 
-    QGridLayout* DisplayWidget(ScheduledRecording *rec = NULL,
-                               QWidget *parent = NULL);
+    QGridLayout* DisplayWidget(QWidget *parent = NULL);
 
     QString GetRecordBasename(void);
     QString GetRecordFilename(const QString &prefix);
 
-    QString MakeUniqueKey(void);
+    QString MakeUniqueKey(void) const;
 
     int CalculateLength(void);
 
     void ToStringList(QStringList &list);
     void FromStringList(QStringList &list, int offset);
+    void FromStringList(QStringList &list, QStringList::iterator &it);
     void ToMap(QSqlDatabase *db, QMap<QString, QString> &progMap);
 
     void SetBookmark(long long pos, QSqlDatabase *db);
     long long GetBookmark(QSqlDatabase *db);
     bool IsEditing(QSqlDatabase *db);
     void SetEditing(bool edit, QSqlDatabase *db);
+    bool IsCommFlagged(QSqlDatabase *db);
+    // 1 = flagged, 2 = processing
+    void SetCommFlagged(int flag, QSqlDatabase *db);
+    bool IsCommProcessing(QSqlDatabase *db);
     void SetAutoExpire(bool autoExpire, QSqlDatabase *db);
     bool GetAutoExpireFromRecorded(QSqlDatabase *db);
 
@@ -141,14 +163,17 @@ class ProgramInfo
                         long long min_frame = -1, long long max_frame = -1);
 
     void DeleteHistory(QSqlDatabase *db);
-    QString NoRecordText(void);
-    QString NoRecordChar(void);
-    QString RecordingChar(void);
-    QString RecordingText(void);
+    QString RecTypeChar(void);
+    QString RecTypeText(void);
+    QString RecStatusChar(void);
+    QString RecStatusText(void);
+    QString RecStatusDesc(void);
     void FillInRecordInfo(vector<ProgramInfo *> &reclist);
     void EditScheduled(QSqlDatabase *db);
     void EditRecording(QSqlDatabase *db);
 
+    int getProgramFlags(QSqlDatabase *db);
+
     static void GetProgramListByQuery(QSqlDatabase *db,
                                         QPtrList<ProgramInfo> *proglist,
                                         const QString &where);
@@ -198,7 +223,7 @@ class ProgramInfo
     bool conflicting;
     bool recording;
     int override;
-    NoRecordType norecord;
+    RecStatusType recstatus;
     unsigned recordid;
     RecordingType rectype;
     RecordingDupsType recdups;
@@ -210,6 +235,8 @@ class ProgramInfo
 
     QString schedulerid;
 
+    int programflags;
+
 private:
     void handleRecording(QSqlDatabase *db);
     void handleNotRecording(QSqlDatabase *db);
@@ -219,4 +246,33 @@ private:
     class ScheduledRecording* record;
 };
 
+class PGInfoCon
+{
+  public:
+    PGInfoCon();
+    ~PGInfoCon();
+    
+    typedef QPtrListIterator<ProgramInfo> PGInfoConIt;
+    
+    void updateAll(const vector<ProgramInfo *> &pginfoList);
+    bool update(const ProgramInfo &pginfo);
+    int count();
+    void clear();
+   
+    int selectedIndex();
+    void setSelected(int delta = 1);
+    bool getSelected(ProgramInfo &pginfo);
+    
+    PGInfoConIt iterator();
+  private:
+    void init();
+    void checkSetIndex();
+    ProgramInfo *find(const ProgramInfo &pginfo);
+    ProgramInfo *findFromKey(const QString &key);
+    ProgramInfo *findNearest(const ProgramInfo &pginfo);
+     
+    QPtrList<ProgramInfo> allData;
+    int selectedPos;
+};
+
 #endif
