@@ -110,7 +110,7 @@ public:
         addSelection("416 kbps", "416");
         addSelection("448 kbps", "448");
         setValue(13);
-        setHelpText("Sets the audio bitrate");
+        setHelpText(QObject::tr("Sets the audio bitrate"));
     };
 };
 
@@ -506,6 +506,18 @@ public:
 
         addTarget("MPEG-4", params);
 
+        // We'll eventually want to add MPEG2 software encoding params here
+        params = new VerticalConfigurationGroup(false);
+        params->setLabel(QObject::tr("MPEG-4 Parameters"));
+        params->addChild(new MPEG4bitrate(parent));
+        params->addChild(new MPEG4MaxQuality(parent));
+        params->addChild(new MPEG4MinQuality(parent));
+        params->addChild(new MPEG4QualDiff(parent));
+        params->addChild(new MPEG4ScaleBitrate(parent));
+        params->addChild(new MPEG4OptionVHQ(parent));
+        params->addChild(new MPEG4Option4MV(parent));
+        addTarget("MPEG-2", params);
+
         params = new VerticalConfigurationGroup();
         params->setLabel(QObject::tr("Hardware MJPEG Parameters"));
         params->addChild(new HardwareMJPEGQuality(parent));
@@ -601,7 +613,9 @@ public:
         if (tvFormat.lower() == "ntsc" || tvFormat.lower() == "ntsc-jp") {
             maxwidth = 720;
             maxheight = 480;
-
+        } else if (tvFormat.lower() == "atsc") {
+            maxwidth = 1920;
+            maxheight = 1088;
         } else {
             maxwidth = 768;
             maxheight = 576;
