@@ -12,7 +12,7 @@ void RemoteGetFreeSpace(int &totalspace, int &usedspace);
 bool RemoteCheckFile(ProgramInfo *pginfo);
 void RemoteQueueTranscode(ProgramInfo *pginfo, int state);
 void RemoteStopRecording(ProgramInfo *pginfo);
-void RemoteDeleteRecording(ProgramInfo *pginfo);
+void RemoteDeleteRecording(ProgramInfo *pginfo, bool forgetHistory);
 bool RemoteGetAllPendingRecordings(vector<ProgramInfo *> &recordinglist);
 void RemoteGetAllScheduledRecordings(vector<ProgramInfo *> &scheduledlist);
 vector<ProgramInfo *> *RemoteGetConflictList(ProgramInfo *pginfo,
@@ -26,4 +26,8 @@ void RemoteFillProginfo(ProgramInfo *pginfo, const QString &playbackhostname);
 int RemoteIsRecording(void);
 int RemoteGetRecordingMask(void);
 
+int RemoteCheckForRecording(ProgramInfo *pginfo);
+int RemoteGetRecordingStatus(ProgramInfo *pginfo, int overrecsecs, 
+                             int underrecsecs);
+
 #endif
