@@ -79,7 +79,6 @@ public:
 
     virtual void fillSelections(QSqlDatabase* db) {
         clearSelections();
-        addSelection(QObject::tr("(unspecified)"), "0");
         RecordingProfile::fillSelections(db, this, 0);
     };
 };
@@ -109,7 +108,7 @@ public:
     SRPreRoll(const ScheduledRecording& parent):
         SpinBoxSetting(-60, 60, 10, true),
         SRSetting(parent, "preroll") {
-        setLabel(QObject::tr("Pre-Roll Minutes"));
+        setLabel(QObject::tr("Start Early (minutes)"));
     };
 };
 
@@ -118,7 +117,7 @@ public:
     SRPostRoll(const ScheduledRecording& parent):
         SpinBoxSetting(-60, 240, 10, true),
         SRSetting(parent, "postroll") {
-        setLabel(QObject::tr("Post-Roll Minutes"));
+        setLabel(QObject::tr("End Late   (minutes)"));
     };
 };
 
@@ -136,7 +135,7 @@ class SRMaxNewest: public CheckBoxSetting, public SRSetting {
 public:
     SRMaxNewest(const ScheduledRecording& parent):
         SRSetting(parent, "maxnewest") {
-        setLabel(QObject::tr("Record new episodes and delete oldest once "
+        setLabel(QObject::tr("Record new and delete oldest once "
                              "maximum count is reached"));
         setValue(false);
     };
@@ -347,6 +346,7 @@ void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
          while (result.next()) {
              ProgramInfo *proginfo = new ProgramInfo;
              proginfo->recording = true;
+             proginfo->recstatus = rsWillRecord;
              proginfo->chanid = result.value(0).toString();
              proginfo->sourceid = result.value(1).toInt();
              proginfo->startts = result.value(2).toDateTime();
@@ -356,7 +356,7 @@ void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
              proginfo->description = QString::fromUtf8(result.value(6).toString());
              proginfo->chanstr = result.value(7).toString();
              proginfo->chansign = result.value(8).toString();
-             proginfo->channame = result.value(9).toString();
+             proginfo->channame = QString::fromUtf8(result.value(9).toString());
              proginfo->category = QString::fromUtf8(result.value(11).toString());
              proginfo->recpriority = result.value(12).toString();
              proginfo->recdups = RecordingDupsType(result.value(13).toInt());
@@ -379,7 +379,7 @@ void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
              if (proginfo->override == 2)
              {
                  proginfo->recording = false;
-                 proginfo->norecord = nrManualOverride;
+                 proginfo->recstatus = rsManualOverride;
              }
              else if (proginfo->rectype != kSingleRecord &&
                       proginfo->override != 1 &&
@@ -390,7 +390,7 @@ void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
                      if (result.value(14).toInt())
                      {
                          proginfo->recording = false;
-                         proginfo->norecord = nrCurrentRecording;
+                         proginfo->recstatus = rsCurrentRecording;
                      }
                  }
                  else
@@ -398,7 +398,7 @@ void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
                      if (result.value(10).toInt())
                      {
                          proginfo->recording = false;
-                         proginfo->norecord = nrPreviousRecording;
+                         proginfo->recstatus = rsPreviousRecording;
                      }
                  }
              }
@@ -417,7 +417,7 @@ void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
              if (proginfo->category == QString::null)
                  proginfo->category = "";
 
-             if (proginfo->endts < now)
+             if (proginfo->recendts < now)
                  delete proginfo;
              else 
                  proglist.push_back(proginfo);
@@ -476,7 +476,7 @@ void ScheduledRecording::findAllScheduledPrograms(QSqlDatabase* db,
                 proginfo->subtitle = QString::fromUtf8(result.value(6).toString());
             proginfo->description = QString::fromUtf8(result.value(7).toString());
             proginfo->recpriority = result.value(8).toString();
-            proginfo->channame = result.value(10).toString();
+            proginfo->channame = QString::fromUtf8(result.value(10).toString());
             proginfo->recstartts = proginfo->startts;
             proginfo->recendts = proginfo->endts;
 
@@ -745,11 +745,12 @@ MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent,
     gContext->GetScreenSettings(wmult, hmult);
 
     MythDialog* dialog = new ConfigurationDialogWidget(parent, name);
-    QVBoxLayout* vbox = new QVBoxLayout(dialog, (int)(10 * wmult));
+    QVBoxLayout* vbox = new QVBoxLayout(dialog, (int)(20 * wmult),
+                                                (int)(10 * wmult));
 
     if (m_pginfo)
     {
-        QGridLayout *grid = m_pginfo->DisplayWidget(this, dialog);
+        QGridLayout *grid = m_pginfo->DisplayWidget(dialog);
         vbox->addLayout(grid);
     }
 
@@ -787,7 +788,7 @@ MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent,
 
 void ScheduledRecording::runProgList(void)
 {
-    ProgLister *pl = new ProgLister(title->getValue(),
+    ProgLister *pl = new ProgLister(plTitle, title->getValue(),
                                     QSqlDatabase::database(),
                                     gContext->GetMainWindow(), "proglist");
     pl->exec();
