@@ -17,6 +17,15 @@ enum RecordingType
     kWeekslotRecord
 };
 
+int RecTypePriority(RecordingType rectype);
+
+enum RecordingDupsType
+{
+    kRecordDupsNever = 0,
+    kRecordDupsIfDeleted,
+    kRecordDupsAlways
+};
+
 class ProgramInfo;
 class QSqlDatabase;
 class ScheduledRecording: public ConfigurationGroup, public ConfigurationDialog {
@@ -38,7 +47,7 @@ public:
 
     void setStart(const QDateTime& start);
     void setEnd(const QDateTime& end);
-    void setRank(const QString& rank);
+    void setRecPriority(const QString& recpriority);
 
     virtual MythDialog* dialogWidget(MythMainWindow *parent, 
                                      const char *name = 0);
@@ -90,6 +99,8 @@ private:
     class SRProfileSelector* profile;
     class SRRecordDups* recorddups;
     class SRAutoExpire* autoexpire;
+    class SRPreRoll* preroll;
+    class SRPostRoll* postroll;
     class SRMaxEpisodes* maxepisodes;
     class SRMaxNewest* maxnewest;
     class SRChannel* channel;
@@ -101,7 +112,7 @@ private:
     class SREndTime* endTime;
     class SREndDate* endDate;
     class SRCategory* category;
-    class SRRank* rank;
+    class SRRecPriority* recpriority;
 
     ProgramInfo* m_pginfo;
 };
