@@ -56,21 +56,21 @@ class TV : public QObject
     bool IsMenuRunning(void) { return menurunning; }
 
     void GetNextProgram(RemoteEncoder *enc, int direction,
-                        QMap<QString, QString> &regexpMap);
+                        QMap<QString, QString> &infoMap);
     void GetNextProgram(RemoteEncoder *enc, int direction,
                         QString &title, QString &subtitle,
                         QString &desc, QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
                         QString &channelname, QString &chanid);
 
-    void GetChannelInfo(RemoteEncoder *enc, QMap<QString, QString> &regexpMap);
+    void GetChannelInfo(RemoteEncoder *enc, QMap<QString, QString> &infoMap);
     void GetChannelInfo(RemoteEncoder *enc, QString &title, QString &subtitle,
                         QString &desc, QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
                         QString &channelname, QString &chanid);
 
     // for the guidegrid to use
-    void EmbedOutput(unsigned long wid, int x, int y, int w, int h);
+    void EmbedOutput(WId wid, int x, int y, int w, int h);
     void StopEmbeddingOutput(void);
     void EPGChannelUpdate(QString chanstr);
    
@@ -110,8 +110,8 @@ class TV : public QObject
     void SetChannel(bool needopen = false);
 
     void ToggleChannelFavorite(void);
-    void ChangeChannel(int direction);
-    void ChangeChannelByString(QString &name);
+    void ChangeChannel(int direction, bool force = false);
+    void ChangeChannelByString(QString &name, bool force = false);
 
     void ChangeVolume(bool up);
     void ToggleMute(void);
@@ -121,7 +121,8 @@ class TV : public QObject
     void ChangeColour(bool up, bool recorder);
     void ChangeHue(bool up, bool recorder);
 
-    void ChannelClear(void); 
+    QString QueuedChannel(void);
+    void ChannelClear(bool hideosd = false); 
     void ChannelKey(int key);
     void ChannelCommit(void);
 
@@ -131,6 +132,7 @@ class TV : public QObject
     void DoPause(void);
     bool UpdatePosOSD(float time, const QString &mesg);
     void DoSeek(float time, const QString &mesg);
+    void DoArbSeek(int dir);
     void NormalSpeed(void);
     void ChangeSpeed(int direction);
     float StopFFRew(void);
@@ -165,11 +167,12 @@ class TV : public QObject
     void BrowseStart(void);
     void BrowseEnd(bool change);
     void BrowseDispInfo(int direction);
-    void BrowseToggleRecord(void);
+    void ToggleRecord(void);
     void BrowseChannel(QString &chan);
 
     void DoTogglePictureAttribute(void);
-    void DoChangePictureAttribute(bool up);
+    void DoToggleRecPictureAttribute(void);
+    void DoChangePictureAttribute(int control, bool up, bool rec);
 
     void DoProgramMenu(void);
     void ProgramMenuAction(int result);
@@ -190,6 +193,7 @@ class TV : public QObject
     bool paused;
 
     int autoCommercialSkip;
+    bool tryUnflaggedSkip;
 
     float frameRate;
 
@@ -244,7 +248,7 @@ class TV : public QObject
 
     VolumeControl *volumeControl;
 
-    unsigned int embedid;
+    WId embedid;
     int embx, emby, embw, embh;
 
     typedef QValueVector<QString> PrevChannelVector;
@@ -268,12 +272,18 @@ class TV : public QObject
     QTimer *keyrepeatTimer;
     
     int picAdjustment;
+    int recAdjustment;
     bool usePicControls;
 
     bool smartForward;
     bool doSmartForward;
 
     UDPNotify *udpnotify;
+
+    QString lastCC;
+    int lastCCDir;
+    bool showBufferedWarnings;
+    int bufferedChannelThreshold;
 };
 
 #endif
