@@ -1,8 +1,8 @@
 #include <qapplication.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <unistd.h>
 #include <pthread.h>
 #include <qsqldatabase.h>
@@ -20,12 +20,15 @@ using namespace std;
 #include "programinfo.h"
 #include "recorderbase.h"
 #include "NuppelVideoRecorder.h"
-#include "mpegrecorder.h"
 #include "hdtvrecorder.h"
 #include "NuppelVideoPlayer.h"
 #include "channel.h"
 #include "commercial_skip.h"
 
+#ifdef USING_IVTV
+#include "mpegrecorder.h"
+#endif
+
 #ifdef USING_DVB
 #include "dvbchannel.h"
 #include "dvbrecorder.h"
@@ -65,7 +68,7 @@ TVRec::TVRec(int capturecardnum)
     QString inputname, startchannel;
 
     GetDevices(capturecardnum, videodev, vbidev, audiodev, audiosamplerate,
-               inputname, startchannel, cardtype, dvb_swfilter, dvb_recordts);
+               inputname, startchannel, cardtype, dvb_options);
 
     if (cardtype == "DVB")
     {
@@ -92,7 +95,7 @@ TVRec::TVRec(int capturecardnum)
         channel = achannel; // here for SetFormat()->RetrieveInputChannels()
         channel->Open();
         achannel->SetFormat(gContext->GetSetting("TVFormat"));
-        achannel->SetFreqTable(gContext->GetSetting("FreqTable"));
+        achannel->SetDefaultFreqTable(gContext->GetSetting("FreqTable"));
         if (inputname.isEmpty())
             channel->SetChannelByString(startchannel);
         else
@@ -177,7 +180,8 @@ void TVRec::RecordPending(ProgramInfo *rcinfo, int secsleft)
 
 int TVRec::StartRecording(ProgramInfo *rcinfo)
 {
-    int retval = 0; //0 = recording canceled, 1 = recording started
+    int retval = 0; //  0 = recording canceled, 1 = recording started
+                    // -1 = other state/already recording
 
     recordPending = false;
     askAllowRecording = false;
@@ -203,42 +207,40 @@ int TVRec::StartRecording(ProgramInfo *rcinfo)
 
     if (internalState == kState_WatchingLiveTV && !cancelNextRecording)
     {
-        QDateTime timeout;
-
         QString message = QString("QUIT_LIVETV %1").arg(m_capturecardnum);
 
         MythEvent me(message);
         gContext->dispatch(me);
 
-        timeout = QDateTime::currentDateTime().addSecs(10);
+        QTime timer;
+        timer.start();
 
-        while (internalState != kState_None &&
-               QDateTime::currentDateTime().secsTo(timeout) > 0)
+        while (internalState != kState_None && timer.elapsed() < 10000)
             usleep(100);
 
         if (internalState != kState_None)
         {
             gContext->dispatch(me);
 
-            timeout = QDateTime::currentDateTime().addSecs(10);
+            timer.restart();
 
-            while (internalState != kState_None &&
-                   QDateTime::currentDateTime().secsTo(timeout) > 0)
+            while (internalState != kState_None && timer.elapsed() < 10000)
                 usleep(100);
         }
 
         if (internalState != kState_None)
         {
             exitPlayer = true;
-            timeout = QDateTime::currentDateTime().addSecs(5);
-            while (internalState != kState_None &&
-                   QDateTime::currentDateTime().secsTo(timeout) > 0)
+            timer.restart();
+            while (internalState != kState_None && timer.elapsed() < 5000)
                 usleep(100);
         }
     }
 
     if (internalState == kState_None)
     {
+        overrecordseconds = gContext->GetNumSetting("RecordOverTime");
+
         outputFilename = rcinfo->GetRecordFilename(recprefix);
         recordEndTime = rcinfo->recendts;
         curRecording = new ProgramInfo(*rcinfo);
@@ -471,6 +473,7 @@ void TVRec::HandleStateChange(void)
             profileName = "Live TV";
             profile.loadByCard(db_conn, profileName, m_capturecardnum);
         }
+
         QString msg = QString("Using profile '%1' to record").arg(profileName);
         VERBOSE(VB_RECORD, msg);
 
@@ -488,13 +491,22 @@ void TVRec::HandleStateChange(void)
         SetVideoFiltersForChannel(channel, channel->GetCurrentName());
         pthread_create(&encode, NULL, SpawnEncode, nvr);
 
-        while (!nvr->IsRecording())
+        while (!nvr->IsRecording() && !nvr->IsErrored())
             usleep(50);
 
-        // evil.
-        channel->SetFd(nvr->GetVideoFd());
-
-        frameRate = nvr->GetFrameRate();
+        if (nvr->IsRecording())
+        {
+            // evil.
+            channel->SetFd(nvr->GetVideoFd());
+            frameRate = nvr->GetFrameRate();
+        }
+        else
+        {
+            FinishedRecording();
+            killRecordingFile = true;
+            closeRecorder = true;
+            tmpInternalState = kState_None;
+        }
     }
 
     if (closeRecorder)
@@ -517,45 +529,24 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
 {
     if (cardtype == "MPEG")
     {
+#ifdef USING_IVTV
         nvr = new MpegRecorder();
         nvr->SetRingBuffer(rbuffer);
-        nvr->SetOption("videodevice", videodev);
-        nvr->SetOption("tvformat", gContext->GetSetting("TVFormat"));
-        nvr->SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
 
-        SetOption(profile, "mpeg2bitrate");
-        SetOption(profile, "mpeg2maxbitrate");
-        nvr->SetOption("mpeg2streamtype", 
-                       profile.byName("mpeg2streamtype")->getValue());
-        nvr->SetOption("mpeg2aspectratio", 
-                       profile.byName("mpeg2aspectratio")->getValue());
-
-        SetOption(profile, "samplerate");
-        nvr->SetOption("mpeg2audtype", 
-                       profile.byName("mpeg2audtype")->getValue());
-        SetOption(profile, "mpeg2audbitratel1");
-        SetOption(profile, "mpeg2audbitratel2");
-        SetOption(profile, "mpeg2audvolume");
-
-        SetOption(profile, "width");
-        SetOption(profile, "height");
-
-        if (ispip)
-        {
-            nvr->SetOption("width", 160);
-            nvr->SetOption("height", 128);
-        }
+        nvr->SetOptionsFromProfile(&profile, videodev, audiodev, vbidev, ispip);
 
         nvr->Initialize();
+#else
+        cerr << "Compiled without ivtv support\n";
+#endif
         return;
     }
     else if (cardtype == "HDTV")
     {
         nvr = new HDTVRecorder();
         nvr->SetRingBuffer(rbuffer);
-        nvr->SetOption("videodevice", videodev);
-        nvr->SetOption("tvformat", gContext->GetSetting("TVFormat"));
-        nvr->SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
+
+        nvr->SetOptionsFromProfile(&profile, videodev, audiodev, vbidev, ispip);
 
         nvr->Initialize();
         return;
@@ -565,9 +556,14 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
 #ifdef USING_DVB
         nvr = new DVBRecorder(dynamic_cast<DVBChannel*>(channel));
         nvr->SetRingBuffer(rbuffer);
-        nvr->SetOption("cardnum", videodev.toInt());
-        nvr->SetOption("swfilter", dvb_swfilter);
-        nvr->SetOption("recordts", dvb_recordts);
+
+        nvr->SetOptionsFromProfile(&profile, videodev, audiodev, vbidev, ispip);
+
+        nvr->SetOption("swfilter", dvb_options.swfilter);
+        nvr->SetOption("recordts", dvb_options.recordts);
+        nvr->SetOption("wait_for_seqstart", dvb_options.wait_for_seqstart);
+        nvr->SetOption("dmx_buf_size", dvb_options.dmx_buf_size);
+        nvr->SetOption("pkt_buf_size", dvb_options.pkt_buf_size);
         nvr->Initialize();
 #endif
         return;
@@ -579,81 +575,11 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
 
     nvr->SetRingBuffer(rbuffer);
 
-    nvr->SetOption("videodevice", videodev);
-    nvr->SetOption("vbidevice", vbidev);
-    nvr->SetOption("tvformat", gContext->GetSetting("TVFormat"));
-    nvr->SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
-    nvr->SetOption("audiodevice", audiodev);
-
-    QString setting = profile.byName("videocodec")->getValue();
-    if (setting == "MPEG-4") 
-    {
-        nvr->SetOption("codec", "mpeg4");
-
-        SetOption(profile, "mpeg4bitrate");
-        SetOption(profile, "mpeg4scalebitrate");
-        SetOption(profile, "mpeg4maxquality");
-        SetOption(profile, "mpeg4minquality");
-        SetOption(profile, "mpeg4qualdiff");
-        SetOption(profile, "mpeg4optionvhq");
-        SetOption(profile, "mpeg4option4mv");
-    } 
-    else if (setting == "RTjpeg") 
-    {
-        nvr->SetOption("codec", "rtjpeg");
-
-        SetOption(profile, "rtjpegquality");
-        SetOption(profile, "rtjpegchromafilter");
-        SetOption(profile, "rtjpeglumafilter");
-    } 
-    else if (setting == "Hardware MJPEG") 
-    {
-        nvr->SetOption("codec", "hardware-mjpeg");
-
-        SetOption(profile, "hardwaremjpegquality");
-        SetOption(profile, "hardwaremjpeghdecimation");
-        SetOption(profile, "hardwaremjpegvdecimation");
-    }
-    else 
-        cerr << "Unknown video codec: " << setting << endl;
-
-    setting = profile.byName("audiocodec")->getValue();
-    if (setting == "MP3") 
-    {
-        nvr->SetOption("audiocompression", 1);
-        SetOption(profile, "mp3quality");
-        SetOption(profile, "samplerate");
-    } 
-    else if (setting == "Uncompressed") 
-        nvr->SetOption("audiocompression", 0);
-    else 
-        cerr << "Unknown audio codec: " << setting << endl;
-
-    SetOption(profile, "width");
-    SetOption(profile, "height");
-
-    if (ispip)
-    {
-        nvr->SetOption("codec", "rtjpeg");
-
-        nvr->SetOption("width", 160);
-        nvr->SetOption("height", 128);
-        nvr->SetOption("rtjpegchromafilter", 0);
-        nvr->SetOption("rtjpeglumafilter", 0);
-        nvr->SetOption("rtjpegquality", 255);
-        nvr->SetOption("audiocompression", 0);
-        nvr->SetOption("pip_mode", 1);
-    }
+    nvr->SetOptionsFromProfile(&profile, videodev, audiodev, vbidev, ispip);
  
     nvr->Initialize();
 }
 
-// Needed by DVBChannel to set the PIDs of DVBRecorder
-RecorderBase* TVRec::GetRecorder()
-{
-    return nvr;
-}
-
 void TVRec::TeardownRecorder(bool killFile)
 {
     QMap<long long, int> blank_frame_map;
@@ -883,8 +809,9 @@ void TVRec::RunTV(void)
                 {
                     recordEndTime = recordEndTime.addSecs(overrecordseconds);
                     inoverrecord = true;
-                    //cout << "switching to overrecord for " 
-                    //     << overrecordseconds << " more seconds\n";
+                    VERBOSE(VB_RECORD, QString("switching to overrecord for " 
+                                               "%1 more seconds")
+                                               .arg(overrecordseconds));
                 }
                 else
                 {
@@ -1071,8 +998,8 @@ void TVRec::DisconnectDB(void)
 
 void TVRec::GetDevices(int cardnum, QString &video, QString &vbi, 
                        QString &audio, int &rate, QString &defaultinput,
-                       QString &startchan, QString &type, int &dvb_swfilter,
-                       int &dvb_recordts)
+                       QString &startchan, QString &type, 
+                       dvb_options_t &dvb_opts)
 {
     video = "";
     vbi = "";
@@ -1087,7 +1014,9 @@ void TVRec::GetDevices(int cardnum, QString &video, QString &vbi,
 
     QString thequery = QString("SELECT videodevice,vbidevice,audiodevice,"
                                "audioratelimit,defaultinput,cardtype,"
-                               "dvb_swfilter,dvb_recordts "
+                               "dvb_swfilter, dvb_recordts,"
+                               "dvb_wait_for_seqstart,dvb_dmx_buf_size,"
+                               "dvb_pkt_buf_size "
                                "FROM capturecard WHERE cardid = %1;")
                               .arg(cardnum);
 
@@ -1112,19 +1041,23 @@ void TVRec::GetDevices(int cardnum, QString &video, QString &vbi,
         if (test != QString::null)
             audio = QString::fromUtf8(test);
         testnum = query.value(3).toInt();
+        if (testnum > 0)
+            rate = testnum;
+        else
+            rate = -1;
+
         test = query.value(4).toString();
         if (test != QString::null)
             defaultinput = QString::fromUtf8(test);
         test = query.value(5).toString();
         if (test != QString::null)
             type = QString::fromUtf8(test);
-        dvb_swfilter = query.value(6).toInt();
-        dvb_recordts = query.value(7).toInt();
 
-        if (testnum > 0)
-            rate = testnum;
-        else
-            rate = -1;
+        dvb_opts.swfilter = query.value(6).toInt();
+        dvb_opts.recordts = query.value(7).toInt();
+        dvb_opts.wait_for_seqstart = query.value(8).toInt();
+        dvb_opts.dmx_buf_size = query.value(9).toInt();
+        dvb_opts.pkt_buf_size = query.value(10).toInt();
     }
 
     thequery = QString("SELECT if(startchan!='', startchan, '3') "
@@ -2056,28 +1989,44 @@ void TVRec::SetReadThreadSock(QSocket *sock)
     }
 }
 
-void TVRec::RequestRingBufferBlock(int size)
+int TVRec::RequestRingBufferBlock(int size)
 {
-    readthreadLock.lock();
-
     if (!readthreadlive || !rbuffer)
-        return;
+        return -1;
 
     char buffer[256001];
+    int tot = 0;
+    int ret = 0;
 
-    if (size > 256000)
-        size = 256000;
-
-    while (size > 0 && !rbuffer->GetStopReads() && readthreadlive)
+    readthreadLock.lock();
+    while (tot < size && !rbuffer->GetStopReads() && readthreadlive)
     {
-        int ret = rbuffer->Read(buffer, size);
-        if (!rbuffer->GetStopReads() && ret > 0)
-            WriteBlock(readthreadSock, buffer, ret);
+        int request = size - tot;
 
-        size -= ret;
+        if (request > 256000)
+            request = 256000;
+ 
+        ret = rbuffer->Read(buffer, request);
+        
+        if (rbuffer->GetStopReads() || ret <= 0)
+            break;
+        
+        if (!WriteBlock(readthreadSock->socketDevice(), buffer, ret))
+        {
+            tot = -1;
+            break;
+        }
+
+        tot += ret;
+        if (ret < request)
+            break; // we hit eof
     }
-
     readthreadLock.unlock();
+
+    if (ret < 0)
+        tot = -1;
+
+    return tot;
 }
 
 void TVRec::DoFlagCommercialsThread(void)
@@ -2152,14 +2101,15 @@ void TVRec::FlagCommercials(void)
 
 void TVRec::RetrieveInputChannels(map<int, QString> &inputChannel,
                                   map<int, QString> &inputTuneTo,
-                                  map<int, QString> &externalChanger)
+                                  map<int, QString> &externalChanger,
+                                  map<int, QString> &sourceid)
 {
     pthread_mutex_lock(&db_lock);
     MythContext::KickDatabase(db_conn);
 
     QString query = QString("SELECT inputname, trim(externalcommand), "
                             "if(tunechan='', 'Undefined', tunechan), "
-                            "if(startchan, startchan, '') "
+                            "if(startchan, startchan, ''), sourceid "
                             "FROM capturecard, cardinput "
                             "WHERE capturecard.cardid = %1 "
                             "AND capturecard.cardid = cardinput.cardid;")
@@ -2184,6 +2134,7 @@ void TVRec::RetrieveInputChannels(map<int, QString> &inputChannel,
             externalChanger[cap] = result.value(1).toString();
             inputTuneTo[cap] = result.value(2).toString();
             inputChannel[cap] = result.value(3).toString();
+            sourceid[cap] = result.value(4).toString();
         }
     }
 
