@@ -30,6 +30,14 @@ typedef enum
     BROWSE_FAVORITE
 } BrowseDirections;
 
+typedef struct _dvb_options_t
+{
+    int swfilter;
+    int recordts;
+    int wait_for_seqstart;
+    int dmx_buf_size;
+    int pkt_buf_size;
+} dvb_options_t;
 
 class TVRec
 {
@@ -75,11 +83,10 @@ class TVRec
 
     void RetrieveInputChannels(map<int, QString> &inputChannel,
                                map<int, QString> &inputTuneTo,
-                               map<int, QString> &externalChanger);
+                               map<int, QString> &externalChanger,
+                               map<int, QString> &sourceid);
     void StoreInputChannels(map<int, QString> &inputChannel);
 
-    RecorderBase *GetRecorder(void);
-
     bool IsBusy(void);
     bool IsReallyRecording(void);
 
@@ -120,7 +127,7 @@ class TVRec
 
     void UnpauseRingBuffer(void);
     void PauseClearRingBuffer(void);
-    void RequestRingBufferBlock(int size);
+    int RequestRingBufferBlock(int size);
     long long SeekRingBuffer(long long curpos, long long pos, int whence);
 
     bool isParsingCommercials(ProgramInfo *pginfo);
@@ -143,7 +150,7 @@ class TVRec
 
     void GetDevices(int cardnum, QString &video, QString &vbi, QString &audio,
                     int &rate, QString &defaultinput, QString &startchannel,
-                    QString &type, int &dvb_swfilter, int &dvb_recordts);
+                    QString &type, dvb_options_t &dvb_opts);
 
     void ConnectDB(int cardnum);
     void DisconnectDB(void);
@@ -222,8 +229,7 @@ class TVRec
     QString profileName;
     int autoTranscode;
 
-    int dvb_swfilter;
-    int dvb_recordts;
+    dvb_options_t dvb_options;
 };
 
 #endif
