@@ -1,15 +1,15 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/time.h>
 #include <unistd.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
-#include <math.h>
-#include <time.h>
-#include <errno.h>
+#include <cmath>
+#include <ctime>
+#include <cerrno>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/user.h>
@@ -30,6 +30,10 @@ extern "C" {
 
 #include "libmyth/mythcontext.h"
 
+#include "NuppelVideoPlayer.h"
+#include "../libavcodec/avcodec.h"
+#include "yuv2rgb.h"
+
 VideoOutputIvtv::VideoOutputIvtv(void)
 {
     videofd = -1;
@@ -37,23 +41,13 @@ VideoOutputIvtv::VideoOutputIvtv(void)
     pixels = NULL;
     lastcleared = false;
     videoDevice = "/dev/video16";
-    skipplay = false;
-    interruptdisplay = false;
+    last_speed = 1.0;
+    last_normal = true;
 }
 
 VideoOutputIvtv::~VideoOutputIvtv()
 {
-    if (videofd)
-    {
-        ivtv_cfg_stop_decode sd;
-        memset(&sd, 0, sizeof(sd));
-
-        sd.hide_last = 1;
-        ioctl(videofd, IVTV_IOC_S_STOP_DECODE, &sd);
-
-        close(videofd);
-    }
-    videofd = -1;
+    Close();
 
     if (fbfd > 0)
     {
@@ -66,8 +60,9 @@ VideoOutputIvtv::~VideoOutputIvtv()
             return;
         }
 
-        fbstate.status = initglobalalpha;
-        fbstate.alpha = storedglobalalpha;
+        fbstate.status |= IVTVFB_STATUS_GLOBAL_ALPHA;
+        fbstate.status &= ~IVTVFB_STATUS_LOCAL_ALPHA;
+        fbstate.alpha = 255;
 
         if (ioctl(fbfd, IVTVFB_IOCTL_SET_STATE, &fbstate) < 0)
         {
@@ -91,7 +86,8 @@ void VideoOutputIvtv::ClearOSD(void)
         struct ivtv_osd_coords osdcoords;
         memset(&osdcoords, 0, sizeof(osdcoords));
 
-        ioctl(fbfd, IVTVFB_IOCTL_GET_ACTIVE_BUFFER, &osdcoords);
+        if (ioctl(fbfd, IVTVFB_IOCTL_GET_ACTIVE_BUFFER, &osdcoords) < 0)
+            perror("IVTVFB_IOCTL_GET_ACTIVE_BUFFER");
 
         struct ivtvfb_ioctl_dma_host_to_ivtv_args prep;
         memset(&prep, 0, sizeof(prep));
@@ -102,7 +98,8 @@ void VideoOutputIvtv::ClearOSD(void)
 
         memset(osdbuf_aligned, 0x00, osdbufsize);
 
-        ioctl(fbfd, IVTVFB_IOCTL_PREP_FRAME, &prep);
+        if (ioctl(fbfd, IVTVFB_IOCTL_PREP_FRAME, &prep) < 0)
+            perror("IVTVFB_IOCTL_PREP_FRAME");
 
         usleep(20000);
 
@@ -110,7 +107,8 @@ void VideoOutputIvtv::ClearOSD(void)
         osdcoords.offset = 0;
         osdcoords.pixel_stride = XJ_width * 2;
 
-        ioctl(fbfd, IVTVFB_IOCTL_SET_ACTIVE_BUFFER, &osdcoords);
+        if (ioctl(fbfd, IVTVFB_IOCTL_SET_ACTIVE_BUFFER, &osdcoords) < 0)
+            perror("IVTVFB_IOCTL_SET_ACTIVE_BUFFER");
     }
 }
 
@@ -126,9 +124,11 @@ int VideoOutputIvtv::GetRefreshRate(void)
 }
 
 bool VideoOutputIvtv::Init(int width, int height, float aspect, 
-                           unsigned int winid, int winx, int winy, int winw, 
-                           int winh, unsigned int embedid)
+                           WId winid, int winx, int winy, int winw, 
+                           int winh, WId embedid)
 {
+    allowpreviewepg = false;
+
     videoDevice = gContext->GetSetting("PVR350VideoDev");
 
     VideoOutput::Init(width, height, aspect, winid, winx, winy, winw, winh, 
@@ -138,7 +138,7 @@ bool VideoOutputIvtv::Init(int width, int height, float aspect,
 
     MoveResize();
 
-    Reopen();
+    Open();
 
     if (videofd <= 0)
         return false;
@@ -179,9 +179,6 @@ bool VideoOutputIvtv::Init(int width, int height, float aspect,
             return false;
         }
 
-        initglobalalpha = fbstate.status;
-        storedglobalalpha = fbstate.alpha;
-
         fbstate.status &= ~IVTVFB_STATUS_GLOBAL_ALPHA;
         fbstate.status |= IVTVFB_STATUS_LOCAL_ALPHA;
         fbstate.alpha = 0;
@@ -195,13 +192,16 @@ bool VideoOutputIvtv::Init(int width, int height, float aspect,
         struct ivtvfb_ioctl_get_frame_buffer igfb;
         memset(&igfb, 0, sizeof(igfb));
 
-        ioctl(fbfd, IVTVFB_IOCTL_GET_FRAME_BUFFER, &igfb);
+        if (ioctl(fbfd, IVTVFB_IOCTL_GET_FRAME_BUFFER, &igfb) < 0)
+            perror("IVTVFB_IOCTL_GET_FRAME_BUFFER");
 
         stride = igfb.sizex * 4;
 
-        osdbuffer = new char[osdbufsize + PAGE_SIZE];
-        osdbuf_aligned = (char *)((int)osdbuffer + (PAGE_SIZE - 1));
-        osdbuf_aligned = (char *)((int)osdbuf_aligned & PAGE_MASK);
+        long pagesize = sysconf(_SC_PAGE_SIZE);
+        long pagemask = ~(pagesize-1);
+        osdbuffer = new char[osdbufsize + pagesize];
+        osdbuf_aligned = (char *)((int)osdbuffer + (pagesize - 1));
+        osdbuf_aligned = (char *)((int)osdbuf_aligned & pagemask);
 
         memset(osdbuf_aligned, 0x00, osdbufsize);
 
@@ -212,66 +212,46 @@ bool VideoOutputIvtv::Init(int width, int height, float aspect,
     return true;
 }
 
-void VideoOutputIvtv::Reopen(int skipframes, int newstartframe)
+void VideoOutputIvtv::Close(void)
+{
+    if (videofd < 0)
+        return;
+
+    Stop(true);
+
+    close(videofd);
+    videofd = -1;
+}
+
+void VideoOutputIvtv::Open(void)
 {
     if (videofd >= 0)
-    {
-        ivtv_cfg_stop_decode sd;
-        memset(&sd, 0, sizeof(sd));
+        return;
 
-        ioctl(videofd, IVTV_IOC_S_STOP_DECODE, &sd);
-        close(videofd);
-
-        skipplay = true;
-    }
-
-    videofd = -1;
-
-    if ((videofd = open(videoDevice.ascii(), O_WRONLY | O_LARGEFILE, 0555)) < 0)
+    if ((videofd = open(videoDevice.ascii(), 
+        O_WRONLY | O_LARGEFILE | O_NONBLOCK, 0555)) < 0)
     {
         perror("Cannot open ivtv video out device");
         return;
     }
-    else
-    {
-        struct v4l2_control ctrl;
-        memset(&ctrl, 0, sizeof(ctrl));
 
-        ctrl.id = V4L2_CID_IVTV_DEC_PREBUFFER;
-        ctrl.value = 0;
+    struct v4l2_control ctrl;
+    memset(&ctrl, 0, sizeof(ctrl));
 
-        ioctl(videofd, VIDIOC_S_CTRL, &ctrl);
+    ctrl.id = V4L2_CID_IVTV_DEC_PREBUFFER;
+    ctrl.value = 1;
 
-        ctrl.id = V4L2_CID_IVTV_DEC_NUM_BUFFERS;
-        ctrl.value = 0;
+    if (ioctl(videofd, VIDIOC_S_CTRL, &ctrl) < 0)
+        perror("VIDIOC_S_CTRL prebuffer");
 
-        ioctl(videofd, VIDIOC_S_CTRL, &ctrl);
-        
-        ivtv_cfg_start_decode startd;
-        memset(&startd, 0, sizeof(startd));
+    ctrl.id = V4L2_CID_IVTV_DEC_NUM_BUFFERS;
+    ctrl.value = 1;
 
-        if (skipframes > 0)
-        {
-            startd.gop_offset = skipframes;
-            startd.muted_audio_frames = 6;
-        }
-
-        ioctl(videofd, IVTV_IOC_S_START_DECODE, &startd);
-
-        // change it back to hide last frame, in case we die unexpectedly.
-        ivtv_cfg_stop_decode stopd;
-        memset(&stopd, 0, sizeof(stopd));
-
-        stopd.hide_last = 1;
-        ioctl(videofd, IVTV_IOC_S_STOP_DECODE, &stopd);
-    }
-
-    startframenum = newstartframe;
-    firstframe = true;
+    if (ioctl(videofd, VIDIOC_S_CTRL, &ctrl) < 0)
+        perror("VIDIOC_S_CTRL numbuffers");
 }
 
-void VideoOutputIvtv::EmbedInWidget(unsigned long wid, int x, int y, int w, 
-                                    int h)
+void VideoOutputIvtv::EmbedInWidget(WId wid, int x, int y, int w, int h)
 {
     if (embedding)
         return;
@@ -282,8 +262,11 @@ void VideoOutputIvtv::EmbedInWidget(unsigned long wid, int x, int y, int w,
     if (ioctl(fbfd, IVTVFB_IOCTL_GET_STATE, &fbstate) < 0)
         perror("IVTVFB_IOCTL_GET_STATE");
 
-    fbstate.status = initglobalalpha;
-    fbstate.alpha = 164;
+    fbstate.status |= IVTVFB_STATUS_GLOBAL_ALPHA;
+    fbstate.status &= ~IVTVFB_STATUS_LOCAL_ALPHA;
+
+    int usealpha = gContext->GetNumSetting("PVR350EPGAlphaValue", 164);
+    fbstate.alpha = usealpha;
 
     if (ioctl(fbfd, IVTVFB_IOCTL_SET_STATE, &fbstate) < 0)
         perror("IVTVFB_IOCTL_SET_STATE");
@@ -329,13 +312,94 @@ void VideoOutputIvtv::UpdatePauseFrame(void)
 { 
 }
 
+void VideoOutputIvtv::ShowPip(VideoFrame *frame, NuppelVideoPlayer *pipplayer)
+{
+    if (!pipplayer)
+        return;
+
+    int pipw, piph;
+
+    VideoFrame *pipimage = pipplayer->GetCurrentFrame(pipw, piph);
+
+    if (!pipimage || !pipimage->buf || pipimage->codec != FMT_YV12)
+    {
+        pipplayer->ReleaseCurrentFrame(pipimage);
+        return;
+    }
+
+    int xoff;
+    int yoff;
+
+    unsigned char *pipbuf = pipimage->buf;
+
+    if (pipw != desired_pipw || piph != desired_piph)
+    {
+        DoPipResize(pipw, piph);
+
+        if (piptmpbuf && pipscontext)
+        {
+            AVPicture img_in, img_out;
+
+            avpicture_fill(&img_out, (uint8_t *)piptmpbuf, PIX_FMT_YUV420P,
+                           pipw_out, piph_out);
+            avpicture_fill(&img_in, (uint8_t *)pipimage->buf, PIX_FMT_YUV420P,
+                           pipw, piph);
+
+            img_resample(pipscontext, &img_out, &img_in);
+
+            pipw = pipw_out;
+            piph = piph_out;
+
+            pipbuf = piptmpbuf;
+        }
+    }
+
+    switch (PIPLocation)
+    {
+        default:
+        case kPIPTopLeft:
+                xoff = 50;
+                yoff = 40;
+                break;
+        case kPIPBottomLeft:
+                xoff = 50;
+                yoff = frame->height - piph - 40;
+                break;
+        case kPIPTopRight:
+                xoff = frame->width - pipw - 50;
+                yoff = 40;
+                break;
+        case kPIPBottomRight:
+                xoff = frame->width - pipw - 50;
+                yoff = frame->height - piph - 40;
+                break;
+    }
+
+    unsigned char *outputbuf = new unsigned char[pipw * piph * 4];
+    yuv2rgb_fun convert = yuv2rgb_init_mmx(32, MODE_RGB);
+
+    convert(outputbuf, pipbuf, pipbuf + (pipw * piph), 
+            pipbuf + (pipw * piph * 5 / 4), pipw, piph,
+            pipw * 4, pipw, pipw / 2, 1);
+
+    pipplayer->ReleaseCurrentFrame(pipimage);
+
+    if (frame->width < 0)
+        frame->width = XJ_width;
+
+    for (int i = 0; i < piph; i++)
+    {
+        memcpy(frame->buf + (i + yoff) * frame->width + xoff * 4,
+               outputbuf + i * pipw * 4, pipw * 4);
+    }
+}
+
 void VideoOutputIvtv::ProcessFrame(VideoFrame *frame, OSD *osd,
                                    FilterChain *filterList, 
                                    NuppelVideoPlayer *pipPlayer) 
 { 
     (void)filterList;
     (void)frame;
-    (void)pipPlayer;
 
     if (fbfd > 0 && osd)
     {
@@ -344,7 +408,9 @@ void VideoOutputIvtv::ProcessFrame(VideoFrame *frame, OSD *osd,
         VideoFrame tmpframe;
         tmpframe.codec = FMT_ARGB32;
         tmpframe.buf = (unsigned char *)osdbuf_aligned;
-
+        tmpframe.width = stride;
+        tmpframe.height = XJ_height;
+        
         int ret = DisplayOSD(&tmpframe, osd, stride);
 
         if (ret < 0 && !lastcleared)
@@ -354,6 +420,13 @@ void VideoOutputIvtv::ProcessFrame(VideoFrame *frame, OSD *osd,
             drawanyway = true;
         }
 
+        if (pipPlayer)
+        {
+            ShowPip(&tmpframe, pipPlayer);
+            drawanyway = true;
+            lastcleared = false;
+        }
+
         if (ret >= 0)
             lastcleared = false;
 
@@ -366,89 +439,155 @@ void VideoOutputIvtv::ProcessFrame(VideoFrame *frame, OSD *osd,
             prep.dest_offset = 0;
             prep.count = XJ_height * stride;
 
-            ioctl(fbfd, IVTVFB_IOCTL_PREP_FRAME, &prep);
+            if (ioctl(fbfd, IVTVFB_IOCTL_PREP_FRAME, &prep) < 0)
+                perror("IVTVFB_IOCTL_PREP_FRAME");
         }
     }
 }
 
-void VideoOutputIvtv::Play()
+void VideoOutputIvtv::Start(int skip, int mute)
 {
-    if (!skipplay)
-        ioctl(videofd, IVTV_IOC_PLAY, 0);
-    skipplay = false;
+    struct ivtv_cfg_start_decode start;
+    memset(&start, 0, sizeof start);
+    start.gop_offset = skip;
+    start.muted_audio_frames = mute;
+
+    while (ioctl(videofd, IVTV_IOC_START_DECODE, &start) < 0)
+    {
+        if (errno != EBUSY)
+        {
+            perror("IVTV_IOC_START_DECODE");
+            break;
+        }
+    }
 }
 
-void VideoOutputIvtv::Pause()
+void VideoOutputIvtv::Stop(bool hide)
 {
-    ioctl(videofd, IVTV_IOC_PAUSE, 0);
+    struct ivtv_cfg_stop_decode stop;
+    memset(&stop, 0, sizeof stop);
+    stop.hide_last = hide;
+
+    while (ioctl(videofd, IVTV_IOC_STOP_DECODE, &stop) < 0)
+    {
+        if (errno != EBUSY)
+        {
+            perror("IVTV_IOC_STOP_DECODE");
+            break;
+        }
+    }
 }
 
-void VideoOutputIvtv::InterruptDisplay(void)
+void VideoOutputIvtv::Pause(void)
 {
-    interruptdisplay = true;
+    while (ioctl(videofd, IVTV_IOC_PAUSE, 0) < 0)
+    {
+        if (errno != EBUSY)
+        {
+            perror("IVTV_IOC_PAUSE");
+            break;
+        }
+    }
 }
 
-int VideoOutputIvtv::WriteBuffer(unsigned char *buf, int count, int &frames)
+void VideoOutputIvtv::Poll(int delay)
 {
-    int n = 0;
-
     struct pollfd polls;
     polls.fd = videofd;
     polls.events = POLLOUT;
     polls.revents = 0;
 
-    int ret = 0;
-    int totalpassed = count;
-    const int maxwrite = 32768;
-    
-    while (count > 0 && !interruptdisplay)
+    if (poll(&polls, 1, delay) < 0)
+        perror("Polling on videodev");
+}
+
+int VideoOutputIvtv::WriteBuffer(unsigned char *buf, int len)
+{
+    int count;
+
+    //cerr << "ivtv writing video... ";
+    count = write(videofd, buf, len);
+    if (count < 0)
     {
-        ret = poll(&polls, 1, 20);
-
-        if (ret == 1 && polls.revents & POLLOUT)
+        if (errno != EAGAIN)
         {
-            n = write(videofd, buf, (count > maxwrite) ? maxwrite : count);
-            if (n < 0)
-            {
-                perror("Writing to videodev");
-                return n;
-            }
-            count -= n;
-            buf += n;
+            perror("Writing to videodev");
+            return count;
         }
+        count = 0;
+    }
+    //cerr << "wrote " << count << " of " << len << endl;
 
-        if (interruptdisplay)
+    return count;
+}
+
+int VideoOutputIvtv::GetFramesPlayed(void)
+{
+    struct ivtv_ioctl_framesync frameinfo;
+    memset(&frameinfo, 0, sizeof frameinfo);
+
+    if (ioctl(videofd, IVTV_IOC_GET_TIMING, &frameinfo) < 0)
+        perror("IVTV_IOC_GET_TIMING");
+
+    return frameinfo.frame;
+}
+
+bool VideoOutputIvtv::Play(float speed, bool normal)
+{
+    if (speed > 3.0)
+        return false;
+
+    struct ivtv_speed play;
+    memset(&play, 0, sizeof play);
+    if (speed >= 2.0)
+        play.scale = (int)roundf(speed);
+    else if (speed <= 0.5)
+        play.scale = (int)roundf(1 / speed);
+    else
+        play.scale = 1;
+    play.smooth = 0;
+    play.speed = (speed > 1.0);
+    play.direction = 0;
+    play.fr_mask = 2;
+    play.b_per_gop = 0;
+    play.aud_mute = !normal;
+    play.fr_field = 0;
+    play.mute = 0;
+
+    while (ioctl(videofd, IVTV_IOC_S_SPEED, &play) < 0)
+    {
+        if (errno != EBUSY)
         {
+            perror("IVTV_IOC_S_SPEED");
             break;
         }
     }
 
-    interruptdisplay = false;
-       
-    struct ivtv_ioctl_framesync frameinfo;
-    memset(&frameinfo, 0, sizeof(frameinfo));
-    if (ioctl(videofd, IVTV_IOC_GET_TIMING, &frameinfo) < 0)
-    {
-        perror("IVTV_IOC_FRAMESYNC");
-        frames = 0;
-        return (totalpassed - count);
-    }
+    last_speed = speed;
+    last_normal = normal;
 
-    // seems the decoder doesn't initialize this properly.
-    if (frameinfo.frame >= 20000000)
-        frameinfo.frame = 0;
+    return true;
+}
 
-    if (firstframe)
+void VideoOutputIvtv::Flush(void)
+{
+    int arg = 0;
+
+    if (ioctl(videofd, IVTV_IOC_DEC_FLUSH, &arg) < 0)
+        perror("IVTV_IOC_DEC_FLUSH");
+}
+
+void VideoOutputIvtv::Step(void)
+{
+    int arg = 0;
+
+    while (ioctl(videofd, IVTV_IOC_DEC_STEP, &arg) < 0)
     {
-        if (frameinfo.frame > 5)
-            frameinfo.frame = 0;
-        else
-            firstframe = false;
+        if (errno != EBUSY)
+        {
+            perror("IVTV_IOC_DEC_STEP");
+            break;
+        }
     }
-
-    interruptdisplay = false;
-    
-    frames = frameinfo.frame + startframenum;
-    return totalpassed - count;
 }
 
