@@ -14,14 +14,14 @@ class VideoOutputIvtv: public VideoOutput
     VideoOutputIvtv();
    ~VideoOutputIvtv();
 
-    bool Init(int width, int height, float aspect, unsigned int winid,
-              int winx, int winy, int winw, int winh, unsigned int embedid = 0);
+    bool Init(int width, int height, float aspect, WId winid,
+              int winx, int winy, int winw, int winh, WId embedid = 0);
     void PrepareFrame(VideoFrame *buffer);
     void Show(void);
 
     void InputChanged(int width, int height, float aspect);
 
-    void EmbedInWidget(unsigned long wid, int x, int y, int w, int h);
+    void EmbedInWidget(WId wid, int x, int y, int w, int h);
     void StopEmbedding(void);
 
     int GetRefreshRate(void);
@@ -33,32 +33,36 @@ class VideoOutputIvtv: public VideoOutput
                       FilterChain *filterList,
                       NuppelVideoPlayer *pipPlayer);
 
-    int WriteBuffer(unsigned char *buf, int count, int &frames);
+    int WriteBuffer(unsigned char *buf, int count);
+    void Poll(int delay);
     void Pause(void);
-    void Play(void);
+    void Start(int skip, int mute);
+    void Stop(bool hide);
 
-    void Reopen(int skipframes = 0, int newstartframe = 0);
+    void Open(void);
+    void Close(void);
 
     void SetFPS(float lfps) { fps = lfps; }
 
     void ClearOSD(void);
 
-    void InterruptDisplay(void);
+    bool Play(float speed, bool normal);
+    bool Play(void) { return Play(last_speed, last_normal); };
+    void Flush(void);
+    void Step(void);
+    int GetFramesPlayed(void);
 
   private:
+    void ShowPip(VideoFrame *frame, NuppelVideoPlayer *pipplayer);
+ 
     int videofd;
     int fbfd;
 
-    int storedglobalalpha;
-    unsigned long initglobalalpha;
-
     float fps;
     QString videoDevice;
 
     QMutex lock;
 
-    long long startframenum;
-
     int mapped_offset;
     int mapped_memlen;
     char *mapped_mem;
@@ -71,12 +75,10 @@ class VideoOutputIvtv: public VideoOutput
     char *osdbuffer;
     char *osdbuf_aligned;
 
-    bool firstframe;
-
     int osdbufsize;
 
-    bool skipplay;
-    bool interruptdisplay;
+    float last_speed;
+    bool last_normal;
 };
 
 #endif
