@@ -58,8 +58,8 @@ int VideoOutputNull::GetRefreshRate(void)
 }
 
 bool VideoOutputNull::Init(int width, int height, float aspect,
-                           unsigned int winid, int winx, int winy, int winw, 
-                           int winh, unsigned int embedid)
+                           WId winid, int winx, int winy, int winw, 
+                           int winh, WId embedid)
 {
     VideoOutput::InitBuffers(kNumBuffers, true, kNeedFreeFrames, 
                              kPrebufferFrames, kKeepPrebuffer);
@@ -92,7 +92,10 @@ bool VideoOutputNull::CreateNullBuffers(void)
         vbuffers[i].bpp = 12;
         vbuffers[i].size = XJ_height * XJ_width * 3 / 2;
         vbuffers[i].codec = FMT_YV12;
-        vbuffers[i].buf = new unsigned char[vbuffers[i].size];
+        vbuffers[i].buf = new unsigned char[vbuffers[i].size + 64];
+        memset(vbuffers[i].buf, 0, XJ_height * XJ_width);
+        memset(vbuffers[i].buf + XJ_height * XJ_width, 127, 
+               XJ_height * XJ_width / 2);
     }
 
     return true;
@@ -117,8 +120,7 @@ void VideoOutputNull::DeleteNullBuffers()
     }
 }
 
-void VideoOutputNull::EmbedInWidget(unsigned long wid, int x, int y, int w, 
-                                    int h)
+void VideoOutputNull::EmbedInWidget(WId wid, int x, int y, int w, int h)
 {
     if (embedding)
         return;
