@@ -1,16 +1,16 @@
 /* Based on xqcam.c by Paul Chinn <loomer@svpal.org> */
  
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/time.h>
 #include <unistd.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
-#include <math.h>
-#include <time.h>
+#include <cmath>
+#include <ctime>
 
 #include <map>
 #include <iostream>
@@ -39,6 +39,8 @@ extern "C" {
 #include "yuv2rgb.h"
 
 extern "C" {
+#include <X11/extensions/Xinerama.h>
+
 extern int      XShmQueryExtension(Display*);
 extern int      XShmGetEventBase(Display*);
 extern XvImage  *XvShmCreateImage(Display*, XvPortID, int, char*, int, int, XShmSegmentInfo*);
@@ -175,8 +177,8 @@ int VideoOutputXv::GetRefreshRate(void)
 }
 
 bool VideoOutputXv::Init(int width, int height, float aspect, 
-                         unsigned int winid, int winx, int winy, int winw, 
-                         int winh, unsigned int embedid)
+                         WId winid, int winx, int winy, int winw, 
+                         int winh, WId embedid)
 {
     pthread_mutex_init(&lock, NULL);
 
@@ -188,6 +190,8 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
                  p_error_base;
     int p_num_adaptors;
     int w_mm, h_mm;
+    bool usingXinerama;
+    int event_base, error_base;
 
     XvAdaptorInfo *ai;
 
@@ -208,7 +212,10 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
 
     w_mm = DisplayWidthMM(data->XJ_disp, XJ_screen_num);
     h_mm = DisplayHeightMM(data->XJ_disp, XJ_screen_num);
-    if (w_mm == 0 || h_mm == 0)
+    usingXinerama = 
+        (XineramaQueryExtension(data->XJ_disp, &event_base, &error_base) &&
+         XineramaIsActive(data->XJ_disp));
+    if (w_mm == 0 || h_mm == 0 || usingXinerama)
         data->display_aspect = XJ_aspect;
     else
         data->display_aspect = (float)w_mm/h_mm;
@@ -405,7 +412,8 @@ bool VideoOutputXv::CreateXvBuffers(void)
                                           colorid, 0, XJ_width, XJ_height,
                                           &(data->XJ_SHMInfo)[i]);
 
-        (data->XJ_SHMInfo)[i].shmid = shmget(IPC_PRIVATE, image->data_size,
+        (data->XJ_SHMInfo)[i].shmid = shmget(IPC_PRIVATE, 
+                                             image->data_size + 64,
                                              IPC_CREAT|0777);
 
         if ((data->XJ_SHMInfo)[i].shmid < 0)
@@ -488,7 +496,7 @@ bool VideoOutputXv::CreateShmBuffers(void)
         vbuffers[i].bpp = 12;
         vbuffers[i].size = XJ_height * XJ_width * 3 / 2;
         vbuffers[i].codec = FMT_YV12;
-        vbuffers[i].buf = new unsigned char[vbuffers[i].size];
+        vbuffers[i].buf = new unsigned char[vbuffers[i].size + 64];
         vbuffers[i].qscale_table = NULL;
         vbuffers[i].qstride = 0;
     }
@@ -516,7 +524,7 @@ bool VideoOutputXv::CreateXBuffers(void)
         vbuffers[i].bpp = 12;
         vbuffers[i].size = XJ_height * XJ_width * 3 / 2;
         vbuffers[i].codec = FMT_YV12;
-        vbuffers[i].buf = new unsigned char[vbuffers[i].size];
+        vbuffers[i].buf = new unsigned char[vbuffers[i].size + 64];
         vbuffers[i].qscale_table = NULL;
         vbuffers[i].qstride = 0;
     }
@@ -619,7 +627,7 @@ void VideoOutputXv::DeleteXBuffers()
     }
 }
 
-void VideoOutputXv::EmbedInWidget(unsigned long wid, int x, int y, int w, int h)
+void VideoOutputXv::EmbedInWidget(WId wid, int x, int y, int w, int h)
 {
     if (embedding)
         return;
@@ -736,6 +744,8 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer)
                            width, height);
             scontext = img_resample_init(dispw, disph, width, height);
             img_resample(scontext, &image_out, &image_in);
+
+            img_resample_close(scontext);
         }
 
         switch (data->fallbackImage->bits_per_pixel)
