@@ -1,14 +1,14 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/time.h>
 #include <unistd.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
-#include <math.h>
-#include <time.h>
+#include <cmath>
+#include <ctime>
 
 #include <map>
 #include <iostream>
@@ -16,7 +16,7 @@ using namespace std;
 
 #include "videoout_xvmc.h"
 #include "../libmyth/util.h"
-#include "mythcontext.h"
+#include "../libmyth/mythcontext.h"
 
 extern "C" {
 #include "../libavcodec/avcodec.h"
@@ -34,6 +34,8 @@ extern "C" {
 #include <X11/extensions/xf86vmode.h>
 
 extern "C" {
+#include <X11/extensions/Xinerama.h>
+
 extern int      XShmQueryExtension(Display*);
 extern int      XShmGetEventBase(Display*);
 extern XvImage  *XvShmCreateImage(Display*, XvPortID, int, char*, int, int, XShmSegmentInfo*);
@@ -42,7 +44,7 @@ extern XvImage  *XvShmCreateImage(Display*, XvPortID, int, char*, int, int, XShm
 const int kNumBuffers = 7;
 const int kPrebufferFrames = 4;
 const int kNeedFreeFrames = 2;
-const int kKeepPrebuffer = 2;
+const int kKeepPrebuffer = 1;
 
 #define NO_SUBPICTURE 0
 #define OVERLAY_SUBPICTURE 1
@@ -158,8 +160,8 @@ int VideoOutputXvMC::GetRefreshRate(void)
 }
 
 bool VideoOutputXvMC::Init(int width, int height, float aspect,
-                           unsigned int winid, int winx, int winy, int winw, 
-                           int winh, unsigned int embedid)
+                           WId winid, int winx, int winy, int winw, 
+                           int winh, WId embedid)
 {
     pthread_mutex_init(&lock, NULL);
 
@@ -170,6 +172,8 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
     unsigned int p_version, p_release, p_request_base, p_event_base, 
                  p_error_base;
     int p_num_adaptors, w_mm, h_mm;
+    bool usingXinerama;
+    int event_base, error_base;
 
     XvAdaptorInfo *ai;
 
@@ -190,7 +194,10 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
 
     w_mm = DisplayWidthMM(data->XJ_disp, XJ_screen_num);
     h_mm = DisplayHeightMM(data->XJ_disp, XJ_screen_num);
-    if (w_mm == 0 || h_mm == 0)
+    usingXinerama = 
+        (XineramaQueryExtension(data->XJ_disp, &event_base, &error_base) &&
+         XineramaIsActive(data->XJ_disp));
+    if (w_mm == 0 || h_mm == 0 || usingXinerama)
         data->display_aspect = XJ_aspect;
     else
         data->display_aspect = (float)w_mm/h_mm;
@@ -616,8 +623,7 @@ void VideoOutputXvMC::DeleteXvMCBuffers()
     XvMCDestroyContext(data->XJ_disp, &data->ctx);
 }
 
-void VideoOutputXvMC::EmbedInWidget(unsigned long wid, int x, int y, int w, 
-                                    int h)
+void VideoOutputXvMC::EmbedInWidget(WId wid, int x, int y, int w, int h)
 {
     if (embedding)
         return;
