@@ -1,12 +1,17 @@
 #ifndef VIDEOOUTBASE_H_
 #define VIDEOOUTBASE_H_
 
+extern "C" {
 #include "frame.h"
 #include "filter.h"
+}
 
+#include <qframe.h>
 #include <qmutex.h>
 #include <qmap.h>
 #include <qptrqueue.h>
+#include <qwaitcondition.h>
+#include <qptrlist.h>
 
 using namespace std;
 
@@ -15,12 +20,17 @@ class OSD;
 class OSDSurface;
 class FilterChain;
 
+extern "C" {
+struct ImgReSampleContext;
+}
+
 enum VideoOutputType
 {
     kVideoOutput_Default = 0,
     kVideoOutput_XvMC,
     kVideoOutput_VIA,
-    kVideoOutput_IVTV
+    kVideoOutput_IVTV,
+    kVideoOutput_Directfb
 };
 
 enum PictureAttribute
@@ -51,8 +61,8 @@ class VideoOutput
                      int needprebuffer);
 
     virtual bool Init(int width, int height, float aspect,
-                      unsigned int winid, int winx, int winy, int winw, 
-                      int winh, unsigned int embedid = 0);
+                      WId winid, int winx, int winy, int winw, 
+                      int winh, WId embedid = 0);
 
     virtual void PrepareFrame(VideoFrame *buffer) = 0;
     virtual void Show(void) = 0;
@@ -60,7 +70,7 @@ class VideoOutput
     virtual void InputChanged(int width, int height, float aspect);
     virtual void AspectChanged(float aspect);
 
-    virtual void EmbedInWidget(unsigned long wid, int x, int y, int w, int h);
+    virtual void EmbedInWidget(WId wid, int x, int y, int w, int h);
     virtual void StopEmbedding(void);
 
     virtual void MoveResize(void);
@@ -82,6 +92,9 @@ class VideoOutput
     int FreeVideoFrames(void);
 
     bool EnoughFreeFrames(void);
+    QWaitCondition *availableVideoBuffersWait(void) 
+                    { return &availableVideoBuffers_wait; }
+
     bool EnoughDecodedFrames(void);
     bool EnoughPrebufferedFrames(void);
     
@@ -114,11 +127,13 @@ class VideoOutput
     int GetCurrentColour(void) { return colour; }
     int GetCurrentHue(void) { return hue; }
 
+    bool AllowPreviewEPG(void) { return allowpreviewepg; }
+
   protected:
     void InitBuffers(int numdecode, bool extra_for_pause, int need_free,
                      int needprebuffer, int keepprebuffer);
 
-    void ShowPip(VideoFrame *frame, NuppelVideoPlayer *pipplayer);
+    virtual void ShowPip(VideoFrame *frame, NuppelVideoPlayer *pipplayer);
     int DisplayOSD(VideoFrame *frame, OSD *osd, int stride = -1);
 
     void BlendSurfaceToYV12(OSDSurface *surface, unsigned char *yuvptr,
@@ -132,6 +147,9 @@ class VideoOutput
 
     void CopyFrame(VideoFrame* to, VideoFrame* from);
 
+    void DoPipResize(int pipwidth, int pipheight);
+    void ShutdownPipResize(void);
+
     int XJ_width, XJ_height;
     float XJ_aspect;
 
@@ -160,6 +178,7 @@ class VideoOutput
     QMap<VideoFrame *, int> vbufferMap;
     QPtrQueue<VideoFrame> availableVideoBuffers;
     QPtrQueue<VideoFrame> usedVideoBuffers;
+    QPtrList<VideoFrame> busyVideoBuffers;
 
     int rpos;
     int vpos;
@@ -169,6 +188,23 @@ class VideoOutput
     int keepprebufferframes;
 
     bool needrepaint;
+
+    QWaitCondition availableVideoBuffers_wait;
+
+#define MAX_NEG_CROP 384
+    unsigned char cropTbl[256 + 2 * MAX_NEG_CROP];
+    unsigned char *cm;
+
+    int desired_piph;
+    int desired_pipw;
+
+    int piph_in, piph_out;
+    int pipw_in, pipw_out;
+    unsigned char *piptmpbuf;
+
+    ImgReSampleContext *pipscontext;
+
+    bool allowpreviewepg;
 };
 
 #endif
