@@ -25,7 +25,6 @@ using namespace std;
 #include "libmythtv/dbcheck.h"
 
 QMap<int, EncoderLink *> tvList;
-MythContext *gContext;
 AutoExpire *expirer = NULL;
 Scheduler *sched = NULL;
 Transcoder *trans = NULL;
@@ -85,6 +84,7 @@ bool setupTVs(bool ismaster)
     else
     {
         cerr << "ERROR: no capture cards are defined in the database.\n";
+        cerr << "Perhaps you should read the installation instructions?\n";
         return false;
     }
 
@@ -115,6 +115,7 @@ int main(int argc, char **argv)
     QApplication a(argc, argv, false);
 
     QString logfile = "";
+    QString binname = basename(a.argv()[0]);
     QString verboseString = QString(" important general");
 
     bool daemonize = false;
@@ -296,7 +297,8 @@ int main(int argc, char **argv)
          
         if (logfd < 0)
         {
-            perror("open(logfile)");
+            perror(logfile.ascii());
+            cerr << "Error opening logfile\n";
             return -1;
         }
     }
@@ -307,7 +309,8 @@ int main(int argc, char **argv)
         pidfs.open(pidfile.ascii());
         if (!pidfs)
         {
-            perror("open(pidfile)");
+            perror(pidfile.ascii());
+            cerr << "Error opening pidfile";
             return -1;
         }
     }
@@ -343,35 +346,35 @@ int main(int argc, char **argv)
     QSqlDatabase *db = QSqlDatabase::addDatabase("QMYSQL3");
     if (!db)
     {
-        printf("Couldn't connect to database\n");
+        cerr << "Couldn't connect to database\n";
         return -1;
     }
 
     QSqlDatabase *subthread = QSqlDatabase::addDatabase("QMYSQL3", "SUBDB");
     if (!subthread)
     {
-        printf("Couldn't connect to database\n");
+        cerr << "Couldn't connect to database\n";
         return -1;
     }
 
     QSqlDatabase *expthread = QSqlDatabase::addDatabase("QMYSQL3", "EXPDB");
     if (!expthread)
     {
-        printf("Couldn't connect to database\n");
+        cerr << "Couldn't connect to database\n";
         return -1;
     }
 
     QSqlDatabase *transthread = QSqlDatabase::addDatabase("QMYSQL3", "TRANSDB");
     if (!transthread)
     {
-        printf("Couldn't connect to database\n");
+        cerr << "Couldn't connect to database\n";
         return -1;
     }
 
     QSqlDatabase *msdb = QSqlDatabase::addDatabase("QMYSQL3", "MSDB");
     if (!msdb)
     {
-        printf("Couldn't connect to database\n");
+        cerr << "Couldn't connect to database\n";
         return -1;
     }
 
@@ -380,7 +383,7 @@ int main(int argc, char **argv)
         !gContext->OpenDatabase(transthread) ||
         !gContext->OpenDatabase(msdb))
     {
-        printf("couldn't open db\n");
+        cerr << "Couldn't open database\n";
         return -1;
     }
 
@@ -456,10 +459,12 @@ int main(int argc, char **argv)
     QSqlDatabase *trandb = QSqlDatabase::database("TRANSDB");
     trans = new Transcoder(&tvList, trandb);
 
+    VERBOSE(VB_ALL, QString("%1 version: %2 www.mythtv.org")
+                            .arg(binname).arg(MYTH_BINARY_VERSION));
+
     VERBOSE(VB_ALL, QString("Enabled verbose msgs :%1").arg(verboseString));
 
     lockfile_location = gContext->GetSetting("RecordFilePrefix") + "/nfslockfile.lock";
-    int nfsfd = -1;
 
     if (ismaster)
 // Create a file in the recording directory.  A slave encoder will 
@@ -469,17 +474,15 @@ int main(int argc, char **argv)
 // If the slave doesn't find this file then it will assume that it has
 // a seperate store.
     {
-       nfsfd = open(lockfile_location.ascii(), O_WRONLY|O_CREAT|O_APPEND, 0664);
-       if (nfsfd < 0) 
-       {
-           cerr << "Unable to create \'" << lockfile_location << "\'!\n"
-                << "Be sure that \'" << gContext->GetSetting("RecordFilePrefix")
-                << "\' exists and that both \nthe directory and that "
-                << "file are writeble by this user.\n";
-          perror("open lockfile"); 
-          return -1;
-       }
-       close(nfsfd);
+        if (creat(lockfile_location.ascii(), 0664) == -1)
+        {
+            perror(lockfile_location.ascii()); 
+            cerr << "Unable to open lockfile!\n"
+                 << "Be sure that \'" << gContext->GetSetting("RecordFilePrefix")
+                 << "\' exists and that both \nthe directory and that "
+                 << "file are writeble by this user.\n";
+            return -1;
+        }
     }
 
     new MainServer(ismaster, port, statusport, &tvList, msdb, sched);
