@@ -9,7 +9,7 @@
 #include <qregexp.h>
 
 #include <unistd.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <fcntl.h>
 #include <linux/soundcard.h>
 #include <sys/ioctl.h>
@@ -32,18 +32,21 @@ using namespace std;
 class ProcessRequestThread : public QThread
 {
   public:
-    ProcessRequestThread(MainServer *ms) { parent = ms; dostuff = false; }
+    ProcessRequestThread(MainServer *ms) { parent = ms; }
    
     void setup(QSocket *sock)
     {
+        lock.lock();
         socket = sock;
-        dostuff = true;
+        lock.unlock();
         waitCond.wakeOne();
     }
 
     void killit(void)
     {
+        lock.lock();
         threadlives = false;
+        lock.unlock();
         waitCond.wakeOne();
     }
 
@@ -51,18 +54,20 @@ class ProcessRequestThread : public QThread
     {
         threadlives = true;
 
-        while (threadlives)
-        {
-            if (!dostuff)
-                waitCond.wait();
+        lock.lock();
 
-            if (dostuff)
-            {
-                parent->ProcessRequest(socket);
-                dostuff = false;
-                parent->MarkUnused(this);
-            }
+        while (1)
+        {
+            waitCond.wait(&lock);
+
+            if (!threadlives)
+                break;
+
+            parent->ProcessRequest(socket);
+            parent->MarkUnused(this);
         }
+
+        lock.unlock();
     }
 
   private:
@@ -70,8 +75,8 @@ class ProcessRequestThread : public QThread
 
     QSocket *socket;
 
+    QMutex lock;
     QWaitCondition waitCond;
-    bool dostuff;
     bool threadlives;
 };
 
@@ -158,8 +163,7 @@ void MainServer::readSocket(void)
 
         if (!prt)
         {
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " waiting for a thread.." << endl;
+            VERBOSE(VB_ALL, "waiting for a thread..");
             usleep(50);
         }
     }
@@ -183,11 +187,19 @@ void MainServer::ProcessRequest(QSocket *sock)
     line = line.simplifyWhiteSpace();
     QStringList tokens = QStringList::split(" ", line);
     QString command = tokens[0];
-    if (command == "ANN")
+    //cerr << "command='" << command << "'\n";
+    if (command == "MYTH_PROTO_VERSION")
+    {
+        if (tokens.size() < 2)
+            VERBOSE(VB_ALL, "Bad MYTH_PROTO_VERSION command");
+        else
+            HandleVersion(sock,tokens[1]);
+        return;
+    }
+    else if (command == "ANN")
     {
         if (tokens.size() < 3 || tokens.size() > 4)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad ANN query" << endl;
+            VERBOSE(VB_ALL, "Bad ANN query");
         else
             HandleAnnounce(listline, tokens, sock);
         return;
@@ -201,16 +213,14 @@ void MainServer::ProcessRequest(QSocket *sock)
     PlaybackSock *pbs = getPlaybackBySock(sock);
     if (!pbs)
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " Unknown socket" << endl;
+        VERBOSE(VB_ALL, "unknown socket");
         return;
     }
 
     if (command == "QUERY_RECORDINGS")
     {
         if (tokens.size() != 2)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad QUERY_RECORDINGS query" << endl;
+            VERBOSE(VB_ALL, "Bad QUERY_RECORDINGS query");
         else
             HandleQueryRecordings(tokens[1], pbs);
     }
@@ -262,8 +272,7 @@ void MainServer::ProcessRequest(QSocket *sock)
     else if (command == "QUERY_GETCONFLICTING")
     {
         if (tokens.size() != 2)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad QUERY_GETCONFLICTING" << endl;
+            VERBOSE(VB_ALL, "Bad QUERY_GETCONFLICTING");
         else
             HandleGetConflictingRecordings(listline, tokens[1], pbs);
     }
@@ -274,16 +283,14 @@ void MainServer::ProcessRequest(QSocket *sock)
     else if (command == "QUERY_RECORDER")
     {
         if (tokens.size() != 2)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad QUERY_RECORDER" << endl;
+            VERBOSE(VB_ALL, "Bad QUERY_RECORDER");
         else
             HandleRecorderQuery(listline, tokens, pbs);
     }
     else if (command == "QUERY_REMOTEENCODER")
     {
         if (tokens.size() != 2)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad QUERY_REMOTEENCODER" << endl;
+            VERBOSE(VB_ALL, "Bad QUERY_REMOTEENCODER");
         else
             HandleRemoteEncoder(listline, tokens, pbs);
     }
@@ -298,8 +305,7 @@ void MainServer::ProcessRequest(QSocket *sock)
     else if (command == "QUERY_FILETRANSFER")
     {
         if (tokens.size() != 2)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad QUERY_FILETRANSFER" << endl;
+            VERBOSE(VB_ALL, "Bad QUERY_FILETRANSFER");
         else
             HandleFileTransferQuery(listline, tokens, pbs);
     }
@@ -326,16 +332,14 @@ void MainServer::ProcessRequest(QSocket *sock)
     else if (command == "FREE_TUNER")
     {
         if (tokens.size() != 2)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad FREE_TUNER query" << endl;
+            VERBOSE(VB_ALL, "Bad FREE_TUNER query");
         else
             HandleFreeTuner(tokens[1].toInt(), pbs);
     }
     else if (command == "QUERY_IS_ACTIVE_BACKEND")
     {
         if (tokens.size() != 1)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad QUERY_IS_ACTIVE_BACKEND" << endl;
+            VERBOSE(VB_ALL, "Bad QUERY_IS_ACTIVE_BACKEND");
         else
             HandleIsActiveBackendQuery(listline, pbs);
         
@@ -343,8 +347,7 @@ void MainServer::ProcessRequest(QSocket *sock)
     else if (command == "SHUTDOWN_NOW")
     {
         if (tokens.size() != 1)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Bad SHUTDOWN_NOW request" << endl;
+            VERBOSE(VB_ALL, "Bad SHUTDOWN_NOW query");
         else if (!ismaster)
         {
             VERBOSE(VB_ALL, "Going down now as of Mainserver request!");
@@ -360,10 +363,7 @@ void MainServer::ProcessRequest(QSocket *sock)
         gContext->dispatch(me);
     }
     else
-    {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " unknown command: " << command << endl;
-    }
+        VERBOSE(VB_ALL, "Unknown command: " + command);
 }
 
 void MainServer::MarkUnused(ProcessRequestThread *prt)
@@ -428,6 +428,24 @@ void MainServer::customEvent(QCustomEvent *e)
     }
 }
 
+void MainServer::HandleVersion(QSocket *socket,QString version)
+{
+    QStringList retlist;
+    if (version != MYTH_PROTO_VERSION)
+    {
+        VERBOSE(VB_GENERAL,
+                "MainServer::HandleVersion - Client speaks protocol version "
+                + version + " but we speak " + MYTH_PROTO_VERSION + "!");
+        retlist << "REJECT" << MYTH_PROTO_VERSION;
+        WriteStringList(socket, retlist);
+        HandleDone(socket);
+        return;
+    }
+
+    retlist << "ACCEPT" << MYTH_PROTO_VERSION;
+    WriteStringList(socket, retlist);
+}
+
 void MainServer::HandleAnnounce(QStringList &slist, QStringList commands, 
                                 QSocket *socket)
 {
@@ -437,17 +455,15 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
     {
         bool wantevents = commands[3].toInt();
         VERBOSE(VB_GENERAL, "MainServer::HandleAnnounce Playback");
-        cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss") 
-             << " adding: " << commands[2] << " as a player " 
-             << wantevents << endl;
+        VERBOSE(VB_ALL, QString("adding: %1 as a client (events: %2)")
+                               .arg(commands[2]).arg(wantevents));
         PlaybackSock *pbs = new PlaybackSock(socket, commands[2], wantevents);
         playbackList.push_back(pbs);
     }
     else if (commands[1] == "SlaveBackend")
     {
-        cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss") 
-             << " adding: " << commands[2] << " as a slave backend server" 
-             << endl;
+        VERBOSE(VB_ALL, QString("adding: %1 as a slave backend server")
+                               .arg(commands[2]));
         PlaybackSock *pbs = new PlaybackSock(socket, commands[2], false);
         pbs->setAsSlaveBackend();
         pbs->setIP(commands[3]);
@@ -468,17 +484,16 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
     }
     else if (commands[1] == "RingBuffer")
     {
-        cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " adding: " << commands[2] << " as a remote ringbuffer" << endl;
+        VERBOSE(VB_ALL, QString("adding: %1 as a remote ringbuffer")
+                               .arg(commands[2]));
         ringBufList.push_back(socket);
         int recnum = commands[3].toInt();
 
         QMap<int, EncoderLink *>::Iterator iter = encoderList->find(recnum);
         if (iter == encoderList->end())
         {
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Unknown encoder." << endl;
-            exit(0);
+            VERBOSE(VB_ALL, "Unknown encoder.");
+            exit(1);
         }
 
         EncoderLink *enc = iter.data();
@@ -521,24 +536,25 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
                     if (dsp_status != -1)
                     {
                         if (!(soundcardcaps & DSP_CAP_DUPLEX))
-                            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                                 << " WARNING:  Capture device " << audiodevice 
-                                 << " is not reporting full duplex "
-                                 << "capability.\nSee docs/mythtv-HOWTO, "
-                                 << "section 20.13 for more information." << endl;
+                            VERBOSE(VB_ALL, QString(" WARNING:  Capture device"
+                                                    " %1 is not reporting full "
+                                                    "duplex capability.\nSee "
+                                                    "docs/mythtv-HOWTO, section"
+                                                    " 20.13 for more "
+                                                    "information.")
+                                                    .arg(audiodevice));
                     }
                     else 
-                        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                             << " Could not get capabilities for "
-                             << "audio device: " << audiodevice << endl;
+                        VERBOSE(VB_ALL, QString("Could not get capabilities for"
+                                                " audio device: %1")
+                                               .arg(audiodevice));
                     close(dsp_fd);
                 }
                 else 
                 {
-                    cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                         << " Could not open audio device: " 
-                         << audiodevice << endl;
                     perror("open");
+                    VERBOSE(VB_ALL, QString("Could not open audio device: %1")
+                                           .arg(audiodevice));
                 }
             }
         }
@@ -546,9 +562,8 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
     else if (commands[1] == "FileTransfer")
     {
         VERBOSE(VB_GENERAL, "MainServer::HandleAnnounce FileTransfer");
-        cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " adding: " << commands[2] << " as a remote file transfer"
-             << endl;
+        VERBOSE(VB_ALL, QString("adding: %1 as a remote file transfer")
+                               .arg(commands[2]));
         QUrl qurl = slist[1];
         QString filename = LocalFilePath(qurl);
 
@@ -595,7 +610,8 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
     MythContext::KickDatabase(m_db);
 
     thequery = "SELECT recorded.chanid,starttime,endtime,title,subtitle,"
-               "description,hostname,channum,name,callsign FROM recorded "
+               "description,hostname,channum,name,callsign,commflagged,cutlist,"
+               "autoexpire,editing,bookmark,category FROM recorded "
                "LEFT JOIN channel ON recorded.chanid = channel.chanid "
                "ORDER BY starttime";
 
@@ -643,7 +659,7 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
             if (!query.value(7).toString().isNull())
             {
                 proginfo->chanstr = query.value(7).toString();
-                proginfo->channame = query.value(8).toString();
+                proginfo->channame = QString::fromUtf8(query.value(8).toString());
                 proginfo->chansign = query.value(9).toString();
             }
             else
@@ -653,6 +669,21 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
                 proginfo->chansign = "#" + proginfo->chanid;
             }
 
+            // Taken out of programinfo.cpp just to reduce the number of queries
+            int flags = 0;
+            flags |= (query.value(10).toInt() == 1) ? FL_COMMFLAG : 0;
+            flags |= query.value(11).toString().length() > 1 ? FL_CUTLIST : 0;
+            flags |= query.value(12).toInt() ? FL_AUTOEXP : 0;
+            if (query.value(13).toInt() || (query.value(10).toInt() == 2))
+                flags |= FL_EDITING;
+            flags |= query.value(14).toString().length() > 1 ? FL_BOOKMARK : 0;
+
+            proginfo->programflags = flags;
+
+            QString test = query.value(15).toString();
+            if (test != QString::null) 
+               proginfo->category = QString::fromUtf8(test);
+
             QString lpath = proginfo->GetRecordFilename(fileprefix);
             PlaybackSock *slave = NULL;
             QFile checkFile(lpath);
@@ -684,9 +715,9 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
             {
                 if (!slave)
                 {
-                    cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                         << " Couldn't find backend for: " << proginfo->title
-                         << " : " << proginfo->subtitle << endl;
+                    VERBOSE(VB_ALL, QString("Couldn't find backend for: %1 : "
+                                            "%2").arg(proginfo->title)
+                                                 .arg(proginfo->subtitle));
                     proginfo->filesize = 0;
                     proginfo->pathname = "file not found";
                 }
@@ -872,7 +903,11 @@ void MainServer::DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
             num = slave->StopRecording(pginfo);
 
             if (num > 0)
+            {
                 (*encoderList)[num]->StopRecording();
+                pginfo->recstatus = rsStopped;
+                m_sched->AddToDontRecord(pginfo);
+            }
 
             if (pbssock)
             {
@@ -903,6 +938,12 @@ void MainServer::DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
             {
                 usleep(100);
             }
+
+            if (ismaster)
+            {
+                pginfo->recstatus = rsStopped;
+                m_sched->AddToDontRecord(pginfo);
+            }
         }
     }
 
@@ -917,9 +958,6 @@ void MainServer::DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
         return;
     }
 
-    QString thequery;
-    QSqlQuery query;
-
     dblock.lock();
 
     MythContext::KickDatabase(m_db);
@@ -935,13 +973,20 @@ void MainServer::DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
     // Set the recorded end time to the current time
     // (we're stopping the recording so it'll never get to its originally 
     // intended end time)
-    thequery = QString("UPDATE recorded SET starttime = %1, endtime = %2 "
-                       "WHERE chanid = %3 AND title = \"%4\" AND "
-                       "starttime = %5 AND endtime = %6;")
-                       .arg(startts).arg(newendts).arg(pginfo->chanid)
-                       .arg(pginfo->title.utf8()).arg(startts).arg(endts);
+    
+    QSqlQuery query(QString::null, m_db);
+    query.prepare("UPDATE recorded SET starttime = :NEWSTARTTIME, "
+                  "endtime = :NEWENDTIME WHERE chanid = :CHANID AND "
+                  "title = :TITLE AND starttime = :STARTTIME AND "
+                  "endtime = :ENDTIME;");
+    query.bindValue(":NEWSTARTTIME", startts);
+    query.bindValue(":NEWENDTIME", newendts);
+    query.bindValue(":CHANID", pginfo->chanid);
+    query.bindValue(":TITLE", pginfo->title.utf8());
+    query.bindValue(":STARTTIME", startts);
+    query.bindValue(":ENDTIME", endts);
 
-    query = m_db->exec(thequery);
+    query.exec();
 
     if (!query.isActive())
     {
@@ -1016,7 +1061,11 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
            num = slave->DeleteRecording(pginfo);
 
            if (num > 0)
-              (*encoderList)[num]->StopRecording();
+           {
+               (*encoderList)[num]->StopRecording();
+               pginfo->recstatus = rsDeleted;
+               m_sched->AddToDontRecord(pginfo);
+           }
 
            if (pbssock)
            {
@@ -1047,11 +1096,15 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
             {
                 usleep(100);
             }
+
+            if (ismaster)
+            {
+                pginfo->recstatus = rsDeleted;
+                m_sched->AddToDontRecord(pginfo);
+            }
         }
     }
 
-    QString thequery;
-
     QString startts = pginfo->recstartts.toString("yyyyMMddhhmm");
     startts += "00";
     QString endts = pginfo->recendts.toString("yyyyMMddhhmm");
@@ -1061,34 +1114,28 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
 
     MythContext::KickDatabase(m_db);
 
-    thequery = QString("DELETE FROM recorded WHERE chanid = %1 AND title "
-                       "= \"%2\" AND starttime = %3 AND endtime = %4;")
-                       .arg(pginfo->chanid).arg(pginfo->title.utf8())
-                       .arg(startts).arg(endts);
+    QSqlQuery query(QString::null, m_db);
+    query.prepare("DELETE FROM recorded WHERE chanid = :CHANID AND "
+                  "title = :TITLE AND starttime = :STARTTIME AND "
+                  "endtime = :ENDTIME;");
+    query.bindValue(":CHANID", pginfo->chanid);
+    query.bindValue(":TITLE", pginfo->title.utf8());
+    query.bindValue(":STARTTIME", startts);
+    query.bindValue(":ENDTIME", endts);
 
-    QSqlQuery query = m_db->exec(thequery);
+    query.exec();
 
     if (!query.isActive())
-    {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " DB Error: recorded program deletion failed, SQL query "
-             << "was:" << endl;
-        cerr << thequery << endl;
-    }
+        MythContext::DBError("Recorded program deletion", query);
 
     // now delete any markups for this program
-    thequery = QString("DELETE FROM recordedmarkup WHERE chanid = %1 "
-                       "AND starttime = %2;")
-                       .arg(pginfo->chanid).arg(startts);
+    QString thequery = QString("DELETE FROM recordedmarkup WHERE chanid = %1 "
+                               "AND starttime = %2;")
+                              .arg(pginfo->chanid).arg(startts);
 
     query = m_db->exec(thequery);
     if (!query.isActive())
-    {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " DB Error: recorded program deletion failed, SQL query "
-             << "was:" << endl;
-        cerr << thequery << endl;
-    }
+        MythContext::DBError("Recorded program deletion (2)", query);
 
     dblock.unlock();
 
@@ -1110,8 +1157,8 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
     }
     else
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " Strange.  File: " << filename << " doesn't exist." << endl;
+        VERBOSE(VB_ALL, QString("Strange, file: %1 doesn't exist.")
+                               .arg(filename));
 
         if (pbssock)
         {
@@ -1331,10 +1378,10 @@ void MainServer::HandleLockTuner(PlaybackSock *pbs)
             VERBOSE(VB_GENERAL, msg);
 
             QString querystr = QString("SELECT videodevice, audiodevice, "
-                                            "vbidevice "
-                                          "FROM capturecard "
-                                          "WHERE cardid = %1;")
-                                          .arg(retval);
+                                       "vbidevice "
+                                       "FROM capturecard "
+                                       "WHERE cardid = %1;")
+                                       .arg(retval);
 
             QSqlQuery query = m_db->exec(querystr);
 
@@ -1382,8 +1429,7 @@ void MainServer::HandleFreeTuner(int cardid, PlaybackSock *pbs)
     QMap<int, EncoderLink *>::Iterator iter = encoderList->find(cardid);
     if (iter == encoderList->end())
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-        << " Unknown encoder " << cardid << endl;
+        VERBOSE(VB_ALL, QString("Unknown encoder: %1").arg(cardid));
         strlist << "FAILED";
     }
     else
@@ -1482,8 +1528,7 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
     QMap<int, EncoderLink *>::Iterator iter = encoderList->find(recnum);
     if (iter == encoderList->end())
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " Unknown encoder." << endl;
+        VERBOSE(VB_ALL, QString("Unknown encoder: %1").arg(recnum));
         QStringList retlist = "bad";
         SendResponse(pbssock, retlist);
         return;
@@ -1535,7 +1580,11 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
         for (int keynum = start; keynum <= end; keynum++)
         {
             long long value = enc->GetKeyframePosition(keynum);
-            encodeLongLong(retlist, value);
+            if (value != -1) 
+            {
+                encodeLongLong(retlist, keynum);
+                encodeLongLong(retlist, value);
+            }
         }
     }
     else if (command == "SETUP_RING_BUFFER")
@@ -1760,8 +1809,8 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
     {
         int size = slist[2].toInt();
 
-        enc->RequestRingBufferBlock(size);
-        retlist << QString::number(false);
+        int ret = enc->RequestRingBufferBlock(size);
+        retlist << QString::number(ret);
     }
     else if (command == "SEEK_RINGBUF")
     {
@@ -1780,8 +1829,7 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
     }
     else
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " unknown command: " << command << endl;
+        VERBOSE(VB_ALL, QString("Unknown command: %1").arg(command));
         retlist << "ok";
     }
 
@@ -1798,8 +1846,7 @@ void MainServer::HandleRemoteEncoder(QStringList &slist, QStringList &commands,
     QMap<int, EncoderLink *>::Iterator iter = encoderList->find(recnum);
     if (iter == encoderList->end())
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " Unknown encoder." << endl;
+        VERBOSE(VB_ALL, QString("Unknown encoder: %1").arg(recnum));
         exit(0);
     }
 
@@ -1880,8 +1927,8 @@ void MainServer::HandleFileTransferQuery(QStringList &slist,
     FileTransfer *ft = getFileTransferByID(recnum);
     if (!ft)
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " Unknown file transfer socket" << endl;
+        VERBOSE(VB_ALL, QString("Unknown file transfer socket: %1")
+                               .arg(recnum));
         exit(0);
     }
 
@@ -1917,8 +1964,7 @@ void MainServer::HandleFileTransferQuery(QStringList &slist,
     }
     else 
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " Unknown command: " << command << endl;
+        VERBOSE(VB_ALL, QString("Unknown command: %1").arg(command));
         retlist << "ok";
     }
 
@@ -2060,9 +2106,8 @@ void MainServer::HandleGenPreviewPixmap(QStringList &slist, PlaybackSock *pbs)
         PlaybackSock *slave = getSlaveByHostname(pginfo->hostname);
 
         if (!slave)
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " Couldn't find backend for: " << pginfo->title
-                 << " : " << pginfo->subtitle << endl;
+            VERBOSE(VB_ALL, QString("Couldn't find backend for: %1 : %2")
+                                   .arg(pginfo->title).arg(pginfo->subtitle));
         else
             slave->GenPreviewPixmap(pginfo);
 
@@ -2074,9 +2119,8 @@ void MainServer::HandleGenPreviewPixmap(QStringList &slist, PlaybackSock *pbs)
 
     if (pginfo->hostname != gContext->GetHostName())
     {
-        cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " Somehow we got a request to generate a preview pixmap for: "
-             << pginfo->hostname << endl;
+        VERBOSE(VB_ALL, QString("Got requested to generate preview pixmap "
+                                "for %1").arg(pginfo->hostname));
 
         QStringList outputlist = "BAD";
         SendResponse(pbssock, outputlist);
@@ -2128,9 +2172,8 @@ void MainServer::endConnection(QSocket *socket)
         {
             if (ismaster && (*it)->isSlaveBackend())
             {
-                cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                     << " Slave backend: " << (*it)->getHostname() 
-                     << " has left the building\n";
+                VERBOSE(VB_ALL, QString("Slave backend: %1 has left the "
+                                        "building").arg((*it)->getHostname()));
 
                 QMap<int, EncoderLink *>::Iterator iter = encoderList->begin();
                 for (; iter != encoderList->end(); ++iter)
@@ -2196,8 +2239,7 @@ void MainServer::endConnection(QSocket *socket)
         }
     }
 
-    cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-         << " unknown socket" << endl;
+    VERBOSE(VB_ALL, "unknown socket");
 }
 
 PlaybackSock *MainServer::getSlaveByHostname(QString &hostname)
@@ -2333,9 +2375,8 @@ void MainServer::reconnectTimeout(void)
     QString server = gContext->GetSetting("MasterServerIP", "localhost");
     int port = gContext->GetNumSetting("MasterServerPort", 6543);
 
-    cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss") 
-         << " - Connecting to master server: " << server << ":" << 
-            port << endl;
+    VERBOSE(VB_ALL, QString("Connecting to master server: %1:%2")
+                           .arg(server).arg(port));
 
     masterServerSock->connectToHost(server, port);
 
@@ -2348,8 +2389,7 @@ void MainServer::reconnectTimeout(void)
         num++;
         if (num > 100)
         {
-            cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-                 << " - Connection to master server timed out." << endl;
+            VERBOSE(VB_ALL, "Connection to master server timed out.");
             masterServerReconnect->start(1000, true);
             delete masterServerSock;
             return;
@@ -2358,15 +2398,13 @@ void MainServer::reconnectTimeout(void)
 
     if (masterServerSock->state() != QSocket::Connected)
     {
-        cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
-             << " - Could not connect to master server." << endl;
+        VERBOSE(VB_ALL, "Could not connect to master server.");
         masterServerReconnect->start(1000, true);
         delete masterServerSock;
         return;
     }
 
-    cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss") 
-         << " - Connected." << endl;
+    VERBOSE(VB_ALL, "Connected successfully");
 
     QString str = QString("ANN SlaveBackend %1 %2")
                           .arg(gContext->GetHostName())
@@ -2757,16 +2795,17 @@ void MainServer::PrintStatus(QSocket *socket)
 
     mfdLastRunStart = gContext->GetSetting("mythfilldatabaseLastRunStart");
     mfdLastRunEnd = gContext->GetSetting("mythfilldatabaseLastRunEnd");
-    mfdLastRunStatus = gContext->GetNumSetting("mythfilldatabaseLastRunStatus");
+    mfdLastRunStatus = gContext->GetSetting("mythfilldatabaseLastRunStatus");
 
     os << "    Last mythfilldatabase run started on " << mfdLastRunStart
-       << " and ended on " << mfdLastRunEnd << ". ";
+       << " and ";
 
-    if(!mfdLastRunStatus)
-       os << "<br /><strong>WARNING</strong>: "
-          << "There was an error during the last run.<br />\r\n";
-    else
-       os << "Last run was successful.<br />\r\n";
+    if (mfdLastRunEnd < mfdLastRunStart) 
+        os << "is ";
+    else 
+        os << "ended on " << mfdLastRunEnd << ". ";
+
+    os << mfdLastRunStatus << "<br />\r\n";    
 
     if (!GuideDataThrough.isNull())
     {
@@ -2785,8 +2824,8 @@ void MainServer::PrintStatus(QSocket *socket)
             os << " <strong>WARNING</strong>: is mythfilldatabase running?";
     }
     else
-        os << "    There's <strong>no guide data</strong> availible! "
-           << "You need to run mythfilldatabase!";
+        os << "    There's <strong>no guide data</strong> available! "
+           << "Have you run mythfilldatabase?";
 
     os << "\r\n  </div>\r\n\r\n</body>\r\n</html>\r\n";
 
