@@ -9,10 +9,10 @@
 #include <fstream>
 #include <string>
 #include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <time.h>
-#include <math.h>
+#include <cstdlib>
+#include <cstdio>
+#include <ctime>
+#include <cmath>
 
 #include "libmyth/mythcontext.h"
 #include "libmythtv/commercial_skip.h"
@@ -21,8 +21,6 @@
 
 using namespace std;
 
-MythContext *gContext;
-
 int testmode = 0;
 int show_blanks = 0;
 int just_blanks = 0;
@@ -56,7 +54,7 @@ void FlagCommercials(QSqlDatabase *db, QString chanid, QString starttime)
             chanid.ascii(), starttime.ascii(), program_info->title.ascii());
         fflush( stdout );
     
-        if ((!force) && (program_info->CheckMarkupFlag(MARK_PROCESSING, db)))
+        if ((!force) && (program_info->IsCommProcessing(db)))
         {
             printf( "IN USE\n" );
             printf( "                        "
@@ -66,7 +64,7 @@ void FlagCommercials(QSqlDatabase *db, QString chanid, QString starttime)
     }
     else
     {
-        if ((!force) && (program_info->CheckMarkupFlag(MARK_PROCESSING, db)))
+        if ((!force) && (program_info->IsCommProcessing(db)))
             return;
     }
 
