@@ -1,6 +1,6 @@
 #include <qapplication.h>
 #include <qsqldatabase.h>
-#include <stdlib.h>
+#include <cstdlib>
 
 #include "libmyth/mythcontext.h"
 #include "libmyth/settings.h"
@@ -8,8 +8,6 @@
 #include "libmythtv/guidegrid.h"
 #include "libmythtv/tv.h"
 
-MythContext *gContext;
-
 int main(int argc, char **argv)
 {
     QApplication a(argc, argv);
@@ -35,7 +33,7 @@ int main(int argc, char **argv)
     gContext->LoadQtConfig();
 
     MythMainWindow *mainWindow = new MythMainWindow();
-    mainWindow->Show();
+    mainWindow->Init();
     gContext->SetMainWindow(mainWindow);
 
     TV::InitKeys();
