@@ -13,15 +13,18 @@
 #include <qsqlquery.h>
 #include <qurl.h>
 
+#include <unistd.h>
+#include <signal.h>
+
 #include <iostream>
 #include <fstream>
 #include <string>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <time.h>
+#include <cstdlib>
+#include <cstdio>
+#include <ctime>
 
 #include "libmyth/mythcontext.h"
+#include "libmythtv/scheduledrecording.h"
 
 using namespace std;
 
@@ -32,8 +35,11 @@ bool from_file = false;
 bool quiet = false;
 bool no_delete = false;
 bool isNorthAmerica = false;
-
-MythContext *gContext;
+bool interrupted = false;
+bool refresh_today = false;
+bool refresh_tomorrow = true;
+bool refresh_second = false;
+int listing_wrap_offset = 0;
 
 class ChanInfo
 {
@@ -651,11 +657,11 @@ void fixProgramList(QValueList<ProgInfo> *fixlist)
 
             cerr << "removing conflicting program: "
                  << (*todelete).channel << " "
-                 << (*todelete).title << " "
+                 << (*todelete).title.local8Bit() << " "
                  << (*todelete).startts << "-" << (*todelete).endts << endl;
             cerr << "conflicted with             : "
                  << (*tokeep).channel << " "
-                 << (*tokeep).title << " "
+                 << (*tokeep).title.local8Bit() << " "
                  << (*tokeep).startts << "-" <<   (*tokeep).endts << endl;
             cerr << endl;
 
@@ -672,13 +678,13 @@ QString getResponse(const QString &query, const QString &def)
 
     if (def != "")
     {
-        cout << " [" << def << "]  ";
+        cout << " [" << (const char *)def.local8Bit() << "]  ";
     }
     
     char response[80];
     cin.getline(response, 80);
 
-    QString qresponse = response;
+    QString qresponse = QString::fromLocal8Bit(response);
 
     if (qresponse == "")
         qresponse = def;
@@ -818,16 +824,16 @@ void handleChannels(int id, QValueList<ChanInfo> *chanlist)
                 cout << "### " << endl;
                 cout << "### Existing channel found" << endl;
                 cout << "### " << endl;
-                cout << "### xmltvid  = " << (*i).xmltvid.ascii() << endl;
-                cout << "### chanid   = " << chanid.ascii()       << endl;
-                cout << "### name     = " << name.ascii()         << endl;
-                cout << "### callsign = " << callsign.ascii()     << endl;
-                cout << "### channum  = " << chanstr.ascii()      << endl;
+                cout << "### xmltvid  = " << (*i).xmltvid.local8Bit() << endl;
+                cout << "### chanid   = " << chanid.local8Bit()       << endl;
+                cout << "### name     = " << name.local8Bit()         << endl;
+                cout << "### callsign = " << callsign.local8Bit()     << endl;
+                cout << "### channum  = " << chanstr.local8Bit()      << endl;
                 if (channel_preset)
-                    cout << "### freqid   = " << freqid.ascii()   << endl;
-                cout << "### finetune = " << finetune.ascii()     << endl;
-                cout << "### tvformat = " << tvformat.ascii()     << endl;
-                cout << "### icon     = " << icon.ascii()         << endl;
+                    cout << "### freqid   = " << freqid.local8Bit()   << endl;
+                cout << "### finetune = " << finetune.local8Bit()     << endl;
+                cout << "### tvformat = " << tvformat.local8Bit()     << endl;
+                cout << "### icon     = " << icon.local8Bit()         << endl;
                 cout << "### " << endl;
 
                 (*i).name = name;
@@ -847,23 +853,23 @@ void handleChannels(int id, QValueList<ChanInfo> *chanlist)
                     icon     != localfile ||
                     tvformat != (*i).tvformat)
                 {
-                    querystr.sprintf("UPDATE channel SET chanid = %s, "
-                                     "name = \"%s\", callsign = \"%s\", "
-                                     "channum = \"%s\", finetune = %d, "
-                                     "icon = \"%s\", freqid = \"%s\", "
-                                     "tvformat = \"%s\", "
-                                     " WHERE xmltvid = \"%s\" "
-                                     "AND sourceid = %d;",
-                                     chanid.ascii(),
-                                     (*i).name.ascii(),
-                                     (*i).callsign.ascii(),
-                                     (*i).chanstr.ascii(),
-                                     atoi((*i).finetune.ascii()),
-                                     localfile.ascii(),
-                                     (*i).freqid.ascii(),
-                                     (*i).tvformat.ascii(),
-                                     (*i).xmltvid.ascii(),
-                                     id);
+                     querystr = QString("UPDATE channel SET chanid = %0, "
+                                        "name = \"%1\", callsign = \"%2\", "
+                                        "channum = \"%3\", finetune = %4, "
+                                        "icon = \"%5\", freqid = \"%6\", "
+                                        "tvformat = \"%7\" "
+                                        " WHERE xmltvid = \"%8\" "
+                                        "AND sourceid = %9;")
+                                        .arg(chanid.utf8())
+                                        .arg((*i).name.utf8())
+                                        .arg((*i).callsign.utf8())
+                                        .arg((*i).chanstr.utf8())
+                                        .arg(atoi((*i).finetune.utf8()))
+                                        .arg(localfile.utf8())
+                                        .arg((*i).freqid.utf8())
+                                        .arg((*i).tvformat.utf8())
+                                        .arg((*i).xmltvid.utf8())
+                                        .arg(id);
 
                     if (!query.exec(querystr))
                     {
@@ -909,34 +915,35 @@ void handleChannels(int id, QValueList<ChanInfo> *chanlist)
                 cout << "### " << endl;
                 cout << "### New channel found" << endl;
                 cout << "### " << endl;
-                cout << "### name     = " << (*i).name.ascii()         << endl;
-                cout << "### callsign = " << (*i).callsign.ascii()     << endl;
-                cout << "### channum  = " << (*i).chanstr.ascii()      << endl;
+                cout << "### name     = " << (*i).name.local8Bit()     << endl;
+                cout << "### callsign = " << (*i).callsign.local8Bit() << endl;
+                cout << "### channum  = " << (*i).chanstr.local8Bit()  << endl;
                 if (channel_preset)
-                    cout << "### freqid   = " << (*i).freqid.ascii()   << endl;
-                cout << "### finetune = " << (*i).finetune.ascii()     << endl;
-                cout << "### tvformat = " << (*i).tvformat.ascii()     << endl;
-                cout << "### icon     = " << localfile.ascii()         << endl;
+                    cout << "### freqid   = " << (*i).freqid.local8Bit() << endl;
+                cout << "### finetune = " << (*i).finetune.local8Bit() << endl;
+                cout << "### tvformat = " << (*i).tvformat.local8Bit() << endl;
+                cout << "### icon     = " << localfile.local8Bit()     << endl;
                 cout << "### " << endl;
 
                 unsigned int chanid = promptForChannelUpdates(i,0);
 
                 if (chanid > 0)
                 {
-                    querystr.sprintf("INSERT INTO channel (chanid,name,"
-                                     "callsign,channum,finetune,icon,"
-                                     "xmltvid,sourceid,freqid,tvformat) "
-                                     "VALUES(%d,\"%s\",\"%s\",\"%s\",%d,"
-                                     "\"%s\",\"%s\",%d,\"%s\",\"%s\");", 
-                                     chanid,
-                                     (*i).name.ascii(),
-                                     (*i).callsign.ascii(),
-                                     (*i).chanstr.ascii(),
-                                     atoi((*i).finetune.ascii()),
-                                     localfile.ascii(),
-                                     (*i).xmltvid.ascii(),
-                                     id, (*i).freqid.ascii(),
-                                     (*i).tvformat.ascii());
+                    querystr = QString("INSERT INTO channel (chanid,name"
+                                       ",callsign,channum,finetune,icon"
+                                       ",xmltvid,sourceid,freqid,tvformat) "
+                                       "VALUES(%0,\"%1\",\"%2\",\"%3\",%4,"
+                                       "\"%5\",\"%6\",%7,\"%8\",\"%9\");")
+                                       .arg(chanid)
+                                       .arg((*i).name.utf8())
+                                       .arg((*i).callsign.utf8())
+                                       .arg((*i).chanstr.utf8())
+                                       .arg(atoi((*i).finetune))
+                                       .arg(localfile.utf8())
+                                       .arg((*i).xmltvid.utf8())
+                                       .arg(id)
+                                       .arg((*i).freqid.utf8())
+                                       .arg((*i).tvformat.utf8());
 
                     if (!query.exec(querystr))
                     {
@@ -980,20 +987,21 @@ void handleChannels(int id, QValueList<ChanInfo> *chanlist)
                         break;
                 }
 
-                querystr.sprintf("INSERT INTO channel (chanid,name,callsign,"
-                                 "channum,finetune,icon,xmltvid,sourceid,"
-                                 "freqid,tvformat) "
-                                 "VALUES(%d,\"%s\",\"%s\",\"%s\",%d,\"%s\","
-                                 "\"%s\",%d,\"%s\",\"%s\");", 
-                                 chanid,
-                                 (*i).name.ascii(),
-                                 (*i).callsign.ascii(),
-                                 (*i).chanstr.ascii(),
-                                 atoi((*i).finetune.ascii()),
-                                 localfile.ascii(),
-                                 (*i).xmltvid.ascii(),
-                                 id, (*i).freqid.ascii(),
-                                 (*i).tvformat.ascii());
+                querystr = QString("INSERT INTO channel (chanid,name,callsign,"
+                                   "channum,finetune,icon,xmltvid,sourceid,"
+                                   "freqid,tvformat) "
+                                   "VALUES(%0,\"%1\",\"%2\",\"%3\",%4,\"%5\","
+                                   "\"%6\",%7,\"%8\",\"%9\");")
+                                   .arg(chanid)
+                                   .arg((*i).name.utf8())
+                                   .arg((*i).callsign)
+                                   .arg((*i).chanstr)
+                                   .arg(atoi((*i).finetune))
+                                   .arg(localfile)
+                                   .arg((*i).xmltvid)
+                                   .arg(id)
+                                   .arg((*i).freqid)
+                                   .arg((*i).tvformat);
 
                 if (!query.exec(querystr))
                     MythContext::DBError("channel insert", query);
@@ -1015,7 +1023,7 @@ void clearDBAtOffset(int offset, int chanid, QDate *qCurrentDate)
         qCurrentDate = &newDate;
     }
 
-    int nextoffset = offset + 1;
+    int nextoffset = 1;
 
     if (offset == -1)
     {
@@ -1023,33 +1031,34 @@ void clearDBAtOffset(int offset, int chanid, QDate *qCurrentDate)
         nextoffset = 10;
     }
 
+    QDateTime from, to;
+    from.setDate(*qCurrentDate);
+    from = from.addDays(offset);
+    from = from.addSecs(listing_wrap_offset);
+    to = from.addDays(nextoffset);
+
     QSqlQuery query;
     QString querystr;
 
-    querystr.sprintf("DELETE FROM program WHERE starttime >= "
-                     "DATE_ADD('%s', INTERVAL %d DAY) "
-                     "AND starttime < DATE_ADD('%s', INTERVAL "
-                     "%d DAY) AND chanid = %d;", 
-                     qCurrentDate->toString("yyyyMMdd").ascii(), offset, 
-                     qCurrentDate->toString("yyyyMMdd").ascii(), nextoffset, 
+    querystr.sprintf("DELETE FROM program "
+                    "WHERE starttime >= '%s' AND starttime < '%s' "
+                    "AND chanid = %d;",
+                     from.toString("yyyyMMddhhmmss").ascii(), 
+                     to.toString("yyyyMMddhhmmss").ascii(),
                      chanid);
     query.exec(querystr);
 
-    querystr.sprintf("DELETE FROM programrating WHERE starttime >= "
-                     "DATE_ADD('%s', INTERVAL %d DAY) "
-                     "AND starttime < DATE_ADD('%s', INTERVAL "
-                     "%d DAY) AND chanid = %d;", 
-                     qCurrentDate->toString("yyyyMMdd").ascii(), offset, 
-                     qCurrentDate->toString("yyyyMMdd").ascii(), nextoffset, 
+    querystr.sprintf("DELETE FROM programrating WHERE starttime >= '%s' "
+                     " AND starttime < '%s' AND chanid = %d;", 
+                     from.toString("yyyyMMddhhmmss").ascii(), 
+                     to.toString("yyyyMMddhhmmss").ascii(),
                      chanid);
     query.exec(querystr);
 
-    querystr.sprintf("DELETE FROM credits WHERE starttime >= "
-                     "DATE_ADD('%s', INTERVAL %d DAY) "
-                     "AND starttime < DATE_ADD('%s', INTERVAL "
-                     "%d DAY) AND chanid = %d;", 
-                     qCurrentDate->toString("yyyyMMdd").ascii(), offset, 
-                     qCurrentDate->toString("yyyyMMdd").ascii(), nextoffset, 
+    querystr.sprintf("DELETE FROM credits WHERE starttime >= '%s' "
+                     "AND starttime < '%s' AND chanid = %d;", 
+                     from.toString("yyyyMMddhhmmss").ascii(), 
+                     to.toString("yyyyMMddhhmmss").ascii(), 
                      chanid);
     query.exec(querystr);
 }
@@ -1144,7 +1153,7 @@ void handlePrograms(int id, int offset, QMap<QString,
 
                     cerr << "inserting new program    : "
                          << (*i).channel << " "
-                         << (*i).title << " "
+                         << (*i).title.local8Bit() << " "
                          << (*i).startts << "-" << (*i).endts << endl;
                     cerr << endl;
 
@@ -1345,6 +1354,10 @@ bool grabData(Source source, int offset, QDate *qCurrentDate = 0)
         command.sprintf("nice %s --days 7 --output %s",
                         xmltv_grabber.ascii(),
                         filename.ascii());
+    else if (xmltv_grabber == "tv_grab_fr")
+        command.sprintf("nice %s --days 7 '%s' --output %s",
+                        xmltv_grabber.ascii(), configfile.ascii(),
+                        filename.ascii());
     else if (xmltv_grabber == "tv_grab_nl")
         command.sprintf("nice %s --output %s",
                         xmltv_grabber.ascii(),
@@ -1359,10 +1372,14 @@ bool grabData(Source source, int offset, QDate *qCurrentDate = 0)
         command.sprintf("nice %s --days=4 --offset %d --config-file '%s' --output %s",
                         xmltv_grabber.ascii(), offset,
                         configfile.ascii(), filename.ascii());
+     else if (xmltv_grabber == "tv_grab_ja")
+         // Use fixed interval of 3 days for Japanese grabber
+         command.sprintf("nice %s --days 3 --config-file '%s' --output %s",
+                         xmltv_grabber.ascii(), configfile.ascii(),
+                         filename.ascii());
     else if (xmltv_grabber == "tv_grab_sn")
-        // Use fixed interval of 14 days for Swedish/Norwegian grabber
-        command.sprintf("nice %s --days 14 --config-file '%s' --output %s",
-                        xmltv_grabber.ascii(), configfile.ascii(),
+        command.sprintf("nice %s --days 1 --offset %d --config-file '%s' --output %s",
+                        xmltv_grabber.ascii(), offset, configfile.ascii(),
                         filename.ascii());
     else if (xmltv_grabber == "tv_grab_dk")
         // Use fixed interval of 7 days for Danish grabber
@@ -1388,7 +1405,9 @@ bool grabData(Source source, int offset, QDate *qCurrentDate = 0)
          xmltv_grabber == "tv_grab_uk" ||
          xmltv_grabber == "tv_grab_uk_rt" ||
          xmltv_grabber == "tv_grab_nl" ||
-         xmltv_grabber == "tv_grab_fi"))
+         xmltv_grabber == "tv_grab_fr" ||
+         xmltv_grabber == "tv_grab_fi" ||
+         xmltv_grabber == "tv_grab_ja"))
          command += " --quiet";
 
 
@@ -1397,20 +1416,39 @@ bool grabData(Source source, int offset, QDate *qCurrentDate = 0)
 
     QDateTime qdtNow = QDateTime::currentDateTime();
     QSqlQuery query;
+    QString status = "currently running.";
+
     query.exec(QString("UPDATE settings SET data ='%1' "
                        "WHERE value='mythfilldatabaseLastRunStart'")
                        .arg(qdtNow.toString("yyyy-MM-dd hh:mm")));
 
-    int status = system(command.ascii());
-
     query.exec(QString("UPDATE settings SET data ='%1' "
                        "WHERE value='mythfilldatabaseLastRunStatus'")
                        .arg(status));
 
+    int systemcall_status = system(command.ascii());
+    bool succeeded = WIFEXITED(systemcall_status) &&
+         WEXITSTATUS(systemcall_status) == 0;
+
     qdtNow = QDateTime::currentDateTime();
     query.exec(QString("UPDATE settings SET data ='%1' "
                        "WHERE value='mythfilldatabaseLastRunEnd'")
                        .arg(qdtNow.toString("yyyy-MM-dd hh:mm")));
+
+    status = "Successful.";
+
+    if (!succeeded)
+    {
+        status = QString("FAILED:  xmltv returned error code %1.")
+                         .arg(systemcall_status);
+
+        query.exec(QString("UPDATE settings SET data ='%1' "
+                           "WHERE value='mythfilldatabaseLastRunStatus'")
+                           .arg(status));
+        if (WIFSIGNALED(systemcall_status) &&
+            (WTERMSIG(systemcall_status) == SIGINT || WTERMSIG(systemcall_status) == SIGQUIT))
+            interrupted = true;
+    }
  
     if (!quiet)
          cout << "------------------ End of XMLTV output ------------------" << endl;
@@ -1420,7 +1458,7 @@ bool grabData(Source source, int offset, QDate *qCurrentDate = 0)
     QFile thefile(filename);
     thefile.remove();
 
-    return (status == 0);
+    return succeeded;
 }
 
 void clearOldDBEntries(void)
@@ -1432,6 +1470,15 @@ void clearOldDBEntries(void)
     if (no_delete)
         offset=7;
 
+    querystr.sprintf("DELETE FROM oldprogram WHERE airdate < "
+                     "DATE_SUB(CURRENT_DATE, INTERVAL 320 DAY);");
+    query.exec(querystr);
+
+    querystr.sprintf("REPLACE INTO oldprogram (oldtitle,airdate) "
+                     "SELECT title,starttime FROM program "
+                     "WHERE starttime < NOW() group by title;");
+    query.exec(querystr);
+
     querystr.sprintf("DELETE FROM program WHERE starttime <= "
                      "DATE_SUB(CURRENT_DATE, INTERVAL %d DAY);", offset);
     query.exec(querystr);
@@ -1443,34 +1490,45 @@ void clearOldDBEntries(void)
     querystr.sprintf("DELETE FROM credits WHERE starttime <= "
                      "DATE_SUB(CURRENT_DATE, INTERVAL %d DAY);", offset);
     query.exec(querystr);
+
+    querystr.sprintf("DELETE FROM record WHERE type = 1 AND enddate < "
+                     "DATE_SUB(CURRENT_DATE, INTERVAL 1 DAY);");
+    query.exec(querystr);
+
+    querystr.sprintf("DELETE FROM recordoverride WHERE endtime < "
+                     "DATE_SUB(CURRENT_DATE, INTERVAL 1 DAY);");
+    query.exec(querystr);
 }
 
 bool fillData(QValueList<Source> &sourcelist)
 {
     QValueList<Source>::Iterator it;
 
+    QString status;
+    QSqlQuery query;
+    QDateTime GuideDataBefore, GuideDataAfter;
+
+    query.exec(QString("SELECT MAX(endtime) FROM program;"));
+    if (query.isActive() && query.numRowsAffected())
+    {
+        query.next();
+        GuideDataBefore = QDateTime::fromString(query.value(0).toString(),
+                                                Qt::ISODate);
+    }
+
     int failures = 0;
     for (it = sourcelist.begin(); it != sourcelist.end(); ++it) {
         QString xmltv_grabber = (*it).xmltvgrabber;
         if (xmltv_grabber == "tv_grab_uk" || xmltv_grabber == "tv_grab_de" ||
             xmltv_grabber == "tv_grab_fi" || xmltv_grabber == "tv_grab_es" ||
-            xmltv_grabber == "tv_grab_nl" || xmltv_grabber == "tv_grab_au")
+            xmltv_grabber == "tv_grab_nl" || xmltv_grabber == "tv_grab_au" ||
+            xmltv_grabber == "tv_grab_fr" || xmltv_grabber == "tv_grab_ja")
         {
             // tv_grab_uk|de doesn't support the --offset option, so just grab a 
             // week.
             if (!grabData(*it, -1))
                 ++failures;
         }
-        else if (xmltv_grabber == "tv_grab_sn")
-        {
-            // tv_grab_sn has problems grabbing one day at a time as the site
-            // (dagenstv.com) "wraps" the day-listings at 06:00am. This means
-            // that programs starting at 02:00am on jul 19 gets "grabbed" in
-            // the xmltv-file for jul 18, causing all sorts of trouble.
-            // The simple solution is to grab the full two weeks every time.
-            if (!grabData(*it, 0))
-                ++failures;
-        }
         else if (xmltv_grabber == "tv_grab_dk")
         {
             if (!grabData(*it, 0))
@@ -1512,15 +1570,41 @@ bool fillData(QValueList<Source> &sourcelist)
             }
         }
         else if (xmltv_grabber == "tv_grab_na" || 
-                 xmltv_grabber == "tv_grab_uk_rt")
+                 xmltv_grabber == "tv_grab_uk_rt" ||
+                 xmltv_grabber == "tv_grab_sn")
         {
+            if (xmltv_grabber == "tv_grab_sn")
+                listing_wrap_offset = 6 * 3600;
+
             QDate qCurrentDate = QDate::currentDate();
 
-            if (!grabData(*it, 1, &qCurrentDate))
-                ++failures;
+            if (refresh_today)
+            {
+                if (!quiet)
+                    cout << "Refreshing Today's data" << endl;
+                if (!grabData(*it, 0, &qCurrentDate))
+                    ++failures;
+            }
+
+            if (refresh_tomorrow)
+            {
+                if (!quiet)
+                    cout << "Refreshing Tomorrow's data" << endl;
+                if (!grabData(*it, 1, &qCurrentDate))
+                    ++failures;
+            }
+
+            if (refresh_second)
+            {
+                if (!quiet)
+                    cout << "Refreshing data for 2 days from today" << endl;
+                if (!grabData(*it, 2, &qCurrentDate))
+                    ++failures;
+            }
 
             int maxday = 9;
-            if (xmltv_grabber == "tv_grab_uk_rt")
+            if (xmltv_grabber == "tv_grab_uk_rt" ||
+                xmltv_grabber == "tv_grab_sn")
                 maxday = 14;
 
             for (int i = 0; i < maxday; i++)
@@ -1601,7 +1685,13 @@ bool fillData(QValueList<Source> &sourcelist)
                     if (!quiet)
                         cout << "Fetching data for " << date << endl;
                     if (!grabData(*it, i, &qCurrentDate))
+                    {
                         ++failures;
+                        if (interrupted)
+                        {
+                            break;
+                        }
+                    }
                 }
                 else
                 {
@@ -1616,6 +1706,32 @@ bool fillData(QValueList<Source> &sourcelist)
             cerr << "Grabbing XMLTV data using " << xmltv_grabber.ascii() 
                  << " is not verified as working.\n";
         }
+        if (interrupted)
+        {
+            break;
+        }
+    }
+
+    query.exec(QString("SELECT MAX(endtime) FROM program;"));
+    if (query.isActive() && query.numRowsAffected())
+    {
+        query.next();
+        GuideDataAfter = QDateTime::fromString(query.value(0).toString(),
+                                               Qt::ISODate);
+    }
+
+    if (failures == 0)
+    {
+        if (GuideDataAfter == GuideDataBefore)
+            status = "mythfilldatabase ran, but did not insert "
+                     "any new data into the Guide.  This can indicate a "
+                     "potential xmltv failure."; 
+        else
+            status = "Successful.";
+
+        query.exec(QString("UPDATE settings SET data ='%1' "
+                           "WHERE value='mythfilldatabaseLastRunStatus'")
+                           .arg(status));
     }
 
     clearOldDBEntries();
@@ -1838,6 +1954,18 @@ int main(int argc, char *argv[])
                  cout << "### reading channels from xawtv configfile\n";
             from_xawfile = true;
         }
+        else if (!strcmp(a.argv()[argpos], "--refresh-today"))
+        {
+            refresh_today = true;
+        }
+        else if (!strcmp(a.argv()[argpos], "--dont-refresh-tomorrow"))
+        {
+            refresh_tomorrow = false;
+        }
+        else if (!strcmp(a.argv()[argpos], "--refresh-second"))
+        {
+            refresh_second = true;
+        }
         else if (!strcmp(a.argv()[argpos], "--quiet"))
         {
              quiet = true;
@@ -1878,6 +2006,12 @@ int main(int argc, char *argv[])
             cout << "   <sourceid>    = cardinput\n";
             cout << "   <xawtvrcfile> = file to read\n";
             cout << "\n";
+            cout << "--refresh-today\n";
+            cout << "--refresh-second\n";
+            cout << "   (Only valid for grabbers: na/uk_rt/sn)\n";
+            cout << "   Force a refresh today or two days from now, to catch the latest changes\n";
+            cout << "--dont-refresh-tomorrow\n";
+            cout << "   Tomorrow will be refreshed always unless this argument is used\n";
             cout << "--help\n";
             cout << "   This text\n";
             cout << "\n";
@@ -1968,6 +2102,8 @@ int main(int argc, char *argv[])
     else if (!quiet)
          cout << update_count << " replacements made.\n";
 
+    ScheduledRecording::signalChange(db);
+
     delete gContext;
 
     return 0;
