@@ -45,12 +45,12 @@ ChannelRecPriority::ChannelRecPriority(QSqlDatabase *ldb,
     theme->SetHMult(hmult);
     if (!theme->LoadTheme(xmldata, "recprioritychannels"))
     {
-        DialogBox diag(gContext->GetMainWindow(), "The theme you are using "
+        DialogBox diag(gContext->GetMainWindow(), tr("The theme you are using "
                        "does not contain a 'recprioritychannels' element.  "
                        "Please contact the theme creator and ask if they could "
                        "please update it.<br><br>The next screen will be empty."
-                       "  Escape out of it to return to the menu.");
-        diag.AddButton("OK");
+                       "  Escape out of it to return to the menu."));
+        diag.AddButton(tr("OK"));
         diag.exec();
 
         return;
@@ -149,6 +149,15 @@ void ChannelRecPriority::keyPressEvent(QKeyEvent *e)
                     update(fullRect);
                 }
             }
+            else if (action == "PREVVIEW" || action == "NEXTVIEW")
+            {
+                if (sortType == byChannel)
+                    sortType = byRecPriority;
+                else
+                    sortType = byChannel; 
+                SortList();     
+                update(fullRect);
+            }
             else
                 handled = false;
         }
@@ -273,6 +282,9 @@ void ChannelRecPriority::cursorDown(bool page)
         inData = dataCount - listsize;
     }
 
+    if (inData < 0) 
+        inData = 0;
+
     if (inList >= listCount)
         inList = listCount - 1;
 
