@@ -268,6 +268,19 @@ public:
     };
 };
 
+class TryUnflaggedSkip: public CheckBoxSetting, public GlobalSetting {
+public:
+    TryUnflaggedSkip():
+        GlobalSetting("TryUnflaggedSkip") {
+        setLabel(QObject::tr("Skip Unflagged Commercials"));
+        setValue(false);
+        setHelpText(QObject::tr("Try to skip commercial breaks even if they "
+                    "have not been flagged.  This does not always work well "
+                    "and can disrupt playback if commercial breaks aren't "
+                    "detected properly."));
+    };
+};
+
 class AutoCommercialFlag: public CheckBoxSetting, public BackendSetting {
 public:
     AutoCommercialFlag():
@@ -374,6 +387,8 @@ public:
         BackendSetting("RecordPreRoll") {
         setLabel(QObject::tr("Time to record before start of show "
                  "(in seconds)"));
+        setHelpText(QObject::tr("This global setting is ignored in case of "
+                    "conflicts with other scheduled programs."));
         setValue(0);
     };
 };
@@ -385,6 +400,8 @@ public:
         BackendSetting("RecordOverTime") {
         setLabel(QObject::tr("Time to record past end of show (in seconds)"));
         setValue(0);
+        setHelpText(QObject::tr("This global setting is ignored in case of "
+                    "conflicts with other scheduled programs."));
     };
 };
 
@@ -586,6 +603,17 @@ public:
     };
 };
 
+class UseVideoTimebase: public CheckBoxSetting, public GlobalSetting {
+public:
+    UseVideoTimebase():
+        GlobalSetting("UseVideoTimebase") {
+        setLabel(QObject::tr("Use video as timebase"));
+        setValue(false);
+        setHelpText(QObject::tr("Use the video as the timebase and warp "
+                    "the audio to keep it in sync. (Experimental)"));
+    };
+};
+
 class ExperimentalSync: public CheckBoxSetting, public GlobalSetting {
 public:
     ExperimentalSync():
@@ -605,7 +633,8 @@ public:
         setValue(false);
         setHelpText(QObject::tr("If this is set, captions will be on by "
                     "default when playing back recordings or watching "
-                    "live TV."));
+                    "live TV.  Closed Captioning can be turned on or off "
+                    "by pressing 'T' during playback."));
     };
 };
 
@@ -674,6 +703,17 @@ public:
     };
 };
 
+class CCWarnSetting: public CheckBoxSetting, public GlobalSetting {
+public:
+    CCWarnSetting():
+       GlobalSetting("CCBufferWarnings") {
+       setLabel(QObject::tr("Enable channel change buffer warnings"));
+       setValue(false);
+       setHelpText(QObject::tr("If set, MythTV will warn you whenever you "
+                   "change the channel but are not caught up to live TV."));
+    };
+};
+
 class UDPNotifyPort: public LineEditSetting, public GlobalSetting {
 public:
     UDPNotifyPort():
@@ -794,6 +834,17 @@ public:
     };
 };
 
+class NoPromptOnExit: public CheckBoxSetting, public GlobalSetting {
+public:
+    NoPromptOnExit():
+        GlobalSetting("NoPromptOnExit") {
+        setLabel(QObject::tr("No Prompt on Exit"));
+        setValue(false);
+        setHelpText(QObject::tr("If set, you will not be prompted when pressing"
+                    " the exit key.  Instead, MythTV will immediately exit."));
+    };
+};
+
 class HaltCommand: public LineEditSetting, public GlobalSetting {
 public:
     HaltCommand():
@@ -893,6 +944,17 @@ public:
     };
 };
 
+class GuiSizeForTV: public CheckBoxSetting, public GlobalSetting {
+public:
+    GuiSizeForTV() :
+        GlobalSetting("GuiSizeForTV") {
+        setLabel(QObject::tr("Use GUI size for TV playback"));
+        setValue(true);
+        setHelpText(QObject::tr("If checked, use the above size for TV. "
+                                "If unchecked, use full screen."));
+    };
+};
+
 class RunInWindow: public CheckBoxSetting, public GlobalSetting {
 public:
     RunInWindow():
@@ -917,15 +979,19 @@ public:
 class MythDateFormat: public ComboBoxSetting, public GlobalSetting {
 public:
     MythDateFormat():
-        ComboBoxSetting(true), GlobalSetting("DateFormat") {
+        GlobalSetting("DateFormat") {
         setLabel(QObject::tr("Date format"));
-        addSelection(QObject::tr("ddd MMM d"), "ddd MMM d");
-        addSelection(QObject::tr("ddd MMMM d"), "ddd MMMM d");
-        addSelection(QObject::tr("MMM d"), "MMM d");
-        addSelection(QObject::tr("MM/dd"), "MM/dd");
-        addSelection(QObject::tr("MM.dd"), "MM.dd");
-        addSelection(QObject::tr("ddd d MMM"), "ddd d MMM");
-        addSelection(QObject::tr("dd.MM.yyyy"), "dd.MM.yyyy");
+
+        QDate sampdate(2004, 1, 31);
+
+        addSelection(sampdate.toString("ddd MMM d"), "ddd MMM d");
+        addSelection(sampdate.toString("ddd MMMM d"), "ddd MMMM d");
+        addSelection(sampdate.toString("MMM d"), "MMM d");
+        addSelection(sampdate.toString("MM/dd"), "MM/dd");
+        addSelection(sampdate.toString("MM.dd"), "MM.dd");
+        addSelection(sampdate.toString("ddd d MMM"), "ddd d MMM");
+        addSelection(sampdate.toString("dd.MM.yyyy"), "dd.MM.yyyy");
+        addSelection(sampdate.toString("yyyy-MM-dd"), "yyyy-MM-dd");
         setHelpText(QObject::tr("Your preferred date format."));
     };
 };
@@ -933,15 +999,19 @@ public:
 class MythShortDateFormat: public ComboBoxSetting, public GlobalSetting {
 public:
     MythShortDateFormat():
-        ComboBoxSetting(true), GlobalSetting("ShortDateFormat") {
+        GlobalSetting("ShortDateFormat") {
         setLabel(QObject::tr("Short Date format"));
-        addSelection(QObject::tr("M/d"), "M/d");
-        addSelection(QObject::tr("d/M"), "d/M");
-        addSelection(QObject::tr("MM/dd"), "MM/dd");
-        addSelection(QObject::tr("dd/MM"), "dd/MM");
-        addSelection(QObject::tr("MM.dd"), "MM.dd");
-        addSelection(QObject::tr("d.M."), "d.M.");
-        addSelection(QObject::tr("dd.MM."), "dd.MM.");
+
+        QDate sampdate(2004, 1, 31);
+   
+        addSelection(sampdate.toString("M/d"), "M/d");
+        addSelection(sampdate.toString("d/M"), "d/M");
+        addSelection(sampdate.toString("MM/dd"), "MM/dd");
+        addSelection(sampdate.toString("dd/MM"), "dd/MM");
+        addSelection(sampdate.toString("MM.dd"), "MM.dd");
+        addSelection(sampdate.toString("d.M."), "d.M.");
+        addSelection(sampdate.toString("dd.MM."), "dd.MM.");
+        addSelection(sampdate.toString("MM-dd"), "MM-dd");
         setHelpText(QObject::tr("Your preferred short date format."));
     };
 };
@@ -949,14 +1019,17 @@ public:
 class MythTimeFormat: public ComboBoxSetting, public GlobalSetting {
 public:
     MythTimeFormat():
-        ComboBoxSetting(true), GlobalSetting("TimeFormat") {
+        GlobalSetting("TimeFormat") {
         setLabel(QObject::tr("Time format"));
-        addSelection(QObject::tr("h:mm AP"), "h:mm AP");
-        addSelection(QObject::tr("h:mm ap"), "h:mm ap");
-        addSelection(QObject::tr("hh:mm AP"), "hh:mm AP");
-        addSelection(QObject::tr("hh:mm ap"), "hh:mm ap");
-        addSelection(QObject::tr("h:mm"), "h:mm");
-        addSelection(QObject::tr("hh:mm"), "hh:mm");
+
+        QTime samptime(6, 56, 0);
+
+        addSelection(samptime.toString("h:mm AP"), "h:mm AP");
+        addSelection(samptime.toString("h:mm ap"), "h:mm ap");
+        addSelection(samptime.toString("hh:mm AP"), "hh:mm AP");
+        addSelection(samptime.toString("hh:mm ap"), "hh:mm ap");
+        addSelection(samptime.toString("h:mm"), "h:mm");
+        addSelection(samptime.toString("hh:mm"), "hh:mm");
         setHelpText(QObject::tr("Your preferred time format.  Choose a format "
                     "with \"AP\" in it for an AM/PM display, otherwise "
                     "your time display will be 24-hour or \"military\" "
@@ -1080,12 +1153,36 @@ public:
     EPGFillType():
         GlobalSetting("EPGFillType") {
         setLabel(QObject::tr("Guide Shading Method"));
-        addSelection(QObject::tr("Colorized (alpha)"), "6");
-        addSelection(QObject::tr("Colorized (shaded)"), "5");
-        addSelection(QObject::tr("Embossed (shaded)"), "3");
-        addSelection(QObject::tr("Embossed (solid)"), "4");
-        addSelection(QObject::tr("Rounded (shaded)"), "1");
-        addSelection(QObject::tr("Rounded (solid)"), "2");
+        QString val = "";
+        addSelection(QObject::tr("Alpha - Transparent (CPU Usage - High)"), 
+                     val.left(0).setNum((int)UIGuideType::Alpha));
+        addSelection(QObject::tr("Blender - Transparent (CPU Usage - Middle)"), 
+                     val.left(0).setNum((int)UIGuideType::Dense));
+        addSelection(QObject::tr("Eco - Transparent (CPU Usage - Low)"), 
+                     val.left(0).setNum((int)UIGuideType::Eco));
+        addSelection(QObject::tr("Solid (CPU Usage - Middle)"), 
+                     val.left(0).setNum((int)UIGuideType::Solid));
+    };
+};
+
+class EPGShowCategoryColors: public CheckBoxSetting, public GlobalSetting {
+public:
+    EPGShowCategoryColors():
+        GlobalSetting("EPGShowCategoryColors") {
+        setLabel(QObject::tr("Display Genre Colors"));
+        setHelpText(QObject::tr("Colorize program guide using "
+                    " genre colors. (Not available for all grabbers.)"));
+        setValue(true);
+    };
+};
+
+class EPGShowCategoryText: public CheckBoxSetting, public GlobalSetting {
+public:
+    EPGShowCategoryText():
+        GlobalSetting("EPGShowCategoryText") {
+        setLabel(QObject::tr("Display Genre Text"));
+        setHelpText(QObject::tr("(Not available for all grabbers.)"));
+        setValue(true);
     };
 };
 
@@ -1256,7 +1353,7 @@ public:
 class MythLanguage: public ComboBoxSetting, public GlobalSetting {
 public:
     MythLanguage():
-        ComboBoxSetting(true), GlobalSetting("Language") {
+        GlobalSetting("Language") {
         setLabel(QObject::tr("Language"));
         addSelection(QString::fromUtf8("English"), "EN");     // English
         addSelection(QString::fromUtf8("Italiano"), "IT");    // Italian
@@ -1268,6 +1365,8 @@ public:
         addSelection(QString::fromUtf8("Dansk"), "DK");       // Danish
         addSelection(QString::fromUtf8("Svenska"), "SV");     // Swedish
         addSelection(QString::fromUtf8("Português"), "PT");   // Portuguese
+        //addSelection(QString::fromUtf8("日本語"), "JA");    // Japanese
+        addSelection(QString::fromUtf8("Nihongo"), "JA");     // Japanese
         setHelpText(QObject::tr("Your preferred language.") );
     };
 };
@@ -1338,6 +1437,18 @@ public:
     };
 };
 
+class EnableMediaMon: public CheckBoxSetting, public GlobalSetting {
+public:
+    EnableMediaMon():
+        GlobalSetting("MonitorDrives") {
+        setLabel(QObject::tr("Monitor CD/DVD"));
+        setHelpText(QObject::tr("This enables support for monitoring "
+                    "your CD/DVD drives for new disks and launching"
+                    "the proper plugin to handle them."));
+        setValue(false);
+    };
+};
+
 class PVR350OutputEnable: public CheckBoxSetting, public GlobalSetting {
 public:
     PVR350OutputEnable():
@@ -1360,13 +1471,46 @@ public:
     };
 };
 
-class PVR350Settings: public  VerticalConfigurationGroup,
-                      public TriggeredConfigurationGroup {
+class PVR350EPGAlphaValue: public SpinBoxSetting, public GlobalSetting {
 public:
-     PVR350Settings():
+    PVR350EPGAlphaValue():
+        SpinBoxSetting(0, 255, 1),
+        GlobalSetting("PVR350EPGAlphaValue") {
+        setLabel(QObject::tr("Program Guide Alpha"));
+        setValue(164);
+        setHelpText(QObject::tr("How much to blend the program guide over the "
+                    "live TV image.  Higher numbers mean more guide and less "
+                    "TV."));
+    };
+};
+
+#ifdef USING_XVMC
+class UseXVMC: public CheckBoxSetting, public GlobalSetting {
+public:
+    UseXVMC(): GlobalSetting("UseXVMC") {
+        setLabel(QObject::tr("Use HW XVMC MPEG Decoding"));
+        setValue(true);
+    };
+};
+#endif
+
+#ifdef USING_VIASLICE
+class UseViaSlice: public CheckBoxSetting, public GlobalSetting {
+public:
+    UseViaSlice(): GlobalSetting("UseViaSlice") {
+        setLabel(QObject::tr("Use VIA HW MPEG Decoding"));
+        setValue(true);
+    };
+};
+#endif
+
+class HwDecSettings: public  VerticalConfigurationGroup,
+                     public TriggeredConfigurationGroup {
+public:
+     HwDecSettings():
          VerticalConfigurationGroup(false),
          TriggeredConfigurationGroup(false) {
-         setLabel(QObject::tr("PVR-350 Settings"));
+         setLabel(QObject::tr("Hardware Decoding Settings"));
          setUseLabel(false);
 
          Setting* pvr350output = new PVR350OutputEnable();
@@ -1375,9 +1519,17 @@ public:
 
          ConfigurationGroup* settings = new VerticalConfigurationGroup(false);
          settings->addChild(new PVR350VideoDev());
+         settings->addChild(new PVR350EPGAlphaValue());
          addTarget("1", settings);
 
          addTarget("0", new VerticalConfigurationGroup(true));
+
+#ifdef USING_XVMC
+         addChild(new UseXVMC());
+#endif
+#ifdef USING_VIASLICE
+         addChild(new UseViaSlice());
+#endif
     };
 };
 
@@ -1388,9 +1540,11 @@ MainGeneralSettings::MainGeneralSettings()
 
     VerticalConfigurationGroup* general = new VerticalConfigurationGroup(false);
     general->addChild(new AllowQuitShutdown());
+    general->addChild(new NoPromptOnExit());
     general->addChild(new HaltCommand());
     general->addChild(new SetupPinCodeRequired());
     general->addChild(new SetupPinCode());
+    general->addChild(new EnableMediaMon());
     general->addChild(new EnableXbox());
     addChild(general);
 }
@@ -1403,6 +1557,7 @@ PlaybackSettings::PlaybackSettings()
     general->addChild(new CustomFilters());
     general->addChild(new ReduceJitter());
     general->addChild(new ExperimentalSync());
+    general->addChild(new UseVideoTimebase());
     general->addChild(new DecodeExtraAudio());
     general->addChild(new PIPLocation());
     addChild(general);
@@ -1414,10 +1569,11 @@ PlaybackSettings::PlaybackSettings()
     gen2->addChild(new ClearSavedPosition());
     gen2->addChild(new AltClearSavedPosition());
     gen2->addChild(new UsePicControls());
+    gen2->addChild(new CCWarnSetting());
     gen2->addChild(new UDPNotifyPort());
     addChild(gen2);
 
-    addChild(new PVR350Settings());
+    addChild(new HwDecSettings());
 
     VerticalConfigurationGroup* seek = new VerticalConfigurationGroup(false);
     seek->setLabel(QObject::tr("Seeking"));
@@ -1438,6 +1594,7 @@ PlaybackSettings::PlaybackSettings()
     comms->addChild(new AggressiveCommDetect());
     comms->addChild(new CommSkipAllBlanks());
     comms->addChild(new AutoCommercialSkip());
+    comms->addChild(new TryUnflaggedSkip());
     addChild(comms);
 
     VerticalConfigurationGroup* oscan = new VerticalConfigurationGroup(false);
@@ -1491,6 +1648,8 @@ EPGSettings::EPGSettings()
     epg->setLabel(QObject::tr("Program Guide"));
 
     epg->addChild(new EPGFillType());
+    epg->addChild(new EPGShowCategoryColors());
+    epg->addChild(new EPGShowCategoryText());
     epg->addChild(new EPGScrollType());
     epg->addChild(new EPGShowChannelIcon());
     epg->addChild(new EPGChanDisplay());
@@ -1537,6 +1696,7 @@ AppearanceSettings::AppearanceSettings()
     screen->addChild(new GuiHeight());
     screen->addChild(new GuiOffsetX());
     screen->addChild(new GuiOffsetY());
+    screen->addChild(new GuiSizeForTV());
     screen->addChild(new RunInWindow());
     addChild(screen);
 
@@ -1549,7 +1709,7 @@ AppearanceSettings::AppearanceSettings()
     addChild(dates);
 
     VerticalConfigurationGroup* qttheme = new VerticalConfigurationGroup(false);
-    qttheme->setLabel("QT");
+    qttheme->setLabel(QObject::tr("QT"));
     qttheme->addChild(new QtFontSmall());
     qttheme->addChild(new QtFontMedium());
     qttheme->addChild(new QtFontBig());
@@ -1562,7 +1722,7 @@ XboxSettings::XboxSettings()
 {
     VerticalConfigurationGroup* xboxset = new VerticalConfigurationGroup(false);
 
-    xboxset->setLabel("Xbox");
+    xboxset->setLabel(QObject::tr("Xbox"));
     xboxset->addChild(new XboxBlinkBIN());
     xboxset->addChild(new XboxLEDDefault());
     xboxset->addChild(new XboxLEDRecording());
