@@ -1,6 +1,7 @@
 #include <qapplication.h>
 #include <qsqldatabase.h>
 #include <qfile.h>
+#include <qfileinfo.h>
 #include <qmap.h>
 #include <unistd.h>
 #include <qdir.h>
@@ -12,6 +13,7 @@
 using namespace std;
 
 #include "tv.h"
+#include "proglist.h"
 #include "progfind.h"
 #include "manualbox.h"
 #include "manualschedule.h"
@@ -31,12 +33,13 @@ using namespace std;
 #include "remoteutil.h"
 #include "xbox.h"
 #include "dbcheck.h"
+#include "mythmediamonitor.h"
 
 #define QUIT     1
 #define HALT     2
 
 ThemedMenu *menu;
-MythContext *gContext;
+QTranslator *translator;
 XBox *xbox = NULL;
 
 void startGuide(void)
@@ -53,6 +56,72 @@ void startFinder(void)
     RunProgramFind();
 }
 
+void startSearchTitle(void)
+{
+  ProgLister searchTitle(plTitleSearch, "",
+                         QSqlDatabase::database(),
+                         gContext->GetMainWindow(), "proglist");
+
+    qApp->unlock();
+    searchTitle.exec();
+    qApp->lock();
+}
+
+void startSearchDesc(void)
+{
+  ProgLister searchDesc(plDescSearch, "",
+                        QSqlDatabase::database(),
+                        gContext->GetMainWindow(), "proglist");
+
+    qApp->unlock();
+    searchDesc.exec();
+    qApp->lock();
+}
+
+void startSearchChannel(void)
+{
+    ProgLister searchChannel(plChannel, "",
+                             QSqlDatabase::database(),
+                             gContext->GetMainWindow(), "proglist");
+
+    qApp->unlock();
+    searchChannel.exec();
+    qApp->lock();
+}
+
+void startSearchCategory(void)
+{
+  ProgLister searchCategory(plCategory, "",
+                            QSqlDatabase::database(),
+                            gContext->GetMainWindow(), "proglist");
+
+    qApp->unlock();
+    searchCategory.exec();
+    qApp->lock();
+}
+
+void startSearchMovie(void)
+{
+    ProgLister searchMovie(plMovies, "",
+                           QSqlDatabase::database(),
+                           gContext->GetMainWindow(), "proglist");
+
+    qApp->unlock();
+    searchMovie.exec();
+    qApp->lock();
+}
+
+void startSearchNew(void)
+{
+    ProgLister searchNew(plNewListings, "",
+                         QSqlDatabase::database(),
+                         gContext->GetMainWindow(), "proglist");
+
+    qApp->unlock();
+    searchNew.exec();
+    qApp->lock();
+}
+
 void startManaged(void)
 {
     QSqlDatabase *db = QSqlDatabase::database();
@@ -126,7 +195,8 @@ void startTV(void)
     QSqlDatabase *db = QSqlDatabase::database();
     TV *tv = new TV(db);
 
-    QDateTime timeout;
+    QTime timer;
+    timer.start();
 
     tv->Init();
 
@@ -143,13 +213,13 @@ void startTV(void)
 
     while (!quitAll)
     {
-        timeout = QDateTime::currentDateTime().addSecs(2);
+        timer.restart();
         while (tryRecorder)
         { //try to play recording, if filenotfound retry until timeout
 
             if (tv->PlayFromRecorder(tv->GetLastRecorderNum()) == 1)
                 tryRecorder = false;
-            else if (QDateTime::currentDateTime().secsTo(timeout) <= 0)
+            else if (timer.elapsed() > 2000)
             {
                 tryRecorder = false;
                 tryTV = true;
@@ -163,13 +233,13 @@ void startTV(void)
              }
         }
 
-        timeout = QDateTime::currentDateTime().addSecs(2);
+        timer.restart();
         while (tryTV)
         {//try to start livetv
             bool showDialogs = false;
             if (tv->LiveTV(showDialogs) == 1) //1 == livetv ok
                 tryTV = false;
-            else if (QDateTime::currentDateTime().secsTo(timeout) <= 0)
+            else if (timer.restart() > 2000)
             {
                 tryTV = false;
                 quitAll = true;
@@ -225,43 +295,57 @@ void showStatus(void)
 
     mfdLastRunStart = gContext->GetSetting("mythfilldatabaseLastRunStart");
     mfdLastRunEnd = gContext->GetSetting("mythfilldatabaseLastRunEnd");
-    mfdLastRunStatus = gContext->GetNumSetting("mythfilldatabaseLastRunStatus");
+    mfdLastRunStatus = gContext->GetSetting("mythfilldatabaseLastRunStatus");
 
-    Status = "Last mythfilldatabase guide update:";
-    Status += "\n   Started:   ";
+    Status = QObject::tr("Myth version:") + " " + MYTH_BINARY_VERSION + "\n";
+
+    Status += QObject::tr("Last mythfilldatabase guide update:");
+    Status += "\n   ";
+    Status += QObject::tr("Started:   ");
     Status += mfdLastRunStart;
-    Status += "\n   Finished: ";
-    Status += mfdLastRunEnd;
+    if (mfdLastRunEnd > mfdLastRunStart)  //if end < start, it's still running.
+    {
+        Status += "\n   ";
+        Status += QObject::tr("Finished: ");
+        Status += mfdLastRunEnd;
+    }
 
-    Status += "\n   Result: ";
-
-    if(!mfdLastRunStatus)
-       Status += "FAILED";
-    else
-       Status += "Successful";
-
-    Status += "\n\nThere is guide data until ";
-    Status += QDateTime(GuideDataThrough).toString("yyyy-MM-dd hh:mm");
+    Status += "\n   ";
+    Status += QObject::tr("Result: ");
+    Status += mfdLastRunStatus;
 
     DaysOfData = qdtNow.daysTo(GuideDataThrough);
 
-    Status += "\nThere ";
+    if (GuideDataThrough.isNull())
+    {
+        Status += QObject::tr("There's no guide data available! ");
+        Status += QObject::tr("Have you run mythfilldatabase?");
+        Status += "\n";
+    }
+    else
+    {
+        Status += "\n\n";
+        Status += QObject::tr("There is guide data until ");
+        Status += QDateTime(GuideDataThrough).toString("yyyy-MM-dd hh:mm");
 
-    if (DaysOfData == 1) 
-       Status += "is ";
-    else 
-       Status += "are ";
-
-    Status += QString::number(DaysOfData);
-    Status += " day";
-
-    if (DaysOfData != 1)
-       Status += "s";
-
-    Status += " of guide data remaining in the database.\n";
+        if (DaysOfData > 0)
+        {
+            Status += QString("\n(%1 ").arg(DaysOfData);
+            if (DaysOfData >1)
+                Status += QObject::tr("days");
+            else
+                Status += QObject::tr("day");
+            Status += ").";
+        }
+        else
+            Status += ".";
+    }
 
     if (DaysOfData <= 3)
-       Status += "\nWARNING: is mythfilldatabase running?";
+    {
+        Status += "\n";
+        Status += QObject::tr("WARNING: is mythfilldatabase running?");
+    }
 
     DialogBox *status_dialog = new DialogBox(gContext->GetMainWindow(), Status);
     status_dialog->AddButton(QObject::tr("OK"));
@@ -293,10 +377,29 @@ void TVMenuCallback(void *data, QString &selection)
         startProgramRecPriorities();
     else if (sel == "tv_progfind")
         startFinder();
+    else if (sel == "tv_search_title")
+        startSearchTitle();
+    else if (sel == "tv_search_desc")
+        startSearchDesc();
+    else if (sel == "tv_search_channel")
+        startSearchChannel();
+    else if (sel == "tv_search_category")
+        startSearchCategory();
+    else if (sel == "tv_search_movie")
+        startSearchMovie();
+    else if (sel == "tv_search_new")
+        startSearchNew();
     else if (sel == "settings appearance") 
     {
         AppearanceSettings settings;
         settings.exec(QSqlDatabase::database());
+
+        qApp->removeTranslator(translator);
+        translator->load(PREFIX + QString("/share/mythtv/i18n/mythfrontend_") +
+                         QString(gContext->GetSetting("Language").lower()) +
+                         QString(".qm"), ".");
+        qApp->installTranslator(translator);
+
         gContext->LoadQtConfig();
         gContext->GetMainWindow()->Init();
         gContext->UpdateImageCache();
@@ -351,6 +454,9 @@ void TVMenuCallback(void *data, QString &selection)
 
 int handleExit(void)
 {
+    if (gContext->GetNumSetting("NoPromptOnExit") == 1)
+        return QUIT;
+
     // first of all find out, if a backend runs on this host...
     bool backendOnLocalhost = false;
 
@@ -467,7 +573,8 @@ QString RandTheme(QString &themename, QSqlDatabase *db)
             themelist.append(theme->fileName());
     }
 
-    themename = themelist[rand() % themelist.size()];
+    if (themelist.size()) 
+        themename = themelist[rand() % themelist.size()];
 
     ThemeSelector Theme;
     Theme.setValue(themename);
@@ -476,10 +583,40 @@ QString RandTheme(QString &themename, QSqlDatabase *db)
     return themename;
 }
 
+int internal_play_media(const char *mrl) 
+{
+    int res = -1;
+    QSqlDatabase *db = QSqlDatabase::database();
+    TV *tv = new TV(db);
+
+    tv->Init();
+    ProgramInfo *pginfo = new ProgramInfo();
+    pginfo->endts = QDateTime::currentDateTime().addSecs(-180);
+    pginfo->pathname = mrl;
+
+    if (tv->Playback(pginfo)) 
+    {
+       while (tv->GetState() != kState_None) 
+       {
+           qApp->unlock();
+           qApp->processEvents();
+           usleep(50);
+           qApp->lock();
+       }
+       res = 0;
+    }
+
+    delete tv;
+    delete pginfo;
+
+    return res;
+}
+
 void InitJumpPoints(void)
 {
     REG_JUMP("Program Guide", "", "", startGuide);
     REG_JUMP("Program Finder", "", "", startFinder);
+    //REG_JUMP("Search Listings", "", "", startSearch);
     REG_JUMP("Manage Recordings / Fix Conflicts", "", "", startManaged);
     REG_JUMP("Program Recording Priorities", "", "", startProgramRecPriorities);
     REG_JUMP("Channel Recording Priorities", "", "", startChannelRecPriorities);
@@ -491,20 +628,29 @@ void InitJumpPoints(void)
     TV::InitKeys();
 }
 
+int internal_media_init() 
+{
+    REG_MEDIAPLAYER("Internal", "MythTV's native media player.", 
+                    internal_play_media);
+    return 0;
+}
+   
 int main(int argc, char **argv)
 {
     QString lcd_host;
     int lcd_port;
 
     QApplication a(argc, argv);
-    QTranslator translator( 0 );
+    translator = new QTranslator(0);
 
     QString logfile = "";
     QString verboseString = QString(" important general");
 
     QString pluginname = "";
 
-    QString binname = basename(a.argv()[0]);
+    QFileInfo finfo(a.argv()[0]);
+
+    QString binname = finfo.baseName();
 
     if (binname != "mythfrontend")
         pluginname = binname;
@@ -675,12 +821,15 @@ int main(int argc, char **argv)
 
     UpgradeTVDatabaseSchema();
 
+    VERBOSE(VB_ALL, QString("%1 version: %2 www.mythtv.org")
+                            .arg(binname).arg(MYTH_BINARY_VERSION));
+
     VERBOSE(VB_ALL, QString("Enabled verbose msgs :%1").arg(verboseString));
 
-    translator.load(PREFIX + QString("/share/mythtv/i18n/mythfrontend_") + 
-                    QString(gContext->GetSetting("Language").lower()) + 
-                    QString(".qm"), ".");
-    a.installTranslator(&translator);
+    translator->load(PREFIX + QString("/share/mythtv/i18n/mythfrontend_") + 
+                     QString(gContext->GetSetting("Language").lower()) + 
+                     QString(".qm"), ".");
+    a.installTranslator(translator);
 
     WriteDefaults(db);
 
@@ -706,7 +855,10 @@ int main(int argc, char **argv)
 
     InitJumpPoints();
 
-    MythPluginManager::init();
+    internal_media_init();
+
+    MythPluginManager *pmanager = new MythPluginManager();
+    gContext->SetPluginManager(pmanager);
 
     gContext->UpdateImageCache();
 
@@ -734,12 +886,12 @@ int main(int argc, char **argv)
 
     if (pluginname != "")
     {
-        if (MythPluginManager::run_plugin(pluginname))
+        if (pmanager->run_plugin(pluginname))
             return 0;
         else
         {
             pluginname = "myth" + pluginname;
-            if (MythPluginManager::run_plugin(pluginname))
+            if (pmanager->run_plugin(pluginname))
                 return 0;
         }
     }            
@@ -747,11 +899,31 @@ int main(int argc, char **argv)
     qApp->lock();
     qApp->unlock();
 
+#ifndef _WIN32
+    MediaMonitor *mon = NULL;
+
+    if (gContext->GetNumSetting("MonitorDrives") == 1)
+    {
+        mon = new MediaMonitor(NULL, 500, true);
+        mon->addFSTab();
+        VERBOSE(VB_ALL, QString("Starting media monitor."));
+        mon->startMonitoring();
+    }
+#endif
+
     int exitstatus = RunMenu(themedir);
 
     if (exitstatus == HALT)
         haltnow();
 
+#ifndef _WIN32
+    if (mon)
+    {
+        mon->stopMonitoring();
+        delete mon;
+    }
+#endif
+
     delete gContext;
     return exitstatus;
 }
