@@ -64,7 +64,7 @@ ProgramRecPriorityInfo& ProgramRecPriorityInfo::operator=(const ProgramInfo &oth
     conflicting = other.conflicting;
     recording = other.recording;
     override = other.override;
-    norecord = other.norecord;
+    recstatus = other.recstatus;
     recordid = other.recordid;
     rectype = other.rectype;
     recdups = other.recdups;
@@ -84,14 +84,10 @@ ProgramRecPriority::ProgramRecPriority(QSqlDatabase *ldb, MythMainWindow *parent
 {
     db = ldb;
 
-    doingSel = false;
-
     curitem = NULL;
     bgTransBackup = NULL;
     pageDowner = false;
 
-    allowKeys = true;
-
     listCount = 0;
     dataCount = 0;
 
@@ -104,12 +100,12 @@ ProgramRecPriority::ProgramRecPriority(QSqlDatabase *ldb, MythMainWindow *parent
     theme->SetHMult(hmult);
     if (!theme->LoadTheme(xmldata, "recpriorityprograms"))
     {
-        DialogBox diag(gContext->GetMainWindow(), "The theme you are using "
+        DialogBox diag(gContext->GetMainWindow(), tr("The theme you are using "
                        "does not contain a 'recpriorityprograms' element.  "
                        "Please contact the theme creator and ask if they could "
                        "please update it.<br><br>The next screen will be empty."
-                       "  Escape out of it to return to the menu.");
-        diag.AddButton("OK");
+                       "  Escape out of it to return to the menu."));
+        diag.AddButton(tr("OK"));
         diag.exec();
 
         return;
@@ -140,7 +136,7 @@ ProgramRecPriority::ProgramRecPriority(QSqlDatabase *ldb, MythMainWindow *parent
 
     FillList();
     sortType = (SortType)gContext->GetNumSetting("ProgramRecPrioritySorting", 
-												 (int)byTitle);
+                                                 (int)byTitle);
 
     SortList(); 
     inList = inData = 0;
@@ -161,9 +157,6 @@ ProgramRecPriority::~ProgramRecPriority()
 
 void ProgramRecPriority::keyPressEvent(QKeyEvent *e)
 {
-    if (!allowKeys)
-        return;
-
     bool handled = false;
     QStringList actions;
     if (gContext->GetMainWindow()->TranslateKeyPress("TV Frontend", e, actions))
@@ -210,6 +203,15 @@ void ProgramRecPriority::keyPressEvent(QKeyEvent *e)
                     update(fullRect);
                 }
             }
+            else if (action == "PREVVIEW" || action == "NEXTVIEW")
+            {
+                if (sortType == byTitle)
+                    sortType = byRecPriority;
+                else
+                    sortType = byTitle;
+                SortList();
+                update(fullRect);
+            }
             else if (action == "SELECT" || action == "MENU" ||
                      action == "INFO")
             {
@@ -281,9 +283,6 @@ void ProgramRecPriority::updateBackground(void)
 
 void ProgramRecPriority::paintEvent(QPaintEvent *e)
 {
-    if (doingSel)
-        return;
-
     QRect r = e->rect();
     QPainter p(this);
  
@@ -404,14 +403,9 @@ void ProgramRecPriority::cursorUp(bool page)
 
 void ProgramRecPriority::edit(void)
 {
-    if (doingSel)
-        return;
-
     if (!curitem)
         return;
 
-    doingSel = true;
-
     ProgramRecPriorityInfo *rec = curitem;
 
     MythContext::KickDatabase(db);
@@ -496,8 +490,6 @@ void ProgramRecPriority::edit(void)
 
         update(fullRect);
     }
-
-    doingSel = false;
 }
 
 void ProgramRecPriority::changeRecPriority(int howMuch) 
@@ -550,9 +542,7 @@ void ProgramRecPriority::FillList(void)
 
     programData.clear();
 
-    allowKeys = false;
     RemoteGetAllScheduledRecordings(recordinglist);
-    allowKeys = true;
 
     vector<ProgramInfo *>::reverse_iterator pgiter = recordinglist.rbegin();
 
@@ -786,8 +776,6 @@ void ProgramRecPriority::updateList(QPainter *p)
                     {
                         ProgramRecPriorityInfo *progInfo = &(it.data());
 
-                        QString key = progInfo->MakeUniqueKey(); 
-
                         int progRecPriority = progInfo->recpriority.toInt();
                         int finalRecPriority = progRecPriority + 
                                         progInfo->channelRecPriority +
