@@ -36,9 +36,7 @@ class ViewScheduled : public MythDialog
     void FillList(void);
     void setShowAll(bool all);
 
-    int curRec;
-    int recCount;
-    QPtrList<ProgramInfo> recList;
+    void fillUpdateAll();
 
     void updateBackground(void);
     void updateList(QPainter *);
@@ -68,12 +66,9 @@ class ViewScheduled : public MythDialog
 
     int listsize;
 
-    bool allowEvents;
-    bool allowUpdates;
-    bool updateAll;
-    bool refillAll;
-
     bool showAll;
+
+    PGInfoCon recList;
 };
 
 #endif
