@@ -7,9 +7,9 @@
 #include <qstringlist.h>
 
 #include <unistd.h>
-#include <stdio.h>
+#include <cstdio>
 #include <fcntl.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 
@@ -17,6 +17,7 @@
 
 #include "libmyth/mythcontext.h"
 #include "libmythtv/videosource.h"
+#include "libmythtv/channeleditor.h"
 #include "libmyth/themedmenu.h"
 #include "backendsettings.h"
 
@@ -24,7 +25,6 @@
 
 using namespace std;
 
-MythContext* gContext;
 QSqlDatabase* db;
 
 QString getResponse(const QString &query, const QString &def)
@@ -83,6 +83,9 @@ void SetupMenuCallback(void* data, QString& selection) {
     } else if (sel == "card inputs") {
         CardInputEditor cie(db);
         cie.exec(db);
+    } else if (sel == "channel editor") {
+        ChannelEditor ce;
+        ce.exec(db);
     }
 }
 
@@ -91,7 +94,8 @@ void SetupMenu(void)
     QString theme = gContext->GetSetting("Theme", "blue");
 
     ThemedMenu* menu = new ThemedMenu(gContext->FindThemeDir(theme),
-                                      "setup.xml", gContext->GetMainWindow());
+                                      "setup.xml", gContext->GetMainWindow(),
+                                      false);
 
     menu->setCallback(SetupMenuCallback, gContext);
     menu->setKillable();
